@@ -13,22 +13,21 @@
 #ifndef FC__ADVMILITARY_H
 #define FC__ADVMILITARY_H
 
-struct city;
-struct player;
-struct unit;
+#include "fc_types.h"
+#include "unittype.h"
+
 struct ai_choice;
 
+Unit_Type_id ai_choose_defender_versus(struct city *pcity, Unit_Type_id v);
 void military_advisor_choose_tech(struct player *pplayer,
 				  struct ai_choice *choice);
 void  military_advisor_choose_build(struct player *pplayer, struct city *pcity,
 				    struct ai_choice *choice);
 void assess_danger_player(struct player *pplayer);
-int assess_danger(struct city *pcity);
-void establish_city_distances(struct player *pplayer, struct city *pcity);
 int assess_defense_quadratic(struct city *pcity);
-int assess_defense_unit(struct city *pcity, struct unit *punit, int igwall);
+int assess_defense_unit(struct city *pcity, struct unit *punit, bool igwall);
 int assess_defense(struct city *pcity);
-int unit_desirability(Unit_Type_id i, int def);
-int unit_attack_desirability(Unit_Type_id i);
+int ai_unit_defence_desirability(Unit_Type_id i);
+int ai_unit_attack_desirability(Unit_Type_id i);
 
 #endif  /* FC__ADVMILITARY_H */
