@@ -13,10 +13,13 @@
 #ifndef FC__ADVSPACE_H
 #define FC__ADVSPACE_H
 
-struct player;
+#include "shared.h"
+
+#include "fc_types.h"
+
 struct player_spaceship;
 
-int ai_spaceship_autoplace(struct player *pplayer,
-			   struct player_spaceship *ship);
+bool ai_spaceship_autoplace(struct player *pplayer,
+			    struct player_spaceship *ship);
 
 #endif  /* FC__ADVSPACE_H */
