@@ -10,66 +10,62 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#ifdef WIN32_NATIVE
+#include <windows.h>	/* LoadLibrary() */
+#endif
+
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
 #include <time.h>
-#include <assert.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
-#ifdef HAVE_WINSOCK
-#include <winsock.h>
-#endif
 
 #include "capstr.h"
+#include "dataio.h"
+#include "diptreaty.h"
+#include "fciconv.h"
 #include "fcintl.h"
 #include "game.h"
 #include "idex.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
+#include "netintf.h"
 #include "packets.h"
 #include "rand.h"
 #include "support.h"
 #include "version.h"
 
+#include "agents.h"
+#include "attribute.h"
+#include "audio.h"
 #include "chatline_g.h"
 #include "citydlg_g.h"
 #include "cityrepdata.h"
 #include "climisc.h"
 #include "clinet.h"
+#include "cma_core.h"		/* kludge */
 #include "connectdlg_g.h"
 #include "control.h" 
 #include "dialogs_g.h"
 #include "diplodlg_g.h"
-#include "gui_main_g.h"
 #include "goto.h"
+#include "gui_main_g.h"
 #include "helpdata.h"		/* boot_help_texts() */
 #include "mapctrl_g.h"
 #include "mapview_g.h"
 #include "menu_g.h"
 #include "messagewin_g.h"
-#include "netintf.h"
 #include "options.h"
 #include "packhand.h"
+#include "pages_g.h"
 #include "plrdlg_g.h"
 #include "repodlgs_g.h"
 #include "tilespec.h"
-#include "attribute.h"
-#include "agents.h"
-#include "audio.h"
-
-#include "cma_core.h" /* kludge */
 
 #include "civclient.h"
 
@@ -77,11 +73,14 @@
    needed (hence, it is not 'extern'd in civclient.h) */
 bool is_server = FALSE;
 
-char tile_set_name[512] = "\0";
+char *logfile = NULL;
+char *scriptfile = NULL;
+static char tileset_name[512] = "\0";
 char sound_plugin_name[512] = "\0";
 char sound_set_name[512] = "\0";
 char server_host[512] = "\0";
-char player_name[512] = "\0";
+char user_name[512] = "\0";
+char password[MAX_LEN_PASSWORD] = "\0";
 char metaserver[512] = "\0";
 int  server_port = -1;
 bool auto_connect = FALSE; /* TRUE = skip "Connect to Freeciv Server" dialog */
@@ -90,9 +89,6 @@ static enum client_states client_state = CLIENT_BOOT_STATE;
 
 int seconds_to_turndone;
 
-int last_turn_gold_amount;
-int turn_gold_difference;
-
 /* TRUE if an end turn request is blocked by busy agents */
 bool waiting_for_end_turn = FALSE;
 
@@ -102,20 +98,84 @@ bool waiting_for_end_turn = FALSE;
  */
 bool turn_done_sent = FALSE;
 
-static void client_remove_all_cli_conn(void);
+/**************************************************************************
+  Convert a text string from the internal to the data encoding, when it
+  is written to the network.
+**************************************************************************/
+static unsigned char *put_conv(const char *src, size_t *length)
+{
+  char *out = internal_to_data_string_malloc(src);
+
+  if (out) {
+    *length = strlen(out);
+    return out;
+  } else {
+    *length = 0;
+    return NULL;
+  }
+}
+
+/**************************************************************************
+  Convert a text string from the data to the internal encoding when it is
+  first read from the network.  Returns FALSE if the destination isn't
+  large enough or the source was bad.
+**************************************************************************/
+static bool get_conv(char *dst, size_t ndst,
+		     const unsigned char *src, size_t nsrc)
+{
+  char *out = data_to_internal_string_malloc(src);
+  bool ret = TRUE;
+  size_t len;
+
+  if (!out) {
+    dst[0] = '\0';
+    return FALSE;
+  }
+
+  len = strlen(out);
+  if (ndst > 0 && len >= ndst) {
+    ret = FALSE;
+    len = ndst - 1;
+  }
+
+  memcpy(dst, out, len);
+  dst[len] = '\0';
+  free(out);
+
+  return ret;
+}
+
+/**************************************************************************
+  Set up charsets for the client.
+**************************************************************************/
+static void charsets_init(void)
+{
+  dio_set_put_conv_callback(put_conv);
+  dio_set_get_conv_callback(get_conv);
+}
 
 /**************************************************************************
 ...
 **************************************************************************/
 int main(int argc, char *argv[])
 {
-  int i;
-  int loglevel;
-  char *logfile=NULL;
+  int i, loglevel;
+  int ui_options = 0;
+  bool ui_separator = FALSE;
   char *option=NULL;
 
+  /* Load win32 post-crash debugger */
+#ifdef WIN32_NATIVE
+# ifndef NDEBUG
+  if (LoadLibrary("exchndl.dll") == NULL) {
+#  ifdef DEBUG
+    fprintf(stderr, "exchndl.dll could not be loaded, no crash debugger\n");
+#  endif
+  }
+# endif
+#endif
+
   init_nls();
-  dont_run_as_root(argv[0], "freeciv_client");
   audio_init();
 
   /* default argument values are set in options.c */
@@ -124,37 +184,53 @@ int main(int argc, char *argv[])
   i = 1;
 
   while (i < argc) {
-   if (is_option("--help", argv[i])) {
-    fprintf(stderr, _("Usage: %s [option ...]\n"
+   if (ui_separator) {
+     argv[1 + ui_options] = argv[i];
+     ui_options++;
+   } else if (is_option("--help", argv[i])) {
+    fc_fprintf(stderr, _("Usage: %s [option ...]\n"
 		      "Valid options are:\n"), argv[0]);
-    fprintf(stderr, _("  -a, --autoconnect\tSkip connect dialog\n"));
+    fc_fprintf(stderr, _("  -a, --autoconnect\tSkip connect dialog\n"));
 #ifdef DEBUG
-    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 4,"
+    fc_fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 4,"
                                   " or 4:file1,min,max:...)\n"));
 #else
-    fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 3)\n"));
+    fc_fprintf(stderr,
+	       _("  -d, --debug NUM\tSet debug log level (0 to 3)\n"));
 #endif
-    fprintf(stderr, _("  -h, --help\t\tPrint a summary of the options\n"));
-    fprintf(stderr, _("  -l, --log FILE\tUse FILE as logfile\n"));
-    fprintf(stderr, _("  -m, --meta HOST\t"
+    fc_fprintf(stderr,
+	       _("  -h, --help\t\tPrint a summary of the options\n"));
+    fc_fprintf(stderr, _("  -l, --log FILE\tUse FILE as logfile "
+                      "(spawned server also uses this)\n"));
+    fc_fprintf(stderr, _("  -m, --meta HOST\t"
 		      "Connect to the metaserver at HOST\n"));
-    fprintf(stderr, _("  -n, --name NAME\tUse NAME as name\n"));
-    fprintf(stderr, _("  -p, --port PORT\tConnect to server port PORT\n"));
-    fprintf(stderr, _("  -P, --Plugin PLUGIN\tUse PLUGIN for sound output %s\n"),
+    fc_fprintf(stderr, _("  -n, --name NAME\tUse NAME as name\n"));
+    fc_fprintf(stderr,
+	       _("  -p, --port PORT\tConnect to server port PORT\n"));
+    fc_fprintf(stderr,
+	       _("  -P, --Plugin PLUGIN\tUse PLUGIN for sound output %s\n"),
 	    audio_get_all_plugin_names());
-    fprintf(stderr, _("  -s, --server HOST\tConnect to the server at HOST\n"));
-    fprintf(stderr, _("  -S, --Sound FILE\tRead sound tags from FILE\n"));
-    fprintf(stderr, _("  -t, --tiles FILE\t"
+    fc_fprintf(stderr, _("  -r, --read FILE\tRead startup script FILE "
+                      "(for spawned server only)\n"));
+    fc_fprintf(stderr,
+	       _("  -s, --server HOST\tConnect to the server at HOST\n"));
+    fc_fprintf(stderr, _("  -S, --Sound FILE\tRead sound tags from FILE\n"));
+    fc_fprintf(stderr, _("  -t, --tiles FILE\t"
 		      "Use data file FILE.tilespec for tiles\n"));
-    fprintf(stderr, _("  -v, --version\t\tPrint the version number\n"));
+    fc_fprintf(stderr, _("  -v, --version\t\tPrint the version number\n"));
+    fc_fprintf(stderr, _("      --\t\t"
+		      "Pass any following options to the UI.\n"
+		      "\t\t\tTry \"%s -- --help\" for more.\n"), argv[0]);
     exit(EXIT_SUCCESS);
    } else if (is_option("--version",argv[i])) {
-    fprintf(stderr, "%s %s\n", freeciv_name_version(), client_string);
+    fc_fprintf(stderr, "%s %s\n", freeciv_name_version(), client_string);
     exit(EXIT_SUCCESS);
-   } else if ((option = get_option("--log",argv,&i,argc)))
+   } else if ((option = get_option("--log",argv,&i,argc))) {
       logfile = mystrdup(option); /* never free()d */
+   } else  if ((option = get_option("--read", argv, &i, argc)))
+      scriptfile = mystrdup(option); /* never free()d */
    else if ((option = get_option("--name",argv,&i,argc)))
-      sz_strlcpy(player_name, option);
+      sz_strlcpy(user_name, option);
    else if ((option = get_option("--meta",argv,&i,argc)))
       sz_strlcpy(metaserver, option);
    else if ((option = get_option("--Sound", argv, &i, argc)))
@@ -163,6 +239,10 @@ int main(int argc, char *argv[])
       sz_strlcpy(sound_plugin_name, option);
    else if ((option = get_option("--port",argv,&i,argc))) {
      if(sscanf(option, "%d", &server_port) != 1) {
+       fc_fprintf(stderr,
+		  _("Invalid port \"%s\" specified with --port option.\n"),
+		  option);
+       fc_fprintf(stderr, _("Try using --help.\n"));
         exit(EXIT_FAILURE);
      }
    } else if ((option = get_option("--server",argv,&i,argc)))
@@ -172,42 +252,75 @@ int main(int argc, char *argv[])
    else if ((option = get_option("--debug",argv,&i,argc))) {
       loglevel=log_parse_level_str(option);
       if (loglevel==-1) {
+	fc_fprintf(stderr,
+		   _("Invalid debug level \"%s\" specified with --debug "
+		     "option.\n"), option);
+	fc_fprintf(stderr, _("Try using --help.\n"));
         exit(EXIT_FAILURE);
       }
    } else if ((option = get_option("--tiles", argv, &i, argc)))
-      sz_strlcpy(tile_set_name, option);
-   else { 
-      fprintf(stderr, _("Unrecognized option: \"%s\"\n"), argv[i]);
+      sz_strlcpy(tileset_name, option);
+   else if (is_option("--", argv[i])) {
+     ui_separator = TRUE;
+   } else { 
+      fc_fprintf(stderr, _("Unrecognized option: \"%s\"\n"), argv[i]);
       exit(EXIT_FAILURE);
    }
    i++;
   } /* of while */
 
+  /* Remove all options except those intended for the UI. */
+  argv[1 + ui_options] = NULL;
+  argc = 1 + ui_options;
+
+  /* disallow running as root -- too dangerous */
+  dont_run_as_root(argv[0], "freeciv_client");
+
   log_init(logfile, loglevel, NULL);
 
   /* after log_init: */
 
-  sz_strlcpy(default_player_name, user_username());
+  sz_strlcpy(default_user_name, user_username());
+  if (!is_valid_username(default_user_name)) {
+    char buf[sizeof(default_user_name)];
+
+    my_snprintf(buf, sizeof(buf), "_%s", default_user_name);
+    if (is_valid_username(buf)) {
+      sz_strlcpy(default_user_name, buf);
+    } else {
+      my_snprintf(default_user_name, sizeof(default_user_name),
+		  "player%d", myrand(10000));
+    }
+  }
 
   /* initialization */
 
+  conn_list_init(&game.all_connections);
+  conn_list_init(&game.est_connections);
+  conn_list_init(&game.game_connections);
+
   ui_init();
+  charsets_init();
   my_init_network();
+  chatline_common_init();
   init_messages_where();
   init_city_report_data();
+  init_player_dlg_common();
+  settable_options_init();
 
   load_general_options();
 
-  if (tile_set_name[0] == '\0') 
-    sz_strlcpy(tile_set_name, default_tile_set_name); 
+  if (tileset_name[0] == '\0') {
+    sz_strlcpy(tileset_name, default_tileset_name);
+  }
   if (sound_set_name[0] == '\0') 
     sz_strlcpy(sound_set_name, default_sound_set_name); 
   if (sound_plugin_name[0] == '\0')
     sz_strlcpy(sound_plugin_name, default_sound_plugin_name); 
   if (server_host[0] == '\0')
     sz_strlcpy(server_host, default_server_host); 
-  if (player_name[0] == '\0')
-    sz_strlcpy(player_name, default_player_name); 
+  if (user_name[0] == '\0')
+    sz_strlcpy(user_name, default_user_name); 
   if (metaserver[0] == '\0')
     sz_strlcpy(metaserver, default_metaserver); 
   if (server_port == -1) server_port = default_server_port;
@@ -218,17 +331,31 @@ int main(int argc, char *argv[])
   mysrand(time(NULL));
 
   boot_help_texts();
-  tilespec_read_toplevel(tile_set_name); /* get tile sizes etc */
+  if (!tilespec_read_toplevel(tileset_name)) {
+    /* get tile sizes etc */
+    exit(EXIT_FAILURE);
+  }
 
   audio_real_init(sound_set_name, sound_plugin_name);
   audio_play_music("music_start", NULL);
 
   /* run gui-specific client */
-
   ui_main(argc, argv);
 
   /* termination */
+  ui_exit();
+  
+  /* not reached */
+  return EXIT_SUCCESS;
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void ui_exit(void)
+{
   attribute_flush();
+  client_remove_all_cli_conn();
   my_shutdown_network();
 
   client_game_free();
@@ -242,229 +369,10 @@ int main(int argc, char *argv[])
 **************************************************************************/
 void handle_packet_input(void *packet, int type)
 {
-  switch(type) {
-  case PACKET_JOIN_GAME_REPLY:
-    handle_join_game_reply((struct packet_join_game_reply *)packet);
-    break;
-
-  case PACKET_SERVER_SHUTDOWN:
-    freelog(LOG_VERBOSE, "server shutdown");
-    break;
-
-  case PACKET_BEFORE_NEW_YEAR:
-    handle_before_new_year();
-    break;
-
-  case PACKET_NEW_YEAR:
-    handle_new_year((struct packet_new_year *)packet);
-    break;
-
-  case PACKET_UNIT_INFO:
-    handle_unit_info((struct packet_unit_info *)packet);
-    break;
-
-   case PACKET_MOVE_UNIT:
-    handle_move_unit();
-    break;
-    
-  case PACKET_TILE_INFO:
-    handle_tile_info((struct packet_tile_info *)packet);
-    break;
-
-  case PACKET_SELECT_NATION:
-    handle_select_nation((struct packet_nations_used *)packet);
-    break;
-
-  case PACKET_PLAYER_INFO:
-    handle_player_info((struct packet_player_info *)packet);
-    break;
-    
-  case PACKET_GAME_INFO:
-    handle_game_info((struct packet_game_info *)packet);
-    break;
-
-  case PACKET_MAP_INFO:
-    handle_map_info((struct packet_map_info *)packet);
-    break;
-    
-  case PACKET_CHAT_MSG:
-    handle_chat_msg((struct packet_generic_message *)packet);
-    break;
-
-  case PACKET_PAGE_MSG:
-    handle_page_msg((struct packet_generic_message *)packet);
-    break;
-    
-  case PACKET_CITY_INFO:
-    handle_city_info((struct packet_city_info *)packet);
-    break;
-
-  case PACKET_SHORT_CITY:
-    handle_short_city((struct packet_short_city *)packet);
-    break;
-
-  case PACKET_REMOVE_UNIT:
-    handle_remove_unit((struct packet_generic_integer *)packet);
-    break;
-
-  case PACKET_REMOVE_CITY:
-    handle_remove_city((struct packet_generic_integer *)packet);
-    break;
-    
-  case PACKET_UNIT_COMBAT:
-    handle_unit_combat((struct packet_unit_combat *)packet);
-    break;
-
-  case PACKET_GAME_STATE:
-    handle_game_state(((struct packet_generic_integer *)packet));
-    break;
-
-  case PACKET_NUKE_TILE:
-    handle_nuke_tile(((struct packet_nuke_tile *)packet));
-    break;
-
-  case PACKET_DIPLOMACY_INIT_MEETING:
-    handle_diplomacy_init_meeting((struct packet_diplomacy_info *)packet);  
-    break;
-
-  case PACKET_DIPLOMACY_CANCEL_MEETING:
-    handle_diplomacy_cancel_meeting((struct packet_diplomacy_info *)packet);  
-    break;
-
-  case PACKET_DIPLOMACY_CREATE_CLAUSE:
-    handle_diplomacy_create_clause((struct packet_diplomacy_info *)packet);  
-    break;
-
-  case PACKET_DIPLOMACY_REMOVE_CLAUSE:
-    handle_diplomacy_remove_clause((struct packet_diplomacy_info *)packet);  
-    break;
-
-  case PACKET_DIPLOMACY_ACCEPT_TREATY:
-    handle_diplomacy_accept_treaty((struct packet_diplomacy_info *)packet);  
-    break;
-
-  case PACKET_REMOVE_PLAYER:
-    handle_remove_player((struct packet_generic_integer *)packet);
-    break;
-
-  case PACKET_RULESET_CONTROL:
-    handle_ruleset_control((struct packet_ruleset_control *)packet);
-    break;
-
-  case PACKET_RULESET_UNIT:
-    handle_ruleset_unit((struct packet_ruleset_unit *)packet);
-    break;
-    
-  case PACKET_RULESET_TECH:
-    handle_ruleset_tech((struct packet_ruleset_tech *)packet);
-    break;
-    
-  case PACKET_RULESET_BUILDING:
-    handle_ruleset_building((struct packet_ruleset_building *)packet);
-    break;
-
-  case PACKET_RULESET_TERRAIN:
-    handle_ruleset_terrain((struct packet_ruleset_terrain *)packet);
-    break;
-
-  case PACKET_RULESET_TERRAIN_CONTROL:
-    handle_ruleset_terrain_control((struct terrain_misc *)packet);
-    break;
-
-  case PACKET_RULESET_GOVERNMENT:
-    handle_ruleset_government((struct packet_ruleset_government *)packet);
-    break;
-  
-  case PACKET_RULESET_GOVERNMENT_RULER_TITLE:
-    handle_ruleset_government_ruler_title((struct packet_ruleset_government_ruler_title *)packet);
-    break;
-  
-  case PACKET_RULESET_NATION:
-    handle_ruleset_nation((struct packet_ruleset_nation *)packet);
-    break;
-
-  case PACKET_RULESET_CITY:
-    handle_ruleset_city((struct packet_ruleset_city *)packet);
-    break;
-
-  case PACKET_RULESET_GAME:
-    handle_ruleset_game((struct packet_ruleset_game *)packet);
-    break;
-
-  case PACKET_INCITE_COST:
-    handle_incite_cost((struct packet_generic_values *)packet);
-    break;
-
-  case PACKET_CITY_OPTIONS:
-    handle_city_options((struct packet_generic_values *)packet);
-    break;
-    
-  case PACKET_SPACESHIP_INFO:
-    handle_spaceship_info((struct packet_spaceship_info *)packet);
-    break;
-    
-  case PACKET_CITY_NAME_SUGGESTION:
-    handle_city_name_suggestion((struct packet_city_name_suggestion *)packet);
-    break;
-
-  case PACKET_SABOTAGE_LIST:
-    handle_sabotage_list((struct packet_sabotage_list *)packet);
-    break;
-
-  case PACKET_DIPLOMAT_ACTION:
-    handle_diplomat_action((struct packet_diplomat_action *)packet);
-    break;
-
-  case PACKET_ADVANCE_FOCUS:
-    handle_advance_focus((struct packet_generic_integer *)packet);
-    break;
-    
-  case PACKET_CONN_INFO:
-    handle_conn_info((struct packet_conn_info *)packet);
-    break;
-    
-  case PACKET_CONN_PING:
-    send_packet_generic_empty(&aconnection, PACKET_CONN_PONG);
-    break;
-
-  case PACKET_SELECT_NATION_OK:
-    handle_select_nation(NULL);
-    break;
-
-  case PACKET_ATTRIBUTE_CHUNK:
-    handle_player_attribute_chunk((struct packet_attribute_chunk *)
- 				  packet);
-    break;
-
-  case PACKET_PROCESSING_STARTED:
-    handle_processing_started();
-    break;
-
-  case PACKET_PROCESSING_FINISHED:
-    handle_processing_finished();
-    break;
-
-  case PACKET_START_TURN:
-    handle_start_turn();
-    break;
-
-  case PACKET_FREEZE_HINT:
-    handle_freeze_hint();
-    break;
-
-  case PACKET_THAW_HINT:
-    handle_thaw_hint();
-    break;
-
-  default:
-    freelog(LOG_ERROR, "Received unknown packet (type %d) from server!", type);
-    /* Old clients (<= some 1.11.5-devel, capstr +1.11) used to exit()
-     * here, so server should not rely on client surviving.
-     */
-    break;
+  if (!client_handle_packet(type, packet)) {
+    freelog(LOG_ERROR, "Received unknown packet (type %d) from server!",
+	    type);
   }
-
-  free(packet);
 }
 
 /**************************************************************************
@@ -480,8 +388,6 @@ void user_ended_turn(void)
 **************************************************************************/
 void send_turn_done(void)
 {
-  struct packet_generic_message gen_packet;
-
   freelog(LOG_DEBUG, "send_turn_done() turn_done_button_state=%d",
 	  get_turn_done_button_state());
 
@@ -503,9 +409,7 @@ void send_turn_done(void)
 
   attribute_flush();
 
-  gen_packet.message[0] = '\0';
-
-  send_packet_generic_message(&aconnection, PACKET_TURN_DONE, &gen_packet);
+  send_packet_player_turn_done(&aconnection);
 
   update_turn_done_button_state();
 }
@@ -513,52 +417,10 @@ void send_turn_done(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void send_unit_info(struct unit *punit)
+void send_goto_unit(struct unit *punit, struct tile *dest_tile)
 {
-  struct packet_unit_info info;
-
-  info.id=punit->id;
-  info.owner=punit->owner;
-  info.x=punit->x;
-  info.y=punit->y;
-  info.homecity=punit->homecity;
-  info.veteran=punit->veteran;
-  info.type=punit->type;
-  info.movesleft=punit->moves_left;
-  info.activity=punit->activity;
-  info.activity_target=punit->activity_target;
-  info.select_it = FALSE;
-  info.packet_use = UNIT_INFO_IDENTITY;
-
-  send_packet_unit_info(&aconnection, &info);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void send_move_unit(struct unit *punit)
-{
-  struct packet_move_unit move;
-
-  move.unid=punit->id;
-  move.x=punit->x;
-  move.y=punit->y;
-
-  send_packet_move_unit(&aconnection, &move);
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void send_goto_unit(struct unit *punit, int dest_x, int dest_y)
-{
-  struct packet_unit_request req;
-
-  req.unit_id = punit->id;
-  req.name[0] = '\0';
-  req.x = dest_x;
-  req.y = dest_y;
-  send_packet_unit_request(&aconnection, &req, PACKET_UNIT_GOTO_TILE);
+  dsend_packet_unit_goto(&aconnection, punit->id,
+			 dest_tile->x, dest_tile->y);
 }
 
 /**************************************************************************
@@ -566,10 +428,7 @@ void send_goto_unit(struct unit *punit, int dest_x, int dest_y)
 **************************************************************************/
 void send_report_request(enum report_type type)
 {
- struct packet_generic_integer pa;
-  
-  pa.value=type;
-  send_packet_generic_integer(&aconnection, PACKET_REPORT_REQUEST, &pa);
+  dsend_packet_report_req(&aconnection, type);
 }
 
 /**************************************************************************
@@ -577,10 +436,6 @@ void send_report_request(enum report_type type)
 **************************************************************************/
 void client_game_init()
 {
-  conn_list_init(&game.all_connections);
-  conn_list_init(&game.est_connections);
-  conn_list_init(&game.game_connections);
-
   game_init();
   attribute_init();
   agents_init();
@@ -591,7 +446,6 @@ void client_game_init()
 **************************************************************************/
 void client_game_free()
 {
-  client_remove_all_cli_conn();
   free_client_goto();
   free_help_texts();
   attribute_free();
@@ -608,19 +462,6 @@ void set_client_state(enum client_states newstate)
       && (newstate == CLIENT_PRE_GAME_STATE);
   enum client_states oldstate = client_state;
 
-  /*
-   * We are currently ignoring the CLIENT_GAME_OVER_STATE state
-   * because the client hasen't been changed to take care of it. So it
-   * breaks the show-whole-map-at-the-end-of-the-game. Nevertheless
-   * the server is so kind and sends the client this information. And
-   * in the future the client can/should take advantage of this
-   * information.
-   *
-   * FIXME: audit all client code to that it copes with
-   * CLIENT_GAME_OVER_STATE and implement specific
-   * CLIENT_GAME_OVER_STATE actions like history browsing. Then remove
-   * the kludge below.
-   */
   if (newstate == CLIENT_GAME_OVER_STATE) {
     /*
      * Extra kludge for end-game handling of the CMA.
@@ -630,10 +471,12 @@ void set_client_state(enum client_states newstate)
         cma_release_city(pcity);
       }
     } city_list_iterate_end;
-    newstate = CLIENT_GAME_RUNNING_STATE;
+    popdown_all_city_dialogs();
+    popdown_all_game_dialogs();
+    set_unit_focus(NULL);
   }
 
-  if(client_state!=newstate) {
+  if (client_state != newstate) {
 
     /* If changing from pre-game state to _either_ select race
        or running state, then we have finished getting ruleset data,
@@ -650,27 +493,38 @@ void set_client_state(enum client_states newstate)
       
     client_state=newstate;
 
-    if(client_state==CLIENT_GAME_RUNNING_STATE) {
+    if (client_state == CLIENT_GAME_RUNNING_STATE) {
       load_ruleset_specific_options();
-      create_event(-1, -1, E_GAME_START, _("Game started."));
+      create_event(NULL, E_GAME_START, _("Game started."));
+      precalc_tech_data();
       update_research(game.player_ptr);
       role_unit_precalcs();
       boot_help_texts();	/* reboot */
+      can_slide = FALSE;
       update_unit_focus();
+      can_slide = TRUE;
+      set_client_page(PAGE_GAME);
     }
-    else if(client_state==CLIENT_PRE_GAME_STATE) {
+    else if (client_state == CLIENT_PRE_GAME_STATE) {
       popdown_all_city_dialogs();
+      popdown_all_game_dialogs();
       close_all_diplomacy_dialogs();
-      set_unit_focus_no_center(NULL);
+      set_unit_focus(NULL);
       clear_notify_window();
       if (oldstate != CLIENT_BOOT_STATE) {
 	client_game_free();
       }
       client_game_init();
+      if (!aconnection.established) {
+	set_client_page(PAGE_MAIN);
+      } else {
+	set_client_page(PAGE_START);
+      }
     }
     update_menus();
   }
-  if (client_state == CLIENT_PRE_GAME_STATE) {
+  if (!aconnection.established && client_state == CLIENT_PRE_GAME_STATE) {
+    gui_server_connect();
     if (auto_connect) {
       if (connect_error) {
 	freelog(LOG_NORMAL,
@@ -678,13 +532,12 @@ void set_client_state(enum client_states newstate)
 	exit(EXIT_FAILURE);
       } else {
 	server_autoconnect();
-        auto_connect = 0;  /* don't try this again */
+	auto_connect = FALSE;	/* don't try this again */
       }
-    } else {
-      gui_server_connect();
-    }
+    } 
   }
   update_turn_done_button_state();
+  update_conn_list_dialog();
 }
 
 
@@ -707,6 +560,7 @@ void client_remove_cli_conn(struct connection *pconn)
   conn_list_unlink(&game.all_connections, pconn);
   conn_list_unlink(&game.est_connections, pconn);
   conn_list_unlink(&game.game_connections, pconn);
+  assert(pconn != &aconnection);
   free(pconn);
 }
 
@@ -714,7 +568,7 @@ void client_remove_cli_conn(struct connection *pconn)
   Remove (and free) all connections from all connection lists in client.
   Assumes game.all_connections is properly maintained with all connections.
 **************************************************************************/
-static void client_remove_all_cli_conn(void)
+void client_remove_all_cli_conn(void)
 {
   while (conn_list_size(&game.all_connections) > 0) {
     struct connection *pconn = conn_list_get(&game.all_connections, 0);
@@ -730,10 +584,7 @@ void dealloc_id(int id) { }/* kludge */
 **************************************************************************/
 void send_attribute_block_request()
 {
-  struct packet_player_request packet;
-
-  send_packet_player_request(&aconnection, &packet,
-			     PACKET_PLAYER_ATTRIBUTE_BLOCK);
+  send_packet_player_attribute_block(&aconnection);
 }
 
 /**************************************************************************
@@ -750,7 +601,7 @@ void wait_till_request_got_processed(int request_id)
 **************************************************************************/
 bool client_is_observer(void)
 {
-  return aconnection.established && find_conn_by_id(game.conn_id)->observer;
+  return aconnection.established && aconnection.observer;
 }
 
 /**************************************************************************
@@ -797,3 +648,44 @@ void real_timer_callback(void)
 
   flip = !flip;
 }
+
+/**************************************************************************
+  Returns TRUE if the client can issue orders (such as giving unit
+  commands).  This function should be called each time before allowing the
+  user to give an order.
+**************************************************************************/
+bool can_client_issue_orders(void)
+{
+  return (!client_is_observer()
+	  && get_client_state() == CLIENT_GAME_RUNNING_STATE);
+}
+
+/**************************************************************************
+  Returns TRUE iff the client can do diplomatic meetings with another 
+  given player.
+**************************************************************************/
+bool can_meet_with_player(struct player *pplayer)
+{
+  return (could_meet_with_player(game.player_ptr, pplayer)
+          && can_client_issue_orders());
+}
+
+/**************************************************************************
+  Returns TRUE iff the client can get intelligence from another 
+  given player.
+**************************************************************************/
+bool can_intel_with_player(struct player *pplayer)
+{
+  return could_intel_with_player(game.player_ptr, pplayer);
+}
+
+/**************************************************************************
+  Return TRUE if the client can change the view; i.e. if the mapview is
+  active.  This function should be called each time before allowing the
+  user to do mapview actions.
+**************************************************************************/
+bool can_client_change_view(void)
+{
+  return (get_client_state() == CLIENT_GAME_RUNNING_STATE
+	  || get_client_state() == CLIENT_GAME_OVER_STATE);
+}
