@@ -15,31 +15,30 @@
 
 #include "map.h"
 
-struct client_goto_map {
-  short **move_cost;
-  char **vector;
-  unsigned char **drawn; /* Should not be modified directly. */
-  int unit_id; /* The unit of the goto map */
-  int src_x, src_y;
-};
-
-extern struct client_goto_map goto_map;
+#include "path_finding.h"
 
 void init_client_goto(void);
 void free_client_goto(void);
 void enter_goto_state(struct unit *punit);
 void exit_goto_state(void);
 bool goto_is_active(void);
-void get_line_dest(int *x, int *y);
+struct tile *get_line_dest(void);
+int get_goto_turns(void);
 void goto_add_waypoint(void);
 bool goto_pop_waypoint(void);
 
-void draw_line(int dest_x, int dest_y);
-int get_drawn(int x, int y, int dir);
-void increment_drawn(int x, int y, int dir);
-void decrement_drawn(int x, int y, int dir);
+void draw_line(struct tile *dest_tile);
+bool is_drawn_line(struct tile *dest_tile, int dir);
+ 
+bool is_endpoint(struct tile *ptile);
 
+void request_orders_cleared(struct unit *punit);
+void send_goto_path(struct unit *punit, struct pf_path *path,
+		    enum unit_activity final_activity);
 void send_patrol_route(struct unit *punit);
 void send_goto_route(struct unit *punit);
+void send_connect_route(struct unit *punit, enum unit_activity activity);
+
+struct pf_path *path_to_nearest_allied_city(struct unit *punit);
 
 #endif /* FC__GOTO_H */
