@@ -32,42 +32,55 @@
 #include "unit.h"
 
 #include "chatline.h"
-#include "citydlg.h"
+#include "citydlg_common.h"
 #include "cityrepdata.h"
+#include "civclient.h"
 #include "clinet.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
+#include "mapctrl.h"    /* is_city_hilited() */
 #include "mapview_common.h"
 #include "optiondlg.h"
 #include "options.h"
 #include "repodlgs.h"
 #include "climisc.h"
 
-#include "cityrep.h"
 #include "cma_fec.h"
 
+#include "citydlg.h"
+#include "cityrep.h"
+
 #define NEG_VAL(x)  ((x)<0 ? (x) : (-x))
 #define CMA_NONE	(-1)
 #define CMA_CUSTOM	(-2)
 
+enum city_operation_type {
+  CO_CHANGE, CO_LAST, CO_NEXT, CO_FIRST, CO_NONE
+};
+
 /******************************************************************/
 static void create_city_report_dialog(bool make_modal);
 static void city_model_init(void);
 
-static void city_center_callback(GtkWidget *w, gpointer data);
-static void city_popup_callback(GtkWidget *w, gpointer data);
 static void city_activated_callback(GtkTreeView *view, GtkTreePath *path,
 				    GtkTreeViewColumn *col, gpointer data);
 
-static void city_buy_callback(GtkWidget *w, gpointer data);
-static void city_refresh_callback(GtkWidget *w, gpointer data);
+static void city_command_callback(struct gui_dialog *dlg, int response);
+
 static void city_selection_changed_callback(GtkTreeSelection *selection);
 
 static void create_select_menu(GtkWidget *item);
 static void create_change_menu(GtkWidget *item);
+static void create_last_menu(GtkWidget *item);
+static void create_first_menu(GtkWidget *item);
+static void create_next_menu(GtkWidget *item);
 
-static GtkWidget *city_dialog_shell=NULL;
+static struct gui_dialog *city_dialog_shell = NULL;
+
+enum {
+  CITY_CENTER = 1, CITY_POPUP, CITY_BUY
+};
 
 static GtkWidget *city_view;
 static GtkTreeSelection *city_selection;
@@ -75,15 +88,33 @@ static GtkListStore *city_model;
 
 static void popup_select_menu(GtkMenuShell *menu, gpointer data);
 static void popup_change_menu(GtkMenuShell *menu, gpointer data);
+static void popup_last_menu(GtkMenuShell *menu, gpointer data);
+static void popup_first_menu(GtkMenuShell *menu, gpointer data);
+static void popup_next_menu(GtkMenuShell *menu, gpointer data);
 
 static GtkWidget *city_center_command, *city_popup_command, *city_buy_command;
 static GtkWidget *city_change_command;
+static GtkWidget *city_last_command, *city_first_command, *city_next_command;
+
 
 static GtkWidget *change_improvements_item;
 static GtkWidget *change_units_item;
 static GtkWidget *change_wonders_item;
 static GtkWidget *change_cma_item;
 
+static GtkWidget *last_improvements_item;
+static GtkWidget *last_units_item;
+static GtkWidget *last_wonders_item;
+
+static GtkWidget *first_improvements_item;
+static GtkWidget *first_units_item;
+static GtkWidget *first_wonders_item;
+
+static GtkWidget *next_improvements_item;
+static GtkWidget *next_units_item;
+static GtkWidget *next_wonders_item;
+
+
 static GtkWidget *select_island_item;
 
 static GtkWidget *select_bunit_item;
@@ -134,12 +165,31 @@ void popup_city_report_dialog(bool make_modal)
     city_dialog_shell_is_modal = make_modal;
     
     create_city_report_dialog(make_modal);
-    gtk_set_relative_position(toplevel, city_dialog_shell, 10, 10);
 
     select_menu_cached = FALSE;
   }
 
-  gtk_window_present(GTK_WINDOW(city_dialog_shell));
+  gui_dialog_present(city_dialog_shell);
+  hilite_cities_from_canvas();
+}
+
+/****************************************************************
+ Raises the city report dialog.
+****************************************************************/
+void raise_city_report_dialog(void)
+{
+  popup_city_report_dialog(FALSE);
+  gui_dialog_raise(city_dialog_shell);
+}
+
+/****************************************************************
+ Closes the city report dialog.
+****************************************************************/
+void popdown_city_report_dialog(void)
+{
+  if (city_dialog_shell) {
+    gui_dialog_destroy(city_dialog_shell);
+  }
 }
 
 
@@ -154,7 +204,8 @@ typedef bool (*TestCityFunc)(struct city *, gint);
 static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 					     bool append_units,
 					     bool append_wonders,
-					     bool change_prod,
+					     enum city_operation_type 
+					     city_operation, 
 					     TestCityFunc test_func,
 					     GCallback callback,
 					     int size)
@@ -177,7 +228,7 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
   menu = gtk_menu_new();
   gtk_menu_item_set_submenu(parent_item, menu);
 
-  if (change_prod) {
+  if (city_operation != CO_NONE) {
     GPtrArray *selected;
     ITree it;
     int num_selected = 0;
@@ -199,22 +250,22 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 
     data = (struct city **)g_ptr_array_free(selected, FALSE);
     cids_used = collect_cids1(cids, data, num_selected, append_units,
-        		      append_wonders, change_prod, test_func);
+        		      append_wonders, TRUE, test_func);
     g_free(data);
   } else {
     cids_used = collect_cids1(cids, NULL, 0, append_units,
-			      append_wonders, change_prod, test_func);
+			      append_wonders, FALSE, test_func);
   }
 
-  name_and_sort_items(cids, cids_used, items, change_prod, NULL);
+  name_and_sort_items(cids, cids_used, items, city_operation != CO_NONE, NULL);
 
   for (i = 0; i < 4; i++) {
     row[i] = buf[i];
   }
 
   g_object_set_data(G_OBJECT(menu), "freeciv_test_func", test_func);
-  g_object_set_data(G_OBJECT(menu), "freeciv_change_prod",
-		    GINT_TO_POINTER(change_prod));
+  g_object_set_data(G_OBJECT(menu), "freeciv_city_operation",
+		    GINT_TO_POINTER(city_operation));
 
   for (i = 0; i < 3; i++) {
     group[i] = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
@@ -273,14 +324,80 @@ static void append_impr_or_unit_to_menu_item(GtkMenuItem *parent_item,
 static void impr_or_unit_iterate(GtkTreeModel *model, GtkTreePath *path,
 				 GtkTreeIter *it, gpointer data)
 {
-  struct packet_city_request packet;
+  cid cid = GPOINTER_TO_INT(data);
   gint id;
 
-  packet = *(struct packet_city_request *)data;
   gtk_tree_model_get(model, it, 1, &id, -1);
 
-  packet.city_id = id;
-  send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
+  city_change_production(find_city_by_id(id), cid_is_unit(cid), cid_id(cid));
+}
+
+/****************************************************************
+ Called by select_impr_or_unit_callback for each city that 
+ is selected in the city list dialog to have a object appended
+ to the worklist.  Sends a packet adding the item to the 
+ end of the worklist.
+*****************************************************************/
+static void worklist_last_impr_or_unit_iterate(GtkTreeModel *model, 
+						 GtkTreePath *path,
+						 GtkTreeIter *it, 
+						 gpointer data)
+{
+  cid cid = GPOINTER_TO_INT(data);
+  gint id;
+  struct city *pcity;  
+
+  gtk_tree_model_get(model, it, 1, &id, -1);
+  pcity = find_city_by_id(id);
+
+  (void) city_queue_insert(pcity, -1, cid_is_unit(cid), cid_id(cid));
+  /* perhaps should warn the user if not successful? */
+}
+
+/****************************************************************
+ Called by select_impr_or_unit_callback for each city that 
+ is selected in the city list dialog to have a object inserted
+ first to the worklist.  Sends a packet adding the current
+ production to the first place after the current production of the
+ worklist.
+ Then changes the production to the requested item.
+*****************************************************************/
+static void worklist_first_impr_or_unit_iterate(GtkTreeModel *model, 
+						 GtkTreePath *path,
+						 GtkTreeIter *it, 
+						 gpointer data)
+{
+  cid cid = GPOINTER_TO_INT(data);
+  gint id;
+  struct city *pcity;  
+
+  gtk_tree_model_get(model, it, 1, &id, -1);
+  pcity = find_city_by_id(id);
+
+  (void) city_queue_insert(pcity, 0, cid_is_unit(cid), cid_id(cid));
+  /* perhaps should warn the user if not successful? */
+}
+
+/****************************************************************
+ Called by select_impr_or_unit_callback for each city that 
+ is selected in the city list dialog to have a object added next
+ to the worklist.  Sends a packet adding the item to the 
+ first place after the current production of the worklist.
+*****************************************************************/
+static void worklist_next_impr_or_unit_iterate(GtkTreeModel *model, 
+						 GtkTreePath *path,
+						 GtkTreeIter *it, 
+						 gpointer data)
+{
+  struct city *pcity;
+  gint id;
+  cid cid = GPOINTER_TO_INT(data);
+
+  gtk_tree_model_get(model, it, 1, &id, -1);
+  pcity = find_city_by_id(id);
+
+  (void) city_queue_insert(pcity, 1, cid_is_unit(cid), cid_id(cid));
+  /* perhaps should warn the user if not successful? */
 }
 
 /****************************************************************
@@ -291,11 +408,11 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
   cid cid = GPOINTER_TO_INT(data);
   GObject *parent = G_OBJECT(w->parent);
   TestCityFunc test_func = g_object_get_data(parent, "freeciv_test_func");
-  bool change_prod = 
-    GPOINTER_TO_INT(g_object_get_data(parent, "freeciv_change_prod"));
+  enum city_operation_type city_operation = 
+    GPOINTER_TO_INT(g_object_get_data(parent, "freeciv_city_operation"));  
 
-  /* If this is not the change production button */
-  if (!change_prod) {
+  /* if this is not a city operation: */
+  if (city_operation == CO_NONE) {
     ITree it;
     GtkTreeModel *model = GTK_TREE_MODEL(city_model);
 
@@ -312,16 +429,30 @@ static void select_impr_or_unit_callback(GtkWidget *w, gpointer data)
       }
     }
   } else {
-    bool is_unit = cid_is_unit(cid);
-    int id = cid_id(cid);
-    struct packet_city_request packet;
-
-    packet.build_id = id;
-    packet.is_build_id_unit_id = is_unit;
-
     connection_do_buffer(&aconnection);
-    gtk_tree_selection_selected_foreach(city_selection, impr_or_unit_iterate,
-	&packet);
+    switch (city_operation) {
+    case CO_LAST:
+      gtk_tree_selection_selected_foreach(city_selection,
+					  worklist_last_impr_or_unit_iterate,
+					  GINT_TO_POINTER(cid));
+      break;
+    case CO_CHANGE:
+      gtk_tree_selection_selected_foreach(city_selection,
+					  impr_or_unit_iterate, GINT_TO_POINTER(cid));
+      break;
+    case CO_FIRST:
+      gtk_tree_selection_selected_foreach(city_selection,
+					  worklist_first_impr_or_unit_iterate,
+					  GINT_TO_POINTER(cid));
+      break;
+    case CO_NEXT:
+      gtk_tree_selection_selected_foreach(city_selection,
+					  worklist_next_impr_or_unit_iterate,
+					  GINT_TO_POINTER(cid));
+      break;
+    default:
+      assert(FALSE); /* should never get here. */
+    }
     connection_do_unbuffer(&aconnection);
   }
 }
@@ -357,7 +488,7 @@ static void select_cma_callback(GtkWidget * w, gpointer data)
   GObject *parent = G_OBJECT(w->parent);
   bool change_cma =
       GPOINTER_TO_INT(g_object_get_data(parent, "freeciv_change_cma"));
-  struct cma_parameter parameter;
+  struct cm_parameter parameter;
 
   /* If this is not the change button but the select cities button. */
   if (!change_cma) {
@@ -382,8 +513,8 @@ static void select_cma_callback(GtkWidget * w, gpointer data)
       } else if (idx == CMA_NONE && !controlled) {
         select = TRUE;
       } else if (idx >= 0 && controlled &&
-        	 cma_are_parameter_equal(&parameter,
-        				 cmafec_preset_get_parameter(idx))) {
+        	 cm_are_parameter_equal(&parameter,
+        				cmafec_preset_get_parameter(idx))) {
         select = TRUE;
       }
 
@@ -409,13 +540,12 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
 {
   GtkWidget *menu;
   int i;
-  struct cma_parameter parameter;
+  struct cm_parameter parameter;
   GtkWidget *w;
 
   gtk_menu_item_remove_submenu(parent_item);
   menu = gtk_menu_new();
   gtk_menu_item_set_submenu(parent_item, menu);
-  gtk_widget_set_name(menu, "Freeciv");
 
   if (change_cma) {
     for (i = -1; i < cmafec_preset_num(); i++) {
@@ -471,8 +601,8 @@ static void append_cma_to_menu_item(GtkMenuItem *parent_item, bool change_cma)
       found = 0;
       city_list_iterate(game.player_ptr->cities, pcity) {
 	if (cma_is_city_under_agent(pcity, &parameter) &&
-	    cma_are_parameter_equal(&parameter,
-				    cmafec_preset_get_parameter(i))) {
+	    cm_are_parameter_equal(&parameter,
+				   cmafec_preset_get_parameter(i))) {
 	  found = 1;
 	  break;
 	}
@@ -547,11 +677,26 @@ static GtkWidget *create_city_report_menubar(void)
 
   menubar = gtk_menu_bar_new();
   
-  item = gtk_menu_item_new_with_mnemonic(_("_Change"));
+  item = gtk_menu_item_new_with_mnemonic(_("Chan_ge"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
   city_change_command = item;
   create_change_menu(item);
 
+  item = gtk_menu_item_new_with_mnemonic(_("Add _First"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  city_first_command = item;
+  create_first_menu(item);
+
+  item = gtk_menu_item_new_with_mnemonic(_("Add _Next"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  city_next_command = item;
+  create_next_menu(item);
+
+  item = gtk_menu_item_new_with_mnemonic(_("Add _Last"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+  city_last_command = item;
+  create_last_menu(item);
+
   item = gtk_menu_item_new_with_mnemonic(_("_Select"));
   gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
   create_select_menu(item);
@@ -618,7 +763,7 @@ static gint cityrep_sort_func(GtkTreeModel *model,
   my_snprintf(buf1, sizeof(buf1), "%*s",NEG_VAL(sp->width),(sp->func)(pcity1));
   my_snprintf(buf2, sizeof(buf2), "%*s",NEG_VAL(sp->width),(sp->func)(pcity2));
 
-  return strcmp(buf1, buf2);
+  return cityrepfield_compare(buf1, buf2);
 }
 
 /****************************************************************
@@ -633,63 +778,38 @@ static void create_city_report_dialog(bool make_modal)
   GtkWidget *w, *sw, *menubar;
   int i;
   
-  city_dialog_shell = gtk_dialog_new_with_buttons(_("Cities"),
-  	NULL,
-	0,
-	NULL);
-  if (dialogs_on_top) {
-    gtk_window_set_transient_for(GTK_WINDOW(city_dialog_shell),
-				 GTK_WINDOW(toplevel));
-  }
-  gtk_window_set_type_hint(GTK_WINDOW(city_dialog_shell),
-			   GDK_WINDOW_TYPE_HINT_NORMAL);
-  gtk_window_set_default_size(GTK_WINDOW(city_dialog_shell), -1, 420);
-  gtk_dialog_set_default_response(GTK_DIALOG(city_dialog_shell),
-				  GTK_RESPONSE_CLOSE);
+  gui_dialog_new(&city_dialog_shell, GTK_NOTEBOOK(top_notebook));
+  gui_dialog_set_title(city_dialog_shell, _("Cities"));
 
-  if (make_modal) {
-    gtk_window_set_transient_for(GTK_WINDOW(city_dialog_shell),
-				 GTK_WINDOW(toplevel));
-    gtk_window_set_modal(GTK_WINDOW(city_dialog_shell), TRUE);
-  }
+  gui_dialog_set_default_size(city_dialog_shell, -1, 420);
 
-  g_signal_connect(city_dialog_shell, "response",
-		   G_CALLBACK(gtk_widget_destroy), NULL);
-  g_signal_connect(city_dialog_shell, "destroy",
-		   G_CALLBACK(gtk_widget_destroyed), &city_dialog_shell);
+  gui_dialog_response_set_callback(city_dialog_shell,
+      city_command_callback);
 
   /* menubar */
   menubar = create_city_report_menubar();
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(city_dialog_shell)->vbox),
+  gtk_box_pack_start(GTK_BOX(city_dialog_shell->vbox),
 	menubar, FALSE, FALSE, 0);
 
   /* buttons */
-  w = gtk_stockbutton_new(GTK_STOCK_ZOOM_FIT, _("Cen_ter"));
-  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(city_dialog_shell)->action_area),
-	w, FALSE, TRUE, 0);
-  g_signal_connect(w, "clicked", G_CALLBACK(city_center_callback), NULL);
+  w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_FIT,
+      _("Cen_ter"), CITY_CENTER);
   city_center_command = w;
 
-  w = gtk_stockbutton_new(GTK_STOCK_ZOOM_IN, _("_Popup"));
-  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(city_dialog_shell)->action_area),
-	w, FALSE, TRUE, 0);
-  g_signal_connect(w, "clicked", G_CALLBACK(city_popup_callback), NULL);
+  w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_ZOOM_IN,
+      _("_Popup"), CITY_POPUP);
   city_popup_command = w;
 
-  w = gtk_stockbutton_new(GTK_STOCK_EXECUTE, _("_Buy"));
-  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(city_dialog_shell)->action_area),
-	w, FALSE, TRUE, 0);
-  g_signal_connect(w, "clicked", G_CALLBACK(city_buy_callback), NULL);
+  w = gui_dialog_add_stockbutton(city_dialog_shell, GTK_STOCK_EXECUTE,
+      _("_Buy"), CITY_BUY);
   city_buy_command = w;
 
-  w = gtk_stockbutton_new(GTK_STOCK_REFRESH, _("_Refresh"));
-  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(city_dialog_shell)->action_area),
-	w, FALSE, TRUE, 0);
-  g_signal_connect(w, "clicked", G_CALLBACK(city_refresh_callback), NULL);
-
-  gtk_dialog_add_button(GTK_DIALOG(city_dialog_shell),
+  gui_dialog_add_button(city_dialog_shell,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
+  gui_dialog_set_default_response(city_dialog_shell,
+				  GTK_RESPONSE_CLOSE);
+
   /* tree view */
   for (i=0; i<NUM_CREPORT_COLS; i++)
     titles[i] = buf[i];
@@ -700,6 +820,7 @@ static void create_city_report_dialog(bool make_modal)
 
   city_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(city_model));
   g_object_unref(city_model);
+  gtk_widget_set_name(city_view, "small font");
   g_signal_connect(city_view, "row_activated",
 		   G_CALLBACK(city_activated_callback), NULL);
   city_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(city_view));
@@ -729,15 +850,14 @@ static void create_city_report_dialog(bool make_modal)
   gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
 				      GTK_SHADOW_ETCHED_IN);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-                                 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_container_add(GTK_CONTAINER(sw), city_view);
 
-  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(city_dialog_shell)->vbox),
+  gtk_box_pack_start(GTK_BOX(city_dialog_shell->vbox),
 	sw, TRUE, TRUE, 0);
 
-  gtk_widget_show_all(GTK_DIALOG(city_dialog_shell)->vbox);
-  gtk_widget_show_all(GTK_DIALOG(city_dialog_shell)->action_area);
   city_model_init();
+  gui_dialog_show_all(city_dialog_shell);
 
   city_selection_changed_callback(city_selection);
 }
@@ -792,7 +912,7 @@ static void city_select_coastal_callback(GtkMenuItem *item, gpointer data)
     itree_get(&it, 0, &res, -1);
     pcity = res;
 
-    if (is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN)) {
+    if (is_ocean_near_tile(pcity->tile)) {
       itree_select(city_selection, &it);
     }
   }
@@ -817,8 +937,8 @@ static void same_island_iterate(GtkTreeModel *model, GtkTreePath *path,
     itree_get(&it, 0, &res, -1);
     pcity = res;
 
-    if (map_get_continent(pcity->x, pcity->y)
-    	    == map_get_continent(selectedcity->x, selectedcity->y)) {
+    if (map_get_continent(pcity->tile)
+	== map_get_continent(selectedcity->tile)) {
       itree_select(city_selection, &it);
     }
   }
@@ -860,80 +980,17 @@ static void city_select_building_callback(GtkMenuItem *item, gpointer data)
   }
 }
 
-/****************************************************************
-...
-*****************************************************************/
-static void refresh_iterate(GtkTreeModel *model, GtkTreePath *path,
-			    GtkTreeIter *it, gpointer data)
-{
-  struct packet_generic_integer packet;
-  gint id;
-
-  *(gboolean *)data = TRUE;
-  gtk_tree_model_get(model, it, 1, &id, -1);
-
-  packet.value = id;
-  send_packet_generic_integer(&aconnection, PACKET_CITY_REFRESH, &packet);
-}
-
-/****************************************************************
-...
-*****************************************************************/
-static void city_refresh_callback(GtkWidget *w, gpointer data)
-
-{ /* added by Syela - I find this very useful */
-  struct packet_generic_integer packet;
-  gboolean found;
-
-  found = FALSE;
-  gtk_tree_selection_selected_foreach(city_selection, refresh_iterate, &found);
-
-  if (!found) {
-    packet.value = 0;
-    send_packet_generic_integer(&aconnection, PACKET_CITY_REFRESH, &packet);
-  }
-}
-
 /****************************************************************
 ...
 *****************************************************************/
 static void buy_iterate(GtkTreeModel *model, GtkTreePath *path,
 			GtkTreeIter *it, gpointer data)
 {
-  struct city *pcity;
-  int value;
-  const char *name;
-  char buf[512];
   gpointer res;
 
   gtk_tree_model_get(model, it, 0, &res, -1);
-  pcity = res;
 
-  value = city_buy_cost(pcity);	 
-  if (pcity->is_building_unit)
-    name = get_unit_type(pcity->currently_building)->name;
-  else
-    name = get_impr_name_ex(pcity, pcity->currently_building);
-
-  if (game.player_ptr->economic.gold >= value) {
-      struct packet_city_request packet;
-
-      packet.city_id = pcity->id;
-      send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
-  } else {
-    my_snprintf(buf, sizeof(buf),
-        	_("Game: %s costs %d gold and you only have %d gold."),
-        	name,value, game.player_ptr->economic.gold);
-    append_output_window(buf);
-  }
-}
-
-/****************************************************************
-...
-*****************************************************************/
-static void city_buy_callback(GtkWidget *w, gpointer data)
-{
-  gtk_tree_selection_selected_foreach(city_selection, buy_iterate, NULL);
+  cityrep_buy(res);
 }
 
 /****************************************************************
@@ -947,15 +1004,7 @@ static void center_iterate(GtkTreeModel *model, GtkTreePath *path,
 
   gtk_tree_model_get(model, it, 0, &res, -1);
   pcity = res;
-  center_tile_mapcanvas(pcity->x, pcity->y);
-}
-
-/****************************************************************
-...
-*****************************************************************/
-static void city_center_callback(GtkWidget *w, gpointer data)
-{
-  gtk_tree_selection_selected_foreach(city_selection, center_iterate, NULL);
+  center_tile_mapcanvas(pcity->tile);
 }
 
 /****************************************************************
@@ -971,7 +1020,7 @@ static void popup_iterate(GtkTreeModel *model, GtkTreePath *path,
   pcity = res;
 
   if (center_when_popup_city) {
-    center_tile_mapcanvas(pcity->x, pcity->y);
+    center_tile_mapcanvas(pcity->tile);
   }
 
   popup_city_dialog(pcity, 0);
@@ -980,9 +1029,22 @@ static void popup_iterate(GtkTreeModel *model, GtkTreePath *path,
 /****************************************************************
 ...
 *****************************************************************/
-static void city_popup_callback(GtkWidget *w, gpointer data)
+static void city_command_callback(struct gui_dialog *dlg, int response)
 {
-  gtk_tree_selection_selected_foreach(city_selection, popup_iterate, NULL);
+  switch (response) {
+  case CITY_CENTER:
+    gtk_tree_selection_selected_foreach(city_selection, center_iterate, NULL);
+    break;
+  case CITY_POPUP:
+    gtk_tree_selection_selected_foreach(city_selection, popup_iterate, NULL);
+    break;
+  case CITY_BUY:
+    gtk_tree_selection_selected_foreach(city_selection, buy_iterate, NULL);
+    break;
+  default:
+    gui_dialog_destroy(dlg);
+    break;
+  }
 }
 
 /****************************************************************
@@ -1051,32 +1113,20 @@ void city_report_dialog_update(void)
 {
   if (city_dialog_shell && !is_report_dialogs_frozen()) {
     GtkTreeModel *model;
-    GtkTreeIter it, it2;
-    GtkTreePath *path;
-    GList *copy;
-    struct city *cursor;
-    bool found;
+    GtkTreeIter it;
+    GHashTable *copy;
 
     model = GTK_TREE_MODEL(city_model);
 
-    gtk_tree_view_get_cursor(GTK_TREE_VIEW(city_view), &path, NULL);
-    cursor = NULL;
-    if (path) {
-      if (gtk_tree_model_get_iter(model, &it, path)) {
-	gtk_tree_model_get(model, &it, 0, &cursor, -1);
-      }
-      gtk_tree_path_free(path);
-    }
-
     /* copy the selection. */
-    copy = NULL;
+    copy = g_hash_table_new(NULL, NULL);
     if (gtk_tree_model_get_iter_first(model, &it)) {
       do {
 	if (gtk_tree_selection_iter_is_selected(city_selection, &it)) {
 	  gpointer pcity;
 
 	  gtk_tree_model_get(model, &it, 0, &pcity, -1);
-	  copy = g_list_append(copy, pcity);
+	  g_hash_table_insert(copy, pcity, NULL);
 	}
       } while (gtk_tree_model_iter_next(model, &it));
     }
@@ -1084,29 +1134,17 @@ void city_report_dialog_update(void)
     /* update. */
     gtk_list_store_clear(city_model);
 
-    found = FALSE;
     city_list_iterate(game.player_ptr->cities, pcity) {
       gtk_list_store_append(city_model, &it);
       update_row(&it, pcity);
 
-      if (g_list_find(copy, pcity)) {
+      if (g_hash_table_remove(copy, pcity)) {
 	gtk_tree_selection_select_iter(city_selection, &it);
       }
-
-      if (pcity == cursor) {
-	it2 = it;
-	found = TRUE;
-      }
     } city_list_iterate_end;
 
-    if (found) {
-      path = gtk_tree_model_get_path(model, &it2);
-      gtk_tree_view_set_cursor(GTK_TREE_VIEW(city_view), path, NULL, FALSE);
-      gtk_tree_path_free(path);
-    }
-	
     /* free the selection. */
-    g_list_free(copy);
+    g_hash_table_destroy(copy);
 
     city_selection_changed_callback(city_selection);
 
@@ -1148,15 +1186,6 @@ void city_report_dialog_update_city(struct city *pcity)
   }
 }
 
-/****************************************************************
-...
-*****************************************************************/
-static void selected_iterate(GtkTreeModel *model, GtkTreePath *path,
-			     GtkTreeIter *it, gpointer data)
-{
-  ++(*(int *)data);
-}
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -1178,6 +1207,65 @@ static void create_change_menu(GtkWidget *item)
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), change_cma_item);
 }
 
+/****************************************************************
+Creates the last menu.
+*****************************************************************/
+static void create_last_menu(GtkWidget *item)
+{
+  GtkWidget *menu;
+
+  menu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+  g_signal_connect(menu, "show", G_CALLBACK(popup_last_menu), NULL);
+
+  last_units_item = gtk_menu_item_new_with_label(_("Units"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), last_units_item);
+  last_improvements_item = gtk_menu_item_new_with_label(_("Improvements"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), last_improvements_item);
+  last_wonders_item = gtk_menu_item_new_with_label(_("Wonders"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), last_wonders_item);
+}
+
+/****************************************************************
+Creates the first menu.
+*****************************************************************/
+static void create_first_menu(GtkWidget *item)
+{
+  GtkWidget *menu;
+
+  menu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+  g_signal_connect(menu, "show", G_CALLBACK(popup_first_menu), NULL);
+
+  first_units_item = gtk_menu_item_new_with_label(_("Units"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), first_units_item);
+  first_improvements_item = gtk_menu_item_new_with_label(_("Improvements"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), first_improvements_item);
+  first_wonders_item = gtk_menu_item_new_with_label(_("Wonders"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), first_wonders_item);
+}
+
+/****************************************************************
+Creates the next menu.
+*****************************************************************/
+static void create_next_menu(GtkWidget *item)
+{
+  GtkWidget *menu;
+
+  menu = gtk_menu_new();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+  g_signal_connect(menu, "show", G_CALLBACK(popup_next_menu), NULL);
+
+  next_units_item = gtk_menu_item_new_with_label(_("Units"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), next_units_item);
+  next_improvements_item = gtk_menu_item_new_with_label(_("Improvements"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), next_improvements_item);
+  next_wonders_item = gtk_menu_item_new_with_label(_("Wonders"));
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), next_wonders_item);
+}
+
+
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1185,24 +1273,94 @@ static void popup_change_menu(GtkMenuShell *menu, gpointer data)
 {
   int n;
 
-  n = 0;
-  gtk_tree_selection_selected_foreach(city_selection, selected_iterate, &n);
+  n = gtk_tree_selection_count_selected_rows(city_selection);
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(change_improvements_item),
-				  FALSE, FALSE, TRUE,
+				  FALSE, FALSE, CO_CHANGE,
 				  city_can_build_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(change_units_item),
-				  TRUE, FALSE, TRUE,
+				  TRUE, FALSE, CO_CHANGE,
 				  city_can_build_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(change_wonders_item),
-				  FALSE, TRUE, TRUE,
+				  FALSE, TRUE, CO_CHANGE,
 				  city_can_build_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), n);
   append_cma_to_menu_item(GTK_MENU_ITEM(change_cma_item), TRUE);
 }
 
+/****************************************************************
+pops up the last menu.
+*****************************************************************/
+static void popup_last_menu(GtkMenuShell *menu, gpointer data)
+{
+  int n;
+
+  n = gtk_tree_selection_count_selected_rows(city_selection);
+
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(last_improvements_item),
+				  FALSE, FALSE, CO_LAST,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(last_units_item),
+				  TRUE, FALSE, CO_LAST,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(last_wonders_item),
+				  FALSE, TRUE, CO_LAST,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+}
+
+/****************************************************************
+pops up the first menu.
+*****************************************************************/
+static void popup_first_menu(GtkMenuShell *menu, gpointer data)
+{
+  int n;
+
+  n = gtk_tree_selection_count_selected_rows(city_selection);
+
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(first_improvements_item),
+				  FALSE, FALSE, CO_FIRST,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(first_units_item),
+				  TRUE, FALSE, CO_FIRST,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(first_wonders_item),
+				  FALSE, TRUE, CO_FIRST,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+}
+
+/****************************************************************
+pops up the next menu.
+*****************************************************************/
+static void popup_next_menu(GtkMenuShell *menu, gpointer data)
+{
+  int n;
+
+  n = gtk_tree_selection_count_selected_rows(city_selection);
+
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(next_improvements_item),
+				  FALSE, FALSE, CO_NEXT,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(next_units_item),
+				  TRUE, FALSE, CO_NEXT,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+  append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(next_wonders_item),
+				  FALSE, TRUE, CO_NEXT,
+				  city_can_build_impr_or_unit,
+				  G_CALLBACK(select_impr_or_unit_callback), n);
+}
+
+
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -1340,50 +1498,49 @@ static void popup_select_menu(GtkMenuShell *menu, gpointer data)
   if (select_menu_cached)
     return;
 
-  n = 0;
-  gtk_tree_selection_selected_foreach(city_selection, selected_iterate, &n);
+  n = gtk_tree_selection_count_selected_rows(city_selection);
   gtk_widget_set_sensitive(select_island_item, (n > 0));
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_bunit_item),
-				  TRUE, FALSE, FALSE,
+				  TRUE, FALSE, CO_NONE,
 				  city_building_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_bimprovement_item),
-				  FALSE, FALSE, FALSE,
+				  FALSE, FALSE, CO_NONE,
 				  city_building_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_bwonder_item),
-				  FALSE, TRUE, FALSE,
+				  FALSE, TRUE, CO_NONE,
 				  city_building_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_supported_item),
-				  TRUE, FALSE, FALSE,
+				  TRUE, FALSE, CO_NONE,
 				  city_unit_supported,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_present_item),
-				  TRUE, FALSE, FALSE,
+				  TRUE, FALSE, CO_NONE,
 				  city_unit_present,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_built_improvements_item),
-				  FALSE, FALSE, FALSE,
-				  city_got_building,
+				  FALSE, FALSE, CO_NONE,
+				   city_building_present,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_built_wonders_item),
-				  FALSE, TRUE, FALSE,
-				  city_got_building,
+				  FALSE, TRUE, CO_NONE,
+				   city_building_present,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
 
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_improvements_item),
-				  FALSE, FALSE, FALSE,
+				  FALSE, FALSE, CO_NONE,
 				  city_can_build_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_units_item),
-				  TRUE, FALSE, FALSE,
+				  TRUE, FALSE, CO_NONE,
 				  city_can_build_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_impr_or_unit_to_menu_item(GTK_MENU_ITEM(select_wonders_item),
-				  FALSE, TRUE, FALSE,
+				  FALSE, TRUE, CO_NONE,
 				  city_can_build_impr_or_unit,
 				  G_CALLBACK(select_impr_or_unit_callback), -1);
   append_cma_to_menu_item(GTK_MENU_ITEM(select_cma_item), FALSE);
@@ -1398,18 +1555,77 @@ static void city_selection_changed_callback(GtkTreeSelection *selection)
 {
   int n;
 
-  n = 0;
-  gtk_tree_selection_selected_foreach(selection, selected_iterate, &n);
+  n = gtk_tree_selection_count_selected_rows(selection);
 
   if (n == 0) {
     gtk_widget_set_sensitive(city_change_command, FALSE);
+    gtk_widget_set_sensitive(city_last_command, FALSE);
+    gtk_widget_set_sensitive(city_first_command, FALSE);
+    gtk_widget_set_sensitive(city_next_command, FALSE);
     gtk_widget_set_sensitive(city_center_command, FALSE);
     gtk_widget_set_sensitive(city_popup_command, FALSE);
     gtk_widget_set_sensitive(city_buy_command, FALSE);
   } else {
-    gtk_widget_set_sensitive(city_change_command, TRUE);
+    gtk_widget_set_sensitive(city_change_command, can_client_issue_orders());
+    gtk_widget_set_sensitive(city_last_command, can_client_issue_orders());
+    gtk_widget_set_sensitive(city_first_command, can_client_issue_orders());
+    gtk_widget_set_sensitive(city_next_command, can_client_issue_orders());
     gtk_widget_set_sensitive(city_center_command, TRUE);
     gtk_widget_set_sensitive(city_popup_command, TRUE);
-    gtk_widget_set_sensitive(city_buy_command, TRUE);
+    gtk_widget_set_sensitive(city_buy_command, can_client_issue_orders());
+  }
+}
+
+/****************************************************************
+ After a selection rectangle is defined, make the cities that
+ are hilited on the canvas exclusively hilited in the
+ City List window.
+*****************************************************************/
+void hilite_cities_from_canvas(void)
+{
+  ITree it;
+  GtkTreeModel *model;
+
+  if (!city_dialog_shell) return;
+
+  model = GTK_TREE_MODEL(city_model);
+
+  gtk_tree_selection_unselect_all(city_selection);
+
+  for (itree_begin(model, &it); !itree_end(&it); itree_next(&it)) {
+    struct city *pcity;
+    gpointer res;
+
+    itree_get(&it, 0, &res, -1);
+    pcity = res;
+
+    if (is_city_hilited(pcity))  {
+      itree_select(city_selection, &it);
+    }
+  }
+}
+
+/****************************************************************
+ Toggle a city's hilited status.
+*****************************************************************/
+void toggle_city_hilite(struct city *pcity, bool on_off)
+{
+  ITree it;
+  GtkTreeModel *model;
+
+  if (!city_dialog_shell) return;
+
+  model = GTK_TREE_MODEL(city_model);
+
+  for (itree_begin(model, &it); !itree_end(&it); itree_next(&it)) {
+    gint id;
+    itree_get(&it, 1, &id, -1);
+
+    if (id == pcity->id) {
+      on_off ?
+        itree_select(city_selection, &it):
+        itree_unselect(city_selection, &it);
+      break;
+    }
   }
 }
