@@ -10,39 +10,41 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 
 #include <gtk/gtk.h>
-#include "gtkpixcomm.h"
 
 #include "city.h"
 #include "fcintl.h"
 #include "game.h"
 #include "genlist.h"
 #include "government.h"
+#include "map.h"
 #include "mem.h"
 #include "shared.h"
+#include "support.h"
 #include "tech.h"
 #include "unit.h"
-#include "map.h"
-#include "support.h"
 #include "version.h"
 
 #include "climisc.h"
-#include "colors.h"
-#include "graphics.h"
-#include "gui_main.h"
-#include "gui_stuff.h"
 #include "helpdata.h"
 #include "tilespec.h"
 
+#include "colors.h"
+#include "graphics.h"
+#include "gtkpixcomm.h"
+#include "gui_main.h"
+#include "gui_stuff.h"
+
 #include "helpdlg.h"
 
 #define TECH_TREE_DEPTH         20
@@ -81,13 +83,13 @@ typedef struct help_tree_node {
   int turns_to_tech;
 } help_tndata;
 
-static char *help_ilabel_name[6] =
+static const char *help_ilabel_name[6] =
 { N_("Cost:"), NULL, N_("Upkeep:"), NULL, N_("Requirement:"), NULL };
 
-static char *help_wlabel_name[6] =
+static const char *help_wlabel_name[6] =
 { N_("Cost:"), NULL, N_("Requirement:"), NULL, N_("Obsolete by:"), NULL };
 
-static char *help_ulabel_name[5][5] =
+static const char *help_ulabel_name[5][5] =
 {
     { N_("Cost:"),		NULL, NULL, N_("Attack:"),	NULL },
     { N_("Defense:"),		NULL, NULL, N_("Move:")	,	NULL },
@@ -96,7 +98,7 @@ static char *help_ulabel_name[5][5] =
     { N_("Requirement:"),	NULL, NULL, N_("Obsolete by:"),	NULL }
 };
 
-static char *help_tlabel_name[4][5] =
+static const char *help_tlabel_name[4][5] =
 {
     { N_("Move/Defense:"),	NULL, NULL, N_("Food/Res/Trade:"),	NULL },
     { N_("Sp1 F/R/T:"),		NULL, NULL, N_("Sp2 F/R/T:"),		NULL },
@@ -104,6 +106,9 @@ static char *help_tlabel_name[4][5] =
     { N_("Mine Rslt/Time:"),	NULL, NULL, N_("Trans. Rslt/Time:"),	NULL }
 };
 
+/* HACK: we use a static string for convenience. */
+static char long_buffer[64000];
+
 
 static void create_help_dialog(void);
 static void help_update_dialog(const struct help_item *pitem);
@@ -390,7 +395,7 @@ static GtkWidget *help_hyperlink_new(GtkWidget *label, enum help_page_type type)
   return button;
 }
 
-static GtkWidget *help_slink_new(gchar *txt, enum help_page_type type)
+static GtkWidget *help_slink_new(const gchar *txt, enum help_page_type type)
 {
   GtkWidget *button;
 
@@ -640,13 +645,13 @@ static void create_help_page(enum help_page_type type)
 static void help_update_improvement(const struct help_item *pitem,
 				    char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
   
   create_help_page(HELP_IMPROVEMENT);
   
   if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d", imp->build_cost);
+    sprintf(buf, "%d", impr_build_shield_cost(which));
     gtk_set_label(help_ilabel[1], buf);
     sprintf(buf, "%d", imp->upkeep);
     gtk_set_label(help_ilabel[3], buf);
@@ -665,7 +670,7 @@ static void help_update_improvement(const struct help_item *pitem,
   }
   gtk_widget_show_all(help_itable);
 
-  helptext_improvement(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   
   gtk_text_freeze(GTK_TEXT(help_text));
   gtk_text_insert(GTK_TEXT(help_text), NULL, NULL, NULL, buf, -1);
@@ -680,20 +685,25 @@ static void help_update_improvement(const struct help_item *pitem,
 static void help_update_wonder(const struct help_item *pitem,
 			       char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
   
   create_help_page(HELP_WONDER);
 
   if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d", imp->build_cost);
+    sprintf(buf, "%d", impr_build_shield_cost(which));
     gtk_set_label(help_wlabel[1], buf);
     if (imp->tech_req == A_LAST) {
       gtk_set_label(help_wlabel[3], _("(Never)"));
     } else {
       gtk_set_label(help_wlabel[3], advances[imp->tech_req].name);
     }
-    gtk_set_label(help_wlabel[5], advances[imp->obsolete_by].name);
+    if (tech_exists(imp->obsolete_by)) {
+      gtk_set_label(help_wlabel[5], advances[imp->obsolete_by].name);
+    } else {
+      gtk_set_label(help_wlabel[5], _("(Never)"));
+    }
+
 /*    create_tech_tree(help_improvement_tree, 0, imp->tech_req, 3);*/
   }
   else {
@@ -705,7 +715,7 @@ static void help_update_wonder(const struct help_item *pitem,
   }
   gtk_widget_show_all(help_wtable);
 
-  helptext_wonder(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   gtk_text_freeze(GTK_TEXT(help_text));
   gtk_text_insert(GTK_TEXT(help_text), NULL, NULL, NULL, buf, -1);
   gtk_text_thaw(GTK_TEXT(help_text));
@@ -725,7 +735,7 @@ static void help_update_unit_type(const struct help_item *pitem,
 
   if (i<game.num_unit_types) {
     struct unit_type *utype = get_unit_type(i);
-    sprintf(buf, "%d", utype->build_cost);
+    sprintf(buf, "%d", unit_build_shield_cost(i));
     gtk_set_label(help_ulabel[0][1], buf);
     sprintf(buf, "%d", utype->attack_strength);
     gtk_set_label(help_ulabel[0][4], buf);
@@ -809,10 +819,6 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
     gtk_widget_show_all(help_tree_buttons_hbox);
 
     helptext_tech(buf, i, pitem->text);
-    if (advances[i].helptext) {
-      if (strlen(buf)) strcat(buf, "\n");
-      sprintf(buf+strlen(buf), "%s\n", _(advances[i].helptext));
-    }
     wordwrap_string(buf, 68);
 
     w = gtk_label_new(buf);
@@ -857,7 +863,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
       gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 0);
     } unit_type_iterate_end;
 
-    for(j=0; j<game.num_tech_types; ++j) {
+    for (j = 0; j < game.num_tech_types; j++) {
       if(i==advances[j].req[0]) {
 	if(advances[j].req[1]==A_NONE) {
           hbox = gtk_hbox_new(FALSE, 0);
@@ -913,113 +919,97 @@ static void help_update_terrain(const struct help_item *pitem,
 
   create_help_page(HELP_TERRAIN);
 
-  if (i < T_COUNT)
-    {
-      sprintf (buf, "%d/%d.%d",
-	       tile_types[i].movement_cost,
-	       (int)(tile_types[i].defense_bonus/10), tile_types[i].defense_bonus%10);
-      gtk_set_label (help_tlabel[0][1], buf);
+  if (i < T_COUNT) {
+    sprintf(buf, "%d/%d.%d",
+	    tile_types[i].movement_cost,
+	    (int)(tile_types[i].defense_bonus/10),
+	    tile_types[i].defense_bonus%10);
+    gtk_set_label(help_tlabel[0][1], buf);
 
-      sprintf (buf, "%d/%d/%d",
-	       tile_types[i].food,
-	       tile_types[i].shield,
-	       tile_types[i].trade);
-      gtk_set_label (help_tlabel[0][4], buf);
+    sprintf(buf, "%d/%d/%d",
+	    tile_types[i].food,
+	    tile_types[i].shield,
+	    tile_types[i].trade);
+    gtk_set_label(help_tlabel[0][4], buf);
 
-      if (*(tile_types[i].special_1_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_1_name);
-	  gtk_set_label (help_tlabel[1][0], buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_1,
-		   tile_types[i].shield_special_1,
-		   tile_types[i].trade_special_1);
-	  gtk_set_label (help_tlabel[1][1], buf);
-	} else {
-	  gtk_set_label (help_tlabel[1][0], "");
-	  gtk_set_label (help_tlabel[1][1], "");
-	}
-
-      if (*(tile_types[i].special_2_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_2_name);
-	  gtk_set_label (help_tlabel[1][3], buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_2,
-		   tile_types[i].shield_special_2,
-		   tile_types[i].trade_special_2);
-	  gtk_set_label (help_tlabel[1][4], buf);
-	} else {
-	  gtk_set_label (help_tlabel[1][3], "");
-	  gtk_set_label (help_tlabel[1][4], "");
-	}
-
-      if (tile_types[i].road_trade_incr > 0)
-	{
-	  sprintf (buf, _("+%d Trade / %d"),
-		   tile_types[i].road_trade_incr,
-		   tile_types[i].road_time);
-	}
-      else if (tile_types[i].road_time > 0)
-	{
-	  sprintf (buf, _("no extra / %d"),
-		   tile_types[i].road_time);
-	}
-      else
-	{
-	  strcpy (buf, _("n/a"));
-	}
-      gtk_set_label (help_tlabel[2][1], buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].irrigation_result == i)
-	{
-	  if (tile_types[i].irrigation_food_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Food / %d"),
-		       tile_types[i].irrigation_food_incr,
-		       tile_types[i].irrigation_time);
-	    }
-	}
-      else if (tile_types[i].irrigation_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].irrigation_result].terrain_name,
-		   tile_types[i].irrigation_time);
-	}
-      gtk_set_label (help_tlabel[2][4], buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].mining_result == i)
-	{
-	  if (tile_types[i].mining_shield_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Res. / %d"),
-		       tile_types[i].mining_shield_incr,
-		       tile_types[i].mining_time);
-	    }
-	}
-      else if (tile_types[i].mining_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].mining_result].terrain_name,
-		   tile_types[i].mining_time);
-	}
-      gtk_set_label (help_tlabel[3][1], buf);
-
-      if (tile_types[i].transform_result != T_LAST)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].transform_result].terrain_name,
-		   tile_types[i].transform_time);
-	} else {
-	  strcpy (buf, "n/a");
-	}
-      gtk_set_label (help_tlabel[3][4], buf);
+    if (*(tile_types[i].special_1_name)) {
+      sprintf(buf, _("%s F/R/T:"),
+	      tile_types[i].special_1_name);
+      gtk_set_label(help_tlabel[1][0], buf);
+      sprintf(buf, "%d/%d/%d",
+	      tile_types[i].food_special_1,
+	      tile_types[i].shield_special_1,
+	      tile_types[i].trade_special_1);
+      gtk_set_label(help_tlabel[1][1], buf);
+    } else {
+      gtk_set_label(help_tlabel[1][0], "");
+      gtk_set_label(help_tlabel[1][1], "");
     }
 
+    if (*(tile_types[i].special_2_name)) {
+      sprintf(buf, _("%s F/R/T:"),
+	      tile_types[i].special_2_name);
+      gtk_set_label(help_tlabel[1][3], buf);
+      sprintf(buf, "%d/%d/%d",
+	      tile_types[i].food_special_2,
+	      tile_types[i].shield_special_2,
+	      tile_types[i].trade_special_2);
+      gtk_set_label(help_tlabel[1][4], buf);
+    } else {
+      gtk_set_label(help_tlabel[1][3], "");
+      gtk_set_label(help_tlabel[1][4], "");
+    }
+
+    if (tile_types[i].road_trade_incr > 0) {
+      sprintf(buf, _("+%d Trade / %d"),
+	      tile_types[i].road_trade_incr,
+	      tile_types[i].road_time);
+    } else if (tile_types[i].road_time > 0) {
+      sprintf(buf, _("no extra / %d"),
+	      tile_types[i].road_time);
+    } else {
+      strcpy(buf, _("n/a"));
+    }
+    gtk_set_label(help_tlabel[2][1], buf);
+
+    strcpy(buf, _("n/a"));
+    if (tile_types[i].irrigation_result == i) {
+      if (tile_types[i].irrigation_food_incr > 0) {
+	sprintf(buf, _("+%d Food / %d"),
+		tile_types[i].irrigation_food_incr,
+		tile_types[i].irrigation_time);
+      }
+    } else if (tile_types[i].irrigation_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      tile_types[tile_types[i].irrigation_result].terrain_name,
+	      tile_types[i].irrigation_time);
+    }
+    gtk_set_label(help_tlabel[2][4], buf);
+
+    strcpy(buf, _("n/a"));
+    if (tile_types[i].mining_result == i) {
+      if (tile_types[i].mining_shield_incr > 0) {
+	sprintf(buf, _("+%d Res. / %d"),
+		tile_types[i].mining_shield_incr,
+		tile_types[i].mining_time);
+      }
+    } else if (tile_types[i].mining_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      tile_types[tile_types[i].mining_result].terrain_name,
+	      tile_types[i].mining_time);
+    }
+    gtk_set_label(help_tlabel[3][1], buf);
+
+    if (tile_types[i].transform_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      tile_types[tile_types[i].transform_result].terrain_name,
+	      tile_types[i].transform_time);
+    } else {
+      strcpy(buf, "n/a");
+    }
+    gtk_set_label(help_tlabel[3][4], buf);
+  }
+
   helptext_terrain(buf, i, pitem->text);
   
   gtk_text_freeze(GTK_TEXT(help_text));
@@ -1062,7 +1052,9 @@ static void help_update_dialog(const struct help_item *pitem)
 
   /* figure out what kind of item is required for pitem ingo */
 
-  for(top=pitem->topic; *top==' '; ++top);
+  for (top = pitem->topic; *top == ' '; top++) {
+    /* nothing */
+  }
 
   gtk_widget_hide_all(help_box);
   gtk_text_freeze(GTK_TEXT(help_text));
