@@ -15,6 +15,8 @@
 
 #include "repodlgs_g.h"
 
-/* nothing else */
+void popdown_science_dialog(void);
+void popdown_economy_report_dialog(void);
+void popdown_activeunits_report_dialog(void);
 
 #endif  /* FC__REPODLGS_H */
