@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -21,15 +22,19 @@
 
 #include "fcintl.h"
 #include "game.h"
-#include "genlist.h"
 #include "map.h"
 #include "mem.h"
 #include "packets.h"
 #include "player.h"
 #include "shared.h"
+#include "spaceship.h"
 #include "support.h"
 
+#include "climisc.h"
 #include "clinet.h"
+#include "text.h"
+#include "tilespec.h"
+
 #include "colors.h"
 #include "dialogs.h"
 #include "graphics.h"
@@ -40,9 +45,6 @@
 #include "mapctrl.h"
 #include "mapview.h"
 #include "repodlgs.h"
-#include "spaceship.h"
-#include "tilespec.h"
-#include "climisc.h"
 
 #include "spaceshipdlg.h"
 
@@ -57,9 +59,16 @@ struct spaceship_dialog {
   GtkWidget *close_command;
 };
 
+#define SPECLIST_TAG dialog
+#define SPECLIST_TYPE struct spaceship_dialog
+#include "speclist.h"
 
-static struct genlist dialog_list;
-static int dialog_list_has_been_initialised;
+#define dialog_list_iterate(dialoglist, pdialog) \
+    TYPED_LIST_ITERATE(struct spaceship_dialog, dialoglist, pdialog)
+#define dialog_list_iterate_end  LIST_ITERATE_END
+
+static struct dialog_list dialog_list;
+static bool dialog_list_has_been_initialised = FALSE;
 
 static struct spaceship_dialog *get_spaceship_dialog(struct player *pplayer);
 static struct spaceship_dialog *create_spaceship_dialog(struct player
@@ -77,18 +86,16 @@ static void spaceship_launch_callback(GtkWidget * w, gpointer data);
 *****************************************************************/
 struct spaceship_dialog *get_spaceship_dialog(struct player *pplayer)
 {
-  struct genlist_iterator myiter;
-
   if(!dialog_list_has_been_initialised) {
-    genlist_init(&dialog_list);
-    dialog_list_has_been_initialised=1;
+    dialog_list_init(&dialog_list);
+    dialog_list_has_been_initialised=TRUE;
   }
-  
-  genlist_iterator_init(&myiter, &dialog_list, 0);
-    
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter))
-    if(((struct spaceship_dialog *)ITERATOR_PTR(myiter))->pplayer==pplayer)
-      return ITERATOR_PTR(myiter);
+
+  dialog_list_iterate(dialog_list, pdialog) {
+    if (pdialog->pplayer == pplayer) {
+      return pdialog;
+    }
+  } dialog_list_iterate_end;
 
   return NULL;
 }
@@ -220,7 +227,7 @@ struct spaceship_dialog *create_spaceship_dialog(struct player *pplayer)
   gtk_signal_connect(GTK_OBJECT(pdialog->launch_command), "clicked",
         GTK_SIGNAL_FUNC(spaceship_launch_callback), pdialog);
 
-  genlist_insert(&dialog_list, pdialog, 0);
+  dialog_list_insert(&dialog_list, pdialog);
 
   gtk_widget_show_all(GTK_DIALOG(pdialog->shell)->vbox);
   gtk_widget_show_all(GTK_DIALOG(pdialog->shell)->action_area);
@@ -332,7 +339,7 @@ void spaceship_dialog_update_image(struct spaceship_dialog *pdialog)
 void close_spaceship_dialog(struct spaceship_dialog *pdialog)
 {
   gtk_widget_destroy(pdialog->shell);
-  genlist_unlink(&dialog_list, pdialog);
+  dialog_list_unlink(&dialog_list, pdialog);
 
   free(pdialog);
 }
@@ -350,16 +357,14 @@ static void spaceship_dialog_returnkey(GtkWidget *w, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-void close_spaceship_dialog_action(GtkWidget *w, gpointer data)
+void close_spaceship_dialog_action(GtkWidget * w, gpointer data)
 {
-  struct genlist_iterator myiter;
-
-  genlist_iterator_init(&myiter, &dialog_list, 0);
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter))
-    if(((struct spaceship_dialog *)ITERATOR_PTR(myiter))->shell==w) {
-      close_spaceship_dialog((struct spaceship_dialog *)ITERATOR_PTR(myiter));
+  dialog_list_iterate(dialog_list, pdialog) {
+    if (pdialog->shell == w) {
+      close_spaceship_dialog(pdialog);
       return;
     }
+  } dialog_list_iterate_end;
 }
 
 
@@ -377,10 +382,6 @@ void spaceship_close_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 void spaceship_launch_callback(GtkWidget *w, gpointer data)
 {
-  struct packet_spaceship_action packet;
-
-  packet.action = SSHIP_ACT_LAUNCH;
-  packet.num = 0;
-  send_packet_spaceship_action(&aconnection, &packet);
+  send_packet_spaceship_launch(&aconnection);
   /* close_spaceship_dialog((struct spaceship_dialog *)data); */
 }
