@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -17,7 +18,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
 
 #include <mui/NListview_MCC.h>
 #include <libraries/mui.h>
@@ -45,6 +45,7 @@
 #include "gui_main.h"
 #include "mapview.h"
 #include "options.h"
+#include "packhand.h"
 
 #include "muistuff.h"
 #include "mapclass.h"
@@ -55,8 +56,7 @@
 *****************************************************************/
 void request_player_revolution(void)
 {
-  struct packet_player_request packet;
-  send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_REVOLUTION);
+  start_revolution();
 }
 /****************************************************************
  ...
@@ -108,7 +108,8 @@ static void notify_close(Object **pwnd)
 /****************************************************************
  Popup the notify window
 *****************************************************************/
-void popup_notify_dialog(char *caption, char *headline, char *lines)
+void popup_notify_dialog(const char *caption, const char *headline,
+			 const char *lines)
 {
   Object *wnd;
   Object *listview;
@@ -165,7 +166,8 @@ static void notify_goto_close(struct notify_goto_close_arg *obj)
 /****************************************************************
 ...
 *****************************************************************/
-void popup_notify_goto_dialog(char *headline, char *lines,int x, int y)
+void popup_notify_goto_dialog(const char *headline, const char *lines,
+			      int x, int y)
 {
   Object *wnd;
   Object *close_button;
@@ -201,13 +203,8 @@ void popup_notify_goto_dialog(char *headline, char *lines,int x, int y)
 *****************************************************************/
 static void diplomat_bribe_yes(struct popup_message_data *data)
 {
-  struct packet_diplomat_action req;
-
-  req.action_type=DIPLOMAT_BRIBE;
-  req.diplomat_id=diplomat_id;
-  req.target_id=diplomat_target_id;
-
-  send_packet_diplomat_action(&aconnection, &req);
+  request_diplomat_action(DIPLOMAT_BRIBE, diplomat_id,
+			  diplomat_target_id, 0);
 
   message_close(data);
 }
@@ -277,17 +274,10 @@ static void advance_steal(struct spy_data *data)
   if(which)
   {
     which-=100;
-    if(find_unit_by_id(diplomat_id) && 
-       find_city_by_id(diplomat_target_id))
-    { 
-      struct packet_diplomat_action req;
-
-      req.action_type=DIPLOMAT_STEAL;
-      req.value=which;
-      req.diplomat_id=diplomat_id;
-      req.target_id=diplomat_target_id;
-			
-      send_packet_diplomat_action(&aconnection, &req);
+    if (find_unit_by_id(diplomat_id) && 
+	find_city_by_id(diplomat_target_id)) { 
+      request_diplomat_action(DIPLOMAT_STEAL, diplomat_id,
+			      diplomat_target_id, which);
     }
   }
 
@@ -383,14 +373,8 @@ static void imprv_sabotage(struct spy_data *data)
   {
     if(find_unit_by_id(diplomat_id) && find_city_by_id(diplomat_target_id))
     { 
-      struct packet_diplomat_action req;
-    
-      req.action_type=DIPLOMAT_SABOTAGE;
-      req.value=which-100+1;
-      req.diplomat_id=diplomat_id;
-      req.target_id=diplomat_target_id;
-
-      send_packet_diplomat_action(&aconnection, &req);
+      request_diplomat_action(DIPLOMAT_SABOTAGE, diplomat_id,
+			      diplomat_target_id, which - 100 + 1);
     }
   }
 
@@ -447,7 +431,7 @@ static void create_improvements_list(struct city *pcity)
 
     DoMethod(listview, MUIM_NList_InsertSingle, 100-1,MUIV_NList_Insert_Bottom);
     built_impr_iterate(pcity, i) {
-      if (i != B_PALACE && !is_wonder(i))
+      if (get_improvement_type(i)->sabotage > 0) {
       {
         DoMethod(listview, MUIM_NList_InsertSingle, i+100,MUIV_NList_Insert_Bottom);
         any_improvements = TRUE;
@@ -472,14 +456,9 @@ static void create_improvements_list(struct city *pcity)
 *****************************************************************/
 static void diplomat_incite_yes(struct popup_message_data *data)
 {
-  struct packet_diplomat_action req;
+  request_diplomat_action(DIPLOMAT_INCITE, diplomat_id,
+			  diplomat_target_id, 0);
 
-  req.action_type=DIPLOMAT_INCITE;
-  req.diplomat_id=diplomat_id;
-  req.target_id=diplomat_target_id;
-
-  send_packet_diplomat_action(&aconnection, &req);
-  
   message_close(data);
   process_diplomat_arrival(NULL, 0);
 }
@@ -535,13 +514,8 @@ static void diplomat_embassy(struct popup_message_data *data)
 
    if(find_unit_by_id(diplomat_id) && 
      (find_city_by_id(diplomat_target_id))) { 
-    struct packet_diplomat_action req;
-    
-    req.action_type=DIPLOMAT_EMBASSY;
-    req.diplomat_id=diplomat_id;
-    req.target_id=diplomat_target_id;
-    
-    send_packet_diplomat_action(&aconnection, &req);
+    request_diplomat_action(DIPLOMAT_EMBASSY, diplomat_id,
+			    diplomat_target_id, 0);
   }
   process_diplomat_arrival(NULL, 0);
 }
@@ -556,13 +530,8 @@ static void diplomat_investigate(struct popup_message_data *data)
 
   if(find_unit_by_id(diplomat_id) && 
      (find_city_by_id(diplomat_target_id))) { 
-    struct packet_diplomat_action req;
-
-    req.action_type=DIPLOMAT_INVESTIGATE;
-    req.diplomat_id=diplomat_id;
-    req.target_id=diplomat_target_id;
-
-    send_packet_diplomat_action(&aconnection, &req);
+    request_diplomat_action(DIPLOMAT_INVESTIGATE, diplomat_id,
+			    diplomat_target_id, 0);
   }
   process_diplomat_arrival(NULL, 0);
 }
@@ -577,14 +546,8 @@ static void diplomat_sabotage(struct popup_message_data *data)
 
   if(find_unit_by_id(diplomat_id) && 
      find_city_by_id(diplomat_target_id)) { 
-    struct packet_diplomat_action req;
-    
-    req.action_type=DIPLOMAT_SABOTAGE;
-    req.diplomat_id=diplomat_id;
-    req.target_id=diplomat_target_id;
-    req.value = -1;
-
-    send_packet_diplomat_action(&aconnection, &req);
+    request_diplomat_action(DIPLOMAT_SABOTAGE, diplomat_id,
+			    diplomat_target_id, -1);
   }
   process_diplomat_arrival(NULL, 0);
 }
@@ -599,14 +562,8 @@ static void diplomat_steal(struct popup_message_data *data)
 
   if(find_unit_by_id(diplomat_id) && 
      find_city_by_id(diplomat_target_id)) { 
-    struct packet_diplomat_action req;
-
-    req.action_type=DIPLOMAT_STEAL;
-    req.diplomat_id=diplomat_id;
-    req.target_id=diplomat_target_id;
-    req.value=0;
-
-    send_packet_diplomat_action(&aconnection, &req);
+    request_diplomat_action(DIPLOMAT_STEAL, diplomat_id,
+			    diplomat_target_id, 0);
   }
   process_diplomat_arrival(NULL, 0);
 }
@@ -660,12 +617,9 @@ static void diplomat_move(struct popup_message_data *data)
 
   if( (punit=find_unit_by_id(diplomat_id))
       && (pcity=find_city_by_id(diplomat_target_id))
-      && !same_pos(punit->x, punit->y, pcity->x, pcity->y)) {
-    struct packet_diplomat_action req;
-    req.action_type = DIPLOMAT_MOVE;
-    req.diplomat_id = diplomat_id;
-    req.target_id = diplomat_target_id;
-    send_packet_diplomat_action(&aconnection, &req);
+      && !same_pos(punit->tile, pcity->tile)) {
+    request_diplomat_action(DIPLOMAT_MOVE, diplomat_id,
+			    diplomat_target_id, 0);
   }
   process_diplomat_arrival(NULL, 0);
 }
@@ -691,13 +645,7 @@ static void spy_poison(struct popup_message_data *data)
 
   if(find_unit_by_id(diplomat_id) &&
     (find_city_by_id(diplomat_target_id))) {
-    struct packet_diplomat_action req;
-
-    req.action_type=SPY_POISON;
-    req.diplomat_id=diplomat_id;
-    req.target_id=diplomat_target_id;
-
-    send_packet_diplomat_action(&aconnection, &req);
+    request_diplomat_action(SPY_POISON, diplomat_id, diplomat_target_id, 0);
   }
   process_diplomat_arrival(NULL, 0);
 }
@@ -713,13 +661,8 @@ static void spy_request_sabotage_list(struct popup_message_data *data)
 
   if(find_unit_by_id(diplomat_id) &&
      (find_city_by_id(diplomat_target_id))) {
-    struct packet_diplomat_action req;
- 
-    req.action_type = SPY_GET_SABOTAGE_LIST;
-    req.diplomat_id = diplomat_id;
-    req.target_id = diplomat_target_id;
-
-    send_packet_diplomat_action(&aconnection, &req);
+    request_diplomat_action(SPY_GET_SABOTAGE_LIST, diplomat_id,
+			    diplomat_target_id, 0);
   }
 }
 
@@ -755,14 +698,9 @@ static void spy_steal(struct popup_message_data *data)
 *****************************************************************/
 static void spy_sabotage_unit(struct popup_message_data *data)
 {
-  struct packet_diplomat_action req;
-  
-  req.action_type=SPY_SABOTAGE_UNIT;
-  req.diplomat_id=diplomat_id;
-  req.target_id=diplomat_target_id;
-  
-  send_packet_diplomat_action(&aconnection, &req);
-  
+  request_diplomat_action(SPY_SABOTAGE_UNIT, diplomat_id,
+			  diplomat_target_id, 0);
+
   message_close(data);
 }
 
@@ -857,7 +795,7 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
 
       if(diplomat_can_do_action(punit, DIPLOMAT_INVESTIGATE, dest_x, dest_y))
       {
-      	msg_dlg[i].label = _("_Investigate city (free)");
+      	msg_dlg[i].label = _("_Investigate city");
       	msg_dlg[i].function = (APTR)diplomat_investigate;
       	msg_dlg[i].data = NULL;
       	i++;
@@ -1035,15 +973,19 @@ void popup_caravan_dialog(struct unit *punit, struct city *phomecity, struct cit
   my_snprintf(buf, sizeof(buf),_("Your caravan from %s reaches the city of %s.\nWhat now?"),
           phomecity->name, pdestcity->name);
 
-  if(can_establish_trade_route(phomecity, pdestcity))
+  if (can_cities_trade(phomecity, pdestcity))
   {
     struct caravan_data *cd = malloc_struct(struct caravan_data);
     if(cd)
     {
       cd->caravan_city_id = pdestcity->id;
       cd->caravan_unit_id = punit->id;
-
-      msg_dlg[i].label = _("_Establish traderoute");
+      if (can_establish_trade_route(phomecity, pdestcity))
+      {
+        msg_dlg[i].label = _("_Establish traderoute");
+      } else {
+	 msg_dlg[i].label = _("_Enter Marketplace");
+      }
       msg_dlg[i].function = (APTR)caravan_establish;
       msg_dlg[i].data = (APTR)cd;
       i++;
@@ -1083,65 +1025,6 @@ bool caravan_dialog_is_open(void)
   return caravan_dialog;
 }
 
-
-/****************************************************************
- Callback for a government button
-*****************************************************************/
-static void government_button(struct popup_message_data *msg)
-{
-  is_showing_government_dialog = 0;
-  request_player_government((int)msg->data);
-  message_close(msg);
-}
-
-
-/****************************************************************
- Fills the New_Msg_Dlg for the requested government
- TRUE, if player can choose the governemt, otherwise FALSE
-*****************************************************************/
-int fill_government_dialog(struct New_Msg_Dlg *dlg, int government)
-{
-  struct government *g = &governments[government];
-
-  if(can_change_to_government(game.player_ptr, government))
-  {
-    dlg->label = g->name;
-    dlg->function = (APTR)government_button;
-    dlg->data = (APTR)g->index;
-    return TRUE;
-  }
-  return FALSE;
-}
-
-/****************************************************************
-...
-*****************************************************************/
-void popup_government_dialog(void)
-{
-  struct New_Msg_Dlg *msg_dlg = (struct New_Msg_Dlg*)malloc(sizeof(struct New_Msg_Dlg)*(game.government_count+1));
-  if(msg_dlg)
-  {
-    if(!is_showing_government_dialog)
-    {
-      int i,j;
-      is_showing_government_dialog=1;
-      j=0;
-
-      for(i=0;i<game.government_count;i++)
-      {
-        if(i == game.government_when_anarchy) continue;
-        if(fill_government_dialog(&msg_dlg[j],i)) j++;
-      }
-
-      msg_dlg[j].label = NULL;
-
-      popup_message_dialog_args(main_wnd, _("Choose Your New Government"),
-                                _("Select government type:"),msg_dlg);
-    }
-    free(msg_dlg);
-  }
-}
-
 /****************************************************************
  Callback for the Yes button in the revolution confirmation
  window
@@ -1429,7 +1312,7 @@ static void unitsel_unit(struct unit **ppunit)
   struct unit *punit = *ppunit;
 
   if(punit && punit->owner == game.player_idx) {
-    request_unit_selected(punit);
+    set_unit_focus_and_select(punit);
   }
   set(unitsel_wnd, MUIA_Window_Open, FALSE);
 }
@@ -1452,7 +1335,6 @@ static void unitsel_ready_all(struct tile **pptile)
       {
         struct unit *punit=unit_list_get(&ptile->units, i);
         if(punit) {
-          request_new_unit_activity(punit, ACTIVITY_IDLE);
           set_unit_focus(punit);
         }
       }
@@ -1591,16 +1473,16 @@ Nation_Type_id get_active_nation(void)
 static void nations_nation_active(void)
 {
   int i, leader_count;
-  char **leaders;
+  struct leader *leaders;
   Object *list = (Object*)xget(nations_leader_poplist,MUIA_Popobject_Object);
   Nation_Type_id nation = get_active_nation();
 
   set(nations_flag_sprite, MUIA_Sprite_Sprite, get_nation_by_idx(nation)->flag_sprite);
 
-  leaders = get_nation_leader_names( nation, &leader_count);
-  setstring(nations_leader_string, leaders[0]);
+  leaders = get_nation_leaders(nation, &leader_count);
+  setstring(nations_leader_string, leaders[0].name);
 
-  set(nations_sex_radio, MUIA_Radio_Active, get_nation_leader_sex(nation,leaders[0])?0:1);
+  set(nations_sex_radio, MUIA_Radio_Active, leaders[0].is_male);
 
   if(list)
   {
@@ -1633,7 +1515,7 @@ static void nations_ok(void)
 
   sz_strlcpy(packet.name, (char*)s);
   
-  if(!get_sane_name(packet.name)) {
+  if (strlen(packet.name) == 0) {
     append_output_window(_("You must type a legal name."));
     return;
   }
@@ -1892,3 +1774,12 @@ void popup_upgrade_dialog(struct unit *punit)
     }
   }
 }
+
+/********************************************************************** 
+  This function is called when the client disconnects or the game is
+  over.  It should close all dialog windows for that game.
+***********************************************************************/
+void popdown_all_game_dialogs(void)
+{
+  /* TODO */
+}
