@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -77,7 +78,7 @@ static void find_ok(void)
     set(find_wnd, MUIA_Window_Open, FALSE);
 
     if ((pcity = game_find_city_by_name(string)))
-      center_tile_mapcanvas(pcity->x, pcity->y);
+      center_tile_mapcanvas(pcity->tile);
   }
 }
 
