@@ -15,10 +15,10 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <assert.h>
 
 #include <libraries/mui.h>
 #include <mui/NListview_MCC.h>
@@ -605,7 +605,7 @@ static void create_help_page(enum help_page_type type)
 static void help_update_improvement(const struct help_item *pitem,
 				    char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
 
   create_help_page(HELP_IMPROVEMENT);
 
@@ -613,12 +613,14 @@ static void help_update_improvement(const struct help_item *pitem,
   {
     struct impr_type *imp = &improvement_types[which];
 
-    DoMethod(help_imprv_cost_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld", imp->build_cost);
-    DoMethod(help_imprv_upkeep_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld", imp->upkeep);
+    DoMethod(help_imprv_cost_text, MUIM_SetAsString,
+	     MUIA_Text_Contents, "%ld", impr_build_shield_cost(which));
+    DoMethod(help_imprv_upkeep_text, MUIM_SetAsString,
+	     MUIA_Text_Contents, "%ld", imp->upkeep);
     UpdateTechButton(help_imprv_needs_button, imp->tech_req);
   }
 
-  helptext_improvement(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   DoMethod(help_text_listview, MUIM_NList_Insert, buf, -2, MUIV_List_Insert_Bottom);
 }
 
@@ -628,7 +630,7 @@ static void help_update_improvement(const struct help_item *pitem,
 static void help_update_wonder(const struct help_item *pitem,
 			       char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
 
   create_help_page(HELP_WONDER);
 
@@ -636,7 +638,8 @@ static void help_update_wonder(const struct help_item *pitem,
   {
     struct impr_type *imp = &improvement_types[which];
 
-    DoMethod(help_wonder_cost_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld", imp->build_cost);
+    DoMethod(help_wonder_cost_text, MUIM_SetAsString,
+	     MUIA_Text_Contents, "%ld", impr_build_shield_cost(which));
 
     UpdateTechButton(help_wonder_needs_button, imp->tech_req);
     UpdateTechButton(help_wonder_obsolete_button, imp->obsolete_by);
@@ -648,7 +651,7 @@ static void help_update_wonder(const struct help_item *pitem,
     set(help_wonder_obsolete_button, MUIA_Text_Contents, TECHTYPE_NONE);
   }
 
-  helptext_wonder(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   DoMethod(help_text_listview, MUIM_NList_Insert, buf, -2, MUIV_List_Insert_Bottom);
 }
 
@@ -668,14 +671,20 @@ static void help_update_unit_type(const struct help_item *pitem,
     struct unit_type *utype = get_unit_type(i);
     char *text;
 
-    settextf(help_unit_cost_text, _("Cost: %ld"), utype->build_cost);
-    DoMethod(help_unit_attack_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld", utype->attack_strength);
-    DoMethod(help_unit_defense_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld", utype->defense_strength);
-    DoMethod(help_unit_move_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld", utype->move_rate / 3);
-    DoMethod(help_unit_firepower_text, MUIM_SetAsString, MUIA_Text_Contents, "%ld", utype->firepower);
+    settextf(help_unit_cost_text,
+	     _("Cost: %ld"), unit_build_shield_cost(which));
+    DoMethod(help_unit_attack_text, MUIM_SetAsString,
+	     MUIA_Text_Contents, "%ld", utype->attack_strength);
+    DoMethod(help_unit_defense_text, MUIM_SetAsString,
+	     MUIA_Text_Contents, "%ld", utype->defense_strength);
+    DoMethod(help_unit_move_text, MUIM_SetAsString,
+	     MUIA_Text_Contents, "%ld", utype->move_rate / 3);
+    DoMethod(help_unit_firepower_text, MUIM_SetAsString,
+	     MUIA_Text_Contents, "%ld", utype->firepower);
     settextf(help_unit_hitpoints_text, "%ld", utype->hp);
     settextf(help_unit_vision_text, "%ld", utype->vision_range);
-    set(help_unit_basic_upkeep_text, MUIA_Text_Contents, helptext_unit_upkeep_str(i));
+    set(help_unit_basic_upkeep_text, MUIA_Text_Contents,
+	helptext_unit_upkeep_str(i));
 
     UpdateTechButton(help_unit_needs_button, utype->tech_requirement);
 
@@ -770,8 +779,7 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 	} unit_type_iterate_end;
 
 
-	for (j = 0; j < game.num_tech_types; ++j)
-	{
+	for (j = 0; j < game.num_tech_types; j++) {
 	  Object *o, *button;
 	  if (i == advances[j].req[0])
 	  {
@@ -962,7 +970,9 @@ static void help_update_dialog(const struct help_item *pitem)
 
   /* figure out what kind of item is required for pitem ingo */
 
-  for (top = pitem->topic; *top == ' '; ++top);
+  for (top = pitem->topic; *top == ' '; top++) {
+    /* nothing */
+  }
 
   switch (pitem->type)
   {
