@@ -10,28 +10,31 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/   
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
- 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
+
 #include <assert.h>          
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+
 #include <windows.h>
 #include <windowsx.h>
 #include <commctrl.h>
 
-
 #include "fcintl.h"
 #include "game.h"
 #include "government.h"
 #include "packets.h"
 #include "shared.h"
 #include "support.h"
- 
+
+#include "chatline_common.h"	/* send_chat() */ 
 #include "cityrep.h"
 #include "civclient.h"
+#include "climisc.h"
 #include "dialogs.h"
 #include "gui_stuff.h"
 #include "gui_main.h"
@@ -39,35 +42,25 @@
 #include "optiondlg.h"    
 #include "repodlgs_common.h"
 #include "repodlgs.h"
+#include "text.h"
 
 static HWND economy_dlg;
 static HWND activeunits_dlg;
 static HWND science_dlg;
+
+static HWND settable_options_dialog_win;
+static HWND tab_ctrl;
+static HWND *tab_wnds;
+static int num_tabs;
+static int *categories;
+
 extern HINSTANCE freecivhinst;
 extern HWND root_window;
 extern struct connection aconnection;               
 int economy_improvement_type[B_LAST];   
 int activeunits_type[U_LAST]; 
 
-/**************************************************************************
-
-**************************************************************************/
-char *
-get_report_title(char *report_name)
-{
- static char buf[512];
- 
-  my_snprintf(buf, sizeof(buf), _("%s\n%s of the %s\n%s %s: %s"),
-          report_name,
-          get_government_name(game.player_ptr->government),
-          get_nation_name_plural(game.player_ptr->nation),
-          get_ruler_title(game.player_ptr->government,
-                          game.player_ptr->is_male, game.player_ptr->nation),
-          game.player_ptr->name,
-          textyear(game.year));
- 
-  return buf;      
-}
+#define ID_OPTIONS_BASE 1000
 
 /**************************************************************************
 
@@ -89,22 +82,12 @@ void
 science_dialog_update(void)
 {
  
-  char text[512], rate[128];
-  int i, hist, id, turns_to_advance;
-  char *report_title;
+  char text[512];
+  int i, hist, id, steps;
 
   if (!science_dlg) return;            
-  report_title=get_report_title(_("Science"));
-  sz_strlcpy(text, report_title);
-  turns_to_advance = tech_turns_to_advance(game.player_ptr);
-  if (turns_to_advance == FC_INFINITY) {
-    my_snprintf(rate, sizeof(rate), _("\n(no research)"));
-  } else {
-    my_snprintf(rate, sizeof(rate),
-		PL_("\n(%d turn/advance)", "\n(%d turns/advance)",
-		    turns_to_advance), turns_to_advance);
-  }
-  sz_strlcat(text, rate);
+  sz_strlcpy(text, get_report_title(_("Science")));
+  sz_strlcat(text, science_dialog_text());
   SetWindowText(GetDlgItem(science_dlg,ID_SCIENCE_TOP),text);
   ListBox_ResetContent(GetDlgItem(science_dlg,ID_SCIENCE_LIST));
   for (i=A_FIRST;i<game.num_tech_types;i++)
@@ -122,35 +105,42 @@ science_dialog_update(void)
 	      total_bulbs_required(game.player_ptr));
   SetWindowText(GetDlgItem(science_dlg,ID_SCIENCE_PROG),text);
   ComboBox_ResetContent(GetDlgItem(science_dlg,ID_SCIENCE_RESEARCH));
-  if (game.player_ptr->research.researching!=A_NONE)
-    {
-      for (i=A_FIRST; i<game.num_tech_types; i++)
-	{
-	  if (get_invention(game.player_ptr,i)!=TECH_REACHABLE)
-	    continue;
-	  
-	  id=ComboBox_AddString(GetDlgItem(science_dlg,ID_SCIENCE_RESEARCH),
-				advances[i].name);
-	  ComboBox_SetItemData(GetDlgItem(science_dlg,ID_SCIENCE_RESEARCH),
-			       id,i);
-	  if (i==game.player_ptr->research.researching)
-	    {
-	      ComboBox_SetCurSel(GetDlgItem(science_dlg,
-					    ID_SCIENCE_RESEARCH)
-				 ,id);
-	    }
-	}
-    }
-  else
-    {
-      /* FIXME future tech */
+
+  if (game.player_ptr->research.researching == A_UNSET) {
+    id = ComboBox_AddString(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
+			    advances[A_NONE].name);
+    ComboBox_SetItemData(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
+			 id, A_NONE);
+    ComboBox_SetCurSel(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
+		       id);
+  }
+
+  if (!is_future_tech(game.player_ptr->research.researching)) {
+    for (i = A_FIRST; i < game.num_tech_types; i++) {
+      if (get_invention(game.player_ptr, i) != TECH_REACHABLE) {
+	continue;
+      }
+
+      id = ComboBox_AddString(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
+			      advances[i].name);
+      ComboBox_SetItemData(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
+			   id, i);
+      if (i == game.player_ptr->research.researching) {
+	ComboBox_SetCurSel(GetDlgItem(science_dlg, ID_SCIENCE_RESEARCH),
+			   id);
+      }
     }
+  } else {
+    /* FIXME future tech */
+  }
   ComboBox_ResetContent(GetDlgItem(science_dlg,ID_SCIENCE_GOAL));
     hist=0;
   for(i=A_FIRST; i<game.num_tech_types; i++) {
-    if(get_invention(game.player_ptr, i) != TECH_KNOWN &&
-       advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST &&
-       num_unknown_techs_for_goal(game.player_ptr, i) < 11) {
+    if (tech_is_available(game.player_ptr, i)
+        && get_invention(game.player_ptr, i) != TECH_KNOWN
+        && advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST
+        && (num_unknown_techs_for_goal(game.player_ptr, i) < 11
+	    || i == game.player_ptr->ai.tech_goal)) {
       id=ComboBox_AddString(GetDlgItem(science_dlg,ID_SCIENCE_GOAL),
 			    advances[i].name);
       ComboBox_SetItemData(GetDlgItem(science_dlg,ID_SCIENCE_GOAL),
@@ -160,10 +150,11 @@ science_dialog_update(void)
 			   id);
       
     }
-  }   
-  my_snprintf(text, sizeof(text), _("(%d steps)"),
-	      num_unknown_techs_for_goal(game.player_ptr,
-					 game.player_ptr->ai.tech_goal));
+  }
+  steps = num_unknown_techs_for_goal(game.player_ptr,
+                                     game.player_ptr->ai.tech_goal);
+  my_snprintf(text, sizeof(text),
+	      PL_("(%d step)", "(%d steps)", steps), steps);
   SetWindowText(GetDlgItem(science_dlg,ID_SCIENCE_STEPS),text);
   fcwin_redo_layout(science_dlg);
 }
@@ -176,7 +167,7 @@ static LONG CALLBACK science_proc(HWND hWnd,
 				  WPARAM wParam,
 				  LPARAM lParam)
 {
-  int to;
+  int to, steps;
   switch(message)
     {
     case WM_CREATE:
@@ -193,10 +184,10 @@ static LONG CALLBACK science_proc(HWND hWnd,
 	    to=ComboBox_GetCurSel(GetDlgItem(hWnd,ID_SCIENCE_RESEARCH));
 	    if (to!=LB_ERR) {
 	      char text[512];
-	      struct packet_player_request packet;
-	      to=ComboBox_GetItemData(GetDlgItem(hWnd,
-						 ID_SCIENCE_RESEARCH),
-				      to);
+
+	      to = ComboBox_GetItemData(GetDlgItem(hWnd,
+						   ID_SCIENCE_RESEARCH),
+					to);
 	      
 	      if (IsDlgButtonChecked(hWnd, ID_SCIENCE_HELP)) {
 		popup_help_dialog_typed(advances[to].name, HELP_TECH);
@@ -206,9 +197,7 @@ static LONG CALLBACK science_proc(HWND hWnd,
 			    game.player_ptr->research.bulbs_researched,
 			    total_bulbs_required(game.player_ptr));
 		SetWindowText(GetDlgItem(hWnd,ID_SCIENCE_PROG),text);
-		packet.tech=to;
-		send_packet_player_request(&aconnection, &packet,
-					   PACKET_PLAYER_RESEARCH);  
+		dsend_packet_player_research(&aconnection, to);
 	      }
 	    }
 	  }
@@ -218,14 +207,15 @@ static LONG CALLBACK science_proc(HWND hWnd,
 	    to=ComboBox_GetCurSel(GetDlgItem(hWnd,ID_SCIENCE_GOAL));
 	    if (to!=LB_ERR) {
 	      char text[512];
-	      struct packet_player_request packet;
-	      to=ComboBox_GetItemData(GetDlgItem(hWnd,ID_SCIENCE_GOAL),to);
-	      my_snprintf(text, sizeof(text), _("(%d steps)"),
-			  num_unknown_techs_for_goal(game.player_ptr, to));
-	      SetWindowText(GetDlgItem(hWnd,ID_SCIENCE_STEPS),text);       
-	      packet.tech=to;
-	      send_packet_player_request(&aconnection, &packet, 
-					 PACKET_PLAYER_TECH_GOAL); 
+
+	      to = ComboBox_GetItemData(GetDlgItem(hWnd, ID_SCIENCE_GOAL),
+					to);
+	      steps = num_unknown_techs_for_goal(game.player_ptr, to);
+	      my_snprintf(text, sizeof(text), 
+	                  PL_("(%d step)", "(%d steps)", steps),
+			  steps);
+	      SetWindowText(GetDlgItem(hWnd,ID_SCIENCE_STEPS), text);
+	      dsend_packet_player_tech_goal(&aconnection, to);
 	    }
 	  }
 	  break;
@@ -304,7 +294,6 @@ economy_report_dialog_update(void)
    
   HWND lv;
   int tax, total, i, entries_used;
-  char  *report_title;
   char   buf0 [64];
   char   buf1 [64];
   char   buf2 [64];
@@ -316,9 +305,8 @@ economy_report_dialog_update(void)
   if(is_report_dialogs_frozen()) return;      
   if(!economy_dlg) return;
   lv=GetDlgItem(economy_dlg,ID_TRADEREP_LIST);
-  report_title=get_report_title(_("Economy"));
-  SetWindowText(GetDlgItem(economy_dlg,ID_TRADEREP_TOP),
-		report_title);
+  SetWindowText(GetDlgItem(economy_dlg, ID_TRADEREP_TOP),
+		get_report_title(_("Economy")));
   ListView_DeleteAllItems(lv);
   row[0] = buf0;
   row[1] = buf1;
@@ -355,46 +343,18 @@ economy_report_dialog_update(void)
 **************************************************************************/
 static void economy_dlg_sell(HWND hWnd,int data)
 {
-  HWND lv;
-  int n,i,count=0,gold=0;
-  struct genlist_iterator myiter;
-  struct city *pcity;
-  struct packet_city_request packet;
-  char str[64];
-  int row;
-  lv=GetDlgItem(hWnd,ID_TRADEREP_LIST);
-  n=ListView_GetItemCount(lv);
-  
-  for(row=0;row<n;row++)
+  HWND lv = GetDlgItem(hWnd, ID_TRADEREP_LIST);
+  char str[1024];
+  int row, n = ListView_GetItemCount(lv);
+
+  for (row = 0; row < n; row++) {
     if (ListView_GetItemState(lv,row,LVIS_SELECTED)) {
-       
-      i=economy_improvement_type[row];
-      
-      genlist_iterator_init(&myiter, &game.player_ptr->cities.list, 0);
-      for(; ITERATOR_PTR(myiter);ITERATOR_NEXT(myiter)) {
-	pcity=(struct city *)ITERATOR_PTR(myiter);
-	if(!pcity->did_sell && city_got_building(pcity, i) &&
-	   (data ||
-	    improvement_obsolete(game.player_ptr,i) ||
-	    wonder_replacement(pcity, i) ))  {
-	  count++; gold+=improvement_value(i);
-	  packet.city_id=pcity->id;
-	  packet.build_id=i;
-	  send_packet_city_request(&aconnection, &packet, PACKET_CITY_SELL);
-	}            
-      }
-      if(count)  {
-	my_snprintf(str, sizeof(str), _("Sold %d %s for %d gold"),
-		    count, get_improvement_name(i), gold);
-      } else {
-	my_snprintf(str, sizeof(str), _("No %s could be sold"),
-		    get_improvement_name(i));
-      }      
-      
+      sell_all_improvements(economy_improvement_type[row],
+			    data == 0, str, sizeof(str));
       ListView_SetItemState(lv,row,0,LVIS_SELECTED);
       popup_notify_dialog(_("Sell-Off:"),_("Results"),str);
-    }   
-  return ;
+    }
+  }
 }
 
 /**************************************************************************
@@ -497,7 +457,8 @@ popup_economy_report_dialog(bool make_modal)
 *****************************************************************/
 static void upgrade_callback_yes(HWND w, void * data)
 {
-  send_packet_unittype_info(&aconnection,(size_t)data,PACKET_UNITTYPE_UPGRADE);  destroy_message_dialog(w);
+  dsend_packet_unit_type_upgrade(&aconnection, (size_t)data);
+  destroy_message_dialog(w);
 }
  
 /****************************************************************
@@ -617,13 +578,12 @@ activeunits_report_dialog_update(void)
 
     struct repoinfo unitarray[U_LAST];
     struct repoinfo unittotals;
-    char  *report_title;
 
     char *row[AU_COL];
     char   buf[AU_COL][64];
 
-    report_title=get_report_title(_("Units"));
-    SetWindowText(GetDlgItem(activeunits_dlg,ID_MILITARY_TOP),report_title);
+    SetWindowText(GetDlgItem(activeunits_dlg, ID_MILITARY_TOP),
+		  get_report_title(_("Units")));
     lv=GetDlgItem(activeunits_dlg,ID_MILITARY_LIST);
     ListView_DeleteAllItems(lv);
     for (i = 0; i < ARRAY_SIZE(row); i++) {
@@ -696,9 +656,9 @@ popup_activeunits_report_dialog(bool make_modal)
     {
       HWND lv;
       LV_COLUMN lvc;
-      const char *headers[]={N_("Unit Type"),N_("U"),
-			     N_("In-Prog"),N_("Active"),
-			     N_("Shield"),N_("Food"),NULL};
+      char *headers[] = {N_("Unit Type"), N_("U"),
+			 N_("In-Prog"), N_("Active"),
+			 N_("Shield"), N_("Food"), NULL};
       int i;
       struct fcwin_box *vbox;
       struct fcwin_box *hbox;
@@ -744,3 +704,269 @@ popup_activeunits_report_dialog(bool make_modal)
   activeunits_report_dialog_update();
   ShowWindow(activeunits_dlg,SW_SHOWNORMAL);
 }
+
+/****************************************************************
+  Show a dialog with player statistics at endgame.
+  TODO: Display all statistics in packet_endgame_report.
+*****************************************************************/
+void popup_endgame_report_dialog(struct packet_endgame_report *packet)
+{
+  char buffer[150 * MAX_NUM_PLAYERS];
+  int i;
+ 
+  buffer[0] = '\0';
+  for (i = 0; i < packet->nscores; i++) {
+    cat_snprintf(buffer, sizeof(buffer),
+                 PL_("%2d: The %s ruler %s scored %d point\n",
+                     "%2d: The %s ruler %s scored %d points\n",
+                     packet->score[i]),
+                 i + 1,
+                 get_nation_name(get_player(packet->id[i])->nation),
+                 get_player(packet->id[i])->name,
+                 packet->score[i]);
+  }
+  popup_notify_dialog(_("Final Report:"),
+                      _("The Greatest Civilizations in the world."),
+                      buffer);
+}
+
+/****************************************************************
+
+*****************************************************************/
+static void destroy_options_window(void)
+{
+  int i;
+  DestroyWindow(settable_options_dialog_win);
+  settable_options_dialog_win = NULL;
+  free(categories);
+  for (i = 0; i < num_tabs; i++) {
+    DestroyWindow(tab_wnds[i]);
+    tab_wnds[i] = NULL;
+  }
+  free(tab_wnds);
+}
+
+/****************************************************************
+
+*****************************************************************/
+static LONG CALLBACK OptionsWndProc(HWND hWnd,
+				    UINT message,
+				    WPARAM wParam,
+				    LPARAM lParam) 
+{
+  LPNMHDR nmhdr;
+
+  switch(message) {
+    case WM_CLOSE:
+      destroy_options_window();
+      break;
+    case WM_COMMAND:
+      if (LOWORD(wParam) == IDOK) {
+	int i, tab;
+        for (i = 0; i < num_settable_options; i++) {
+	  tab = categories[settable_options[i].category];
+	  if (settable_options[i].type == 0) {
+	    /* checkbox */
+	    int val = Button_GetState(GetDlgItem(tab_wnds[tab],
+				      ID_OPTIONS_BASE + i)) == BST_CHECKED;
+	    if (val != settable_options[i].val) {
+	      char buffer[MAX_LEN_MSG];
+	      my_snprintf(buffer, MAX_LEN_MSG, "/set %s %d",
+			  settable_options[i].name, val);
+	      send_chat(buffer);
+	    }
+	  } else if (settable_options[i].type == 1) {
+	    char buf[512];
+	    int val;
+	    Edit_GetText(GetDlgItem(tab_wnds[tab], ID_OPTIONS_BASE + i), buf,
+				    512);
+	    val = atoi(buf);
+	    if (val != settable_options[i].val) {
+	      char buffer[MAX_LEN_MSG];
+	      my_snprintf(buffer, MAX_LEN_MSG, "/set %s %d",
+			  settable_options[i].name, val);
+	      send_chat(buffer);
+	    }	    
+	  } else {
+	    char strval[512];
+	    Edit_GetText(GetDlgItem(tab_wnds[tab], ID_OPTIONS_BASE + i),
+			 strval, 512);
+	    if (strcmp(strval, settable_options[i].strval) != 0) {
+	      char buffer[MAX_LEN_MSG];
+	      my_snprintf(buffer, MAX_LEN_MSG, "/set %s %s",
+			  settable_options[i].name, strval);
+	      send_chat(buffer);
+	    }
+	  }
+	}
+      }
+      if ((LOWORD(wParam) == IDCANCEL) || (LOWORD(wParam) == IDOK)) {
+        destroy_options_window();
+      }
+      break;
+    case WM_NOTIFY:
+      nmhdr = (LPNMHDR)lParam;
+      if (nmhdr->hwndFrom == tab_ctrl) {
+	int i, sel;
+	sel = TabCtrl_GetCurSel(tab_ctrl);
+	for (i = 0; i < num_tabs; i++) {
+	  ShowWindow(tab_wnds[i], SW_HIDE);
+	}
+	if ((sel >= 0) && (sel < num_tabs)) {
+	  ShowWindow(tab_wnds[sel], SW_SHOWNORMAL);
+	}
+      }
+      break;
+    default:
+      return DefWindowProc(hWnd,message,wParam,lParam);
+  }
+  return (0);
+}
+
+/****************************************************************
+
+*****************************************************************/
+static LONG CALLBACK OptionsWndProc2(HWND hWnd,
+				     UINT message,
+				     WPARAM wParam,
+				     LPARAM lParam) 
+{
+  switch(message) {
+    default:
+      return DefWindowProc(hWnd,message,wParam,lParam);
+  }
+  return (0);
+}
+
+/*************************************************************************
+  Server options dialog
+*************************************************************************/
+static void create_settable_options_dialog(void)
+{
+  HWND win;
+  struct fcwin_box *vbox, *hbox, **tab_boxes;
+  char **titles;
+  void **user_data;
+  WNDPROC *tab_procs;
+  bool *used;
+  int i, j;
+
+  num_tabs = 0;
+
+  used = fc_malloc(num_options_categories * sizeof(bool));
+  categories = fc_malloc(num_options_categories * sizeof(int));
+
+  for(i = 0; i < num_options_categories; i++) {
+    used[i] = FALSE;
+  }
+
+  for (i = 0; i < num_settable_options; i++) {
+    used[settable_options[i].category] = TRUE;
+  }
+
+  for(i = 0; i < num_options_categories; i++) {
+    if (used[i]) {
+      categories[i] = num_tabs;
+      num_tabs++;
+    }
+  }
+
+  titles = fc_malloc(sizeof(titles) * num_tabs);
+  tab_procs = fc_malloc(sizeof(tab_procs) * num_tabs);
+  tab_wnds = fc_malloc(sizeof(tab_wnds) * num_tabs);
+  tab_boxes = fc_malloc(sizeof(tab_boxes) * num_tabs);
+  user_data = fc_malloc(sizeof(user_data) * num_tabs);
+
+  j = 0;
+  for (i = 0; i < num_options_categories; i++) {
+    if (used[i]) {
+      titles[j] = _(options_categories[i]);
+      tab_procs[j] = OptionsWndProc2;
+      j++;
+    }
+  }
+
+  win = fcwin_create_layouted_window(OptionsWndProc, _("Game Options"),
+				     WS_OVERLAPPEDWINDOW, 20, 20,
+				     root_window, NULL, REAL_CHILD, NULL);
+  settable_options_dialog_win = win;
+  vbox = fcwin_vbox_new(win, FALSE);
+
+  /* create a notebook for the options */
+  tab_ctrl = fcwin_box_add_tab(vbox, tab_procs, tab_wnds, titles,
+			       user_data, num_tabs, 0, 0, TRUE, TRUE, 5);
+  j = 0;
+  for (i = 0; i < num_options_categories; i++) {
+    if (used[i]) {
+      tab_boxes[j] = fcwin_vbox_new(tab_wnds[j], FALSE);
+      fcwin_set_box(tab_wnds[j], tab_boxes[j]);
+      j++;
+    }
+  }
+
+  for (i = 0; i < num_settable_options; i++) {
+    j = categories[settable_options[i].category];
+    hbox = fcwin_hbox_new(tab_wnds[j], FALSE);
+    fcwin_box_add_static(hbox, _(settable_options[i].short_help),
+			 0, 0, FALSE, TRUE, 5);
+    fcwin_box_add_static(hbox, "", 0, 0, TRUE, TRUE, 0);
+    if (settable_options[i].type == 0) {
+      HWND check;
+      /* boolean */
+      check = fcwin_box_add_checkbox(hbox, "", ID_OPTIONS_BASE + i, 0, FALSE,
+				     TRUE, 5);
+      Button_SetCheck(check,
+		      settable_options[i].val ? BST_CHECKED : BST_UNCHECKED);
+    } else if (settable_options[i].type == 1) {
+      /* integer */
+      char buf[80];
+      int length;
+      my_snprintf(buf, 80, "%d", settable_options[i].max);
+      buf[79] = 0;
+      length = strlen(buf);
+      my_snprintf(buf, 80, "%d", settable_options[i].min);
+      buf[79] = 0;
+      if (length < strlen(buf)) {
+        length = strlen(buf);
+      }
+      my_snprintf(buf, 80, "%d", settable_options[i].val);
+      fcwin_box_add_edit(hbox, buf, length, ID_OPTIONS_BASE + i, 0, FALSE,
+			 TRUE, 5);
+    } else {
+      /* string */
+      fcwin_box_add_edit(hbox, settable_options[i].strval, 40,
+			 ID_OPTIONS_BASE + i, 0, FALSE, TRUE, 5);
+    }
+    fcwin_box_add_box(tab_boxes[j], hbox, FALSE, TRUE, 5);
+  }
+
+  hbox = fcwin_hbox_new(win, FALSE);
+  fcwin_box_add_button(hbox, _("OK"), IDOK, 0, TRUE, TRUE, 5);
+  fcwin_box_add_button(hbox, _("Cancel"), IDCANCEL, 0, TRUE, TRUE, 5);
+  fcwin_box_add_box(vbox, hbox, TRUE, TRUE, 5);
+
+  free(used);
+  free(titles);
+  free(tab_procs);
+  free(user_data);
+  free(tab_boxes);
+
+  fcwin_set_box(win, vbox);
+
+  for (i = 0; i < num_tabs; i++) {
+    fcwin_redo_layout(tab_wnds[i]);
+  }
+  fcwin_redo_layout(win);
+  ShowWindow(win, SW_SHOWNORMAL);
+  ShowWindow(tab_wnds[0], SW_SHOWNORMAL);
+}
+
+/**************************************************************************
+  Show a dialog with the server options.
+**************************************************************************/
+void popup_settable_options_dialog(void)
+{
+  if (!settable_options_dialog_win) {
+    create_settable_options_dialog();
+  }
+}
