@@ -11,6 +11,7 @@
    GNU General Public License for more details.
 
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -36,8 +37,10 @@
 #include "canvas.h"
 #include "pixcomm.h"
 
+#include "fciconv.h"
 #include "fcintl.h"
 #include "game.h"
+#include "government.h"
 #include "log.h"
 #include "map.h"
 #include "support.h"
@@ -64,12 +67,21 @@
 
 #include "freeciv.ico"
 
+const char *client_string = "gui-xaw";
+
+client_option gui_options[] = {
+  /* None. */
+};
+const int num_gui_options = ARRAY_SIZE(gui_options);
+
 static AppResources appResources;
 
 /* ids of the units icons in information display: (or 0) */
 static int unit_ids[MAX_NUM_UNITS_BELOW];  
 
 static void setup_widgets(void);
+void fill_econ_label_pixmaps(void);
+void fill_unit_below_pixmaps(void);
 
 /**************************************************************************
 ...
@@ -155,6 +167,9 @@ XtAppContext app_context;
 /* this GC will be the default one all thru freeciv */
 GC civ_gc; 
 
+/* This is for drawing border lines */
+GC border_line_gc; 
+
 /* and this one is used for filling with the bg color */
 GC fill_bg_gc;
 GC fill_tile_gc;
@@ -162,10 +177,10 @@ Pixmap gray50,gray25;
 
 /* overall font GC                                    */
 GC font_gc;
-XFontStruct *main_font_struct;
+XFontSet main_font_set;
 /* productions font GC                                */
 GC prod_font_gc;
-XFontStruct *prod_font_struct;
+XFontSet prod_font_set;
 
 Widget toplevel, main_form, menu_form, below_menu_form, left_column_form;
 Widget bottom_form;
@@ -186,18 +201,6 @@ Pixmap unit_below_pixmap[MAX_NUM_UNITS_BELOW];
 Widget more_arrow_label;
 Window root_window;
 
-/* this pixmap acts as a backing store for the map_canvas widget */
-Pixmap map_canvas_store = 0;
-int map_canvas_store_twidth, map_canvas_store_theight;
-
-/* this pixmap acts as a backing store for the overview_canvas widget */
-Pixmap overview_canvas_store;
-int overview_canvas_store_width, overview_canvas_store_height;
-
-/* this pixmap is used when moving units etc */
-Pixmap single_tile_pixmap;
-int single_tile_pixmap_width, single_tile_pixmap_height;
-
 XtInputId x_input_id;
 XtIntervalId x_interval_id;
 Atom wm_delete_window;
@@ -222,7 +225,7 @@ static int myerr(Display *p, XErrorEvent *e)
 static void print_usage(const char *argv0)
 {
   /* add client-specific usage information here */
-  fprintf(stderr, _("Report bugs to <%s>.\n"), BUG_EMAIL_ADDRESS);
+  fc_fprintf(stderr, _("Report bugs to <%s>.\n"), BUG_EMAIL_ADDRESS);
 }
 
 /**************************************************************************
@@ -238,7 +241,7 @@ static void parse_options(int argc, char **argv)
     if (is_option("--help", argv[i]))
     {
       print_usage(argv[0]);
-      exit(0);
+      exit(EXIT_SUCCESS);
     }
     i += 1;
   }
@@ -256,6 +259,14 @@ static Boolean toplevel_work_proc(XtPointer client_data)
   return (True);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void ui_init(void)
+{
+  init_character_encodings(NULL, TRUE);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -267,7 +278,7 @@ void ui_main(int argc, char *argv[])
   parse_options(argc, argv);
 
   /* include later - pain to see the warning at every run */
-  /* XtSetLanguageProc(NULL, (XtLanguageProc)NULL, NULL); */
+  XtSetLanguageProc(NULL, NULL, NULL);
   
   toplevel = XtVaAppInitialize(
 	       &app_context,               /* Application context */
@@ -293,18 +304,18 @@ void ui_main(int argc, char *argv[])
     freelog(LOG_FATAL, _("No version number in resources."));
     freelog(LOG_FATAL, _("You probably have an old (circa V1.0)"
 			 " Freeciv resource file somewhere."));
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   /* TODO: Use capabilities here instead of version numbers */
-  if(strncmp(appResources.version, VERSION_STRING,
-	     strlen(appResources.version))) {
+  if (0 != strncmp(appResources.version, VERSION_STRING,
+		   strlen(appResources.version))) {
     freelog(LOG_FATAL, _("Game version does not match Resource version."));
     freelog(LOG_FATAL, _("Game version: %s - Resource version: %s"), 
 	    VERSION_STRING, appResources.version);
     freelog(LOG_FATAL, _("You might have an old Freeciv resourcefile"
 			 " in /usr/lib/X11/app-defaults"));
-    exit(1);
+    exit(EXIT_FAILURE);
   }
   
   if(!appResources.gotAppDefFile) {
@@ -320,47 +331,79 @@ void ui_main(int argc, char *argv[])
   
   if(display_color_type!=COLOR_DISPLAY) {
     freelog(LOG_FATAL, _("Only color displays are supported for now..."));
-    /*    exit(1); */
+    /*    exit(EXIT_FAILURE); */
   }
   
   icon_pixmap = XCreateBitmapFromData(display,
 				      RootWindowOfScreen(XtScreen(toplevel)),
 				      freeciv_bits,
-				      freeciv_width, freeciv_height );
+				      freeciv_width, freeciv_height);
   XtVaSetValues(toplevel, XtNiconPixmap, icon_pixmap, NULL);
 
   init_color_system();
 
   {
     XGCValues values;
+    char **missing_charset_list_return;
+    int missing_charset_count_return;
+    char *def_string_return;
 
     values.graphics_exposures = False;
     civ_gc = XCreateGC(display, root_window, GCGraphicsExposures, &values);
 
-    main_font_struct=XLoadQueryFont(display, city_names_font);
-    if(main_font_struct==0) {
-      freelog(LOG_FATAL, _("Failed loading font: %s"), city_names_font);
-      exit(1);
+    free(city_names_font);
+    city_names_font = mystrdup("-*-*-*-*-*--14-*");
+
+    free(city_productions_font_name);
+    city_productions_font_name = mystrdup("-*-*-*-*-*--14-*");
+
+    main_font_set = XCreateFontSet(display, city_names_font,
+	&missing_charset_list_return,
+	&missing_charset_count_return,
+	&def_string_return);
+    if(!main_font_set) {
+      freelog(LOG_FATAL, _("Unable to open fontset: %s"),
+	  city_names_font);
+      exit(EXIT_FAILURE);
+    }
+    for(i = 0; i < missing_charset_count_return; i++) {
+      freelog(LOG_ERROR, _("Font for charset %s is lacking"),
+	  missing_charset_list_return[i]);
     }
     values.foreground = colors_standard[COLOR_STD_WHITE];
     values.background = colors_standard[COLOR_STD_BLACK];
-    values.font = main_font_struct->fid;
     font_gc= XCreateGC(display, root_window, 
-		       GCForeground|GCBackground|GCFont|GCGraphicsExposures, 
+		       GCForeground|GCBackground|GCGraphicsExposures, 
 		       &values);
 
-    prod_font_struct=XLoadQueryFont(display, city_productions_font_name);
-    if(prod_font_struct==0) {
-      freelog(LOG_FATAL, _("Failed loading font: %s"), city_productions_font_name);
-      exit(1);
+    prod_font_set = XCreateFontSet(display, city_productions_font_name,
+	&missing_charset_list_return,
+	&missing_charset_count_return,
+	&def_string_return);
+    if(!prod_font_set) {
+      freelog(LOG_FATAL, _("Unable to open fontset: %s"),
+	  city_productions_font_name);
+      exit(EXIT_FAILURE);
+    }
+    for(i = 0; i < missing_charset_count_return; i++) {
+      freelog(LOG_ERROR, _("Font for charset %s is lacking"),
+	  missing_charset_list_return[i]);
     }
     values.foreground = colors_standard[COLOR_STD_WHITE];
     values.background = colors_standard[COLOR_STD_BLACK];
-    values.font = prod_font_struct->fid;
     prod_font_gc= XCreateGC(display, root_window,
-			    GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+			    GCForeground|GCBackground|GCGraphicsExposures,
 			    &values);
 
+    values.line_width = BORDER_WIDTH;
+    values.line_style = LineOnOffDash;
+    values.cap_style = CapNotLast;
+    values.join_style = JoinMiter;
+    values.fill_style = FillSolid;
+    border_line_gc = XCreateGC(display, root_window,
+			       GCGraphicsExposures|GCLineWidth|GCLineStyle
+			       |GCCapStyle|GCJoinStyle|GCFillStyle, &values);
+
     values.foreground = 0;
     values.background = 0;
     fill_bg_gc= XCreateGC(display, root_window, 
@@ -374,14 +417,14 @@ void ui_main(int argc, char *argv[])
   }
 
   {
-    unsigned char d1[]={0x03,0x0c,0x03,0x0c};
-    unsigned char d2[]={0x08,0x02,0x08,0x02};
-    gray50=XCreateBitmapFromData(display,root_window,d1,4,4);
-    gray25=XCreateBitmapFromData(display,root_window,d2,4,4);
+    char d1[]={0x03,0x0c,0x03,0x0c};
+    char d2[]={0x08,0x02,0x08,0x02};
+    gray50 = XCreateBitmapFromData(display, root_window, d1, 4, 4);
+    gray25 = XCreateBitmapFromData(display, root_window, d2, 4, 4);
   }
   
   /* 135 below is rough value (could be more intelligent) --dwp */
-  num_units_below = 135/(int)NORMAL_TILE_WIDTH;
+  num_units_below = 135 / UNIT_TILE_WIDTH;
   num_units_below = MIN(num_units_below,MAX_NUM_UNITS_BELOW);
   num_units_below = MAX(num_units_below,1);
   
@@ -399,12 +442,8 @@ void ui_main(int argc, char *argv[])
   InitializeActions(app_context);
 
   /* Do this outside setup_widgets() so after tiles are loaded */
-  for(i=0;i<10;i++)  {
-    XtVaSetValues(econ_label[i], XtNbitmap,
-		  get_citizen_pixmap(i<5?1:2), NULL);
-    XtAddCallback(econ_label[i], XtNcallback, taxrates_callback,
-		  (XtPointer)i);  
-  }
+
+  fill_econ_label_pixmaps();
 		
   XtAddCallback(map_horizontal_scrollbar, XtNjumpProc, 
 		scrollbar_jump_callback, NULL);
@@ -420,28 +459,12 @@ void ui_main(int argc, char *argv[])
 
   XtRealizeWidget(toplevel);
 
+  x_interval_id = XtAppAddTimeOut(app_context, TIMER_INTERVAL,
+				  timer_callback, NULL);
 
-  x_interval_id=XtAppAddTimeOut(app_context, 500,
-				(XtTimerCallbackProc)timer_callback, NULL);
+  init_mapcanvas_and_overview();
 
-  map_canvas_resize();
-
-  overview_canvas_store=0;
-  set_overview_dimensions(80, 50);
-
-  XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_WHITE]);
-  XFillRectangle(display, overview_canvas_store, fill_bg_gc, 0, 0, 
-		 overview_canvas_store_width, overview_canvas_store_height);
-
-  single_tile_pixmap=XCreatePixmap(display, XtWindow(overview_canvas), 
-				   NORMAL_TILE_WIDTH,
-				   NORMAL_TILE_HEIGHT,
-				   display_depth);
-
-  for(i=0; i<num_units_below; i++)
-    unit_below_pixmap[i]=XCreatePixmap(display, XtWindow(overview_canvas), 
-				       NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT, 
-				       display_depth);  
+  fill_unit_below_pixmaps();
 
   set_indicator_icons(0, 0, 0, 0);
 
@@ -450,8 +473,6 @@ void ui_main(int argc, char *argv[])
   XtOverrideTranslations(toplevel,
     XtParseTranslationTable ("<Message>WM_PROTOCOLS: msg-quit-freeciv()"));
 
-  load_options();
-
   XtSetSensitive(toplevel, FALSE);
 
   XtAppMainLoop(app_context);
@@ -475,7 +496,6 @@ static void unit_icon_callback(Widget w, XtPointer client_data,
   punit=find_unit_by_id(unit_ids[i]);
   if(punit) { /* should always be true at this point */
     if (punit->owner == game.player_idx) {  /* may be non-true if alliance */
-      request_new_unit_activity(punit, ACTIVITY_IDLE);
       set_unit_focus(punit);
     }
   }
@@ -602,8 +622,8 @@ void setup_widgets(void)
   unit_pix_canvas = XtVaCreateManagedWidget("unitpixcanvas", 
 					   pixcommWidgetClass,
 					   left_column_form, 
-					   XtNwidth, NORMAL_TILE_WIDTH,
-					   XtNheight, NORMAL_TILE_HEIGHT,
+					   XtNwidth, UNIT_TILE_WIDTH,
+					   XtNheight, UNIT_TILE_HEIGHT,
 					   NULL);
 
   for(i=0; i<num_units_below; i++) {
@@ -613,8 +633,10 @@ void setup_widgets(void)
     unit_below_canvas[i] = XtVaCreateManagedWidget(unit_below_name,
 						   pixcommWidgetClass,
 						   left_column_form, 
-						   XtNwidth, NORMAL_TILE_WIDTH,
-						   XtNheight, NORMAL_TILE_HEIGHT,
+						   XtNwidth,
+						   UNIT_TILE_WIDTH,
+						   XtNheight,
+						   UNIT_TILE_HEIGHT,
 						   NULL);
     XtAddCallback(unit_below_canvas[i], XtNcallback, unit_icon_callback,
 		  (XtPointer)i);  
@@ -665,9 +687,10 @@ void setup_widgets(void)
 /**************************************************************************
 ...
 **************************************************************************/
-void main_quit_freeciv(void)
-{ 
-  exit(0);
+void xaw_ui_exit(void)
+{
+  tilespec_free_tiles();
+  ui_exit();
 }
 
 /**************************************************************************
@@ -683,22 +706,26 @@ void main_show_info_popup(XEvent *event)
     char buf[512];
 
     my_snprintf(buf, sizeof(buf),
-		 _("%s People\n"
-		   "Year: %s\n"
-		   "Gold: %d\n"
-		   "Net Income: %d\n"
-		   "Tax:%d Lux:%d Sci:%d\n"
-		   "Researching %s: %d/%d"),
-	    int_to_text(civ_population(game.player_ptr)),
-	    textyear(game.year),
-	    game.player_ptr->economic.gold,
-	    turn_gold_difference,
-	    game.player_ptr->economic.tax,
-	    game.player_ptr->economic.luxury,
-	    game.player_ptr->economic.science,
-	    advances[game.player_ptr->research.researching].name,
-	    game.player_ptr->research.researched,
-	    research_time(game.player_ptr));
+		_("%s People\n"
+		  "Year: %s Turn: %d\n"
+		  "Gold: %d\n"
+		  "Net Income: %d\n"
+		  "Tax:%d Lux:%d Sci:%d\n"
+		  "Researching %s: %d/%d\n"
+		  "Government: %s"),
+		population_to_text(civ_population(game.player_ptr)),
+		textyear(game.year), game.turn,
+		game.player_ptr->economic.gold,
+		player_get_expected_income(game.player_ptr),
+		game.player_ptr->economic.tax,
+		game.player_ptr->economic.luxury,
+		game.player_ptr->economic.science,
+		(game.player_ptr->research.researching == A_UNSET) ?
+		  advances[A_NONE].name :
+		  advances[game.player_ptr->research.researching].name,
+		game.player_ptr->research.bulbs_researched,
+		total_bulbs_required(game.player_ptr),
+		get_government_name(game.player_ptr->government));
 
     p=XtCreatePopupShell("popupinfo", 
 			 overrideShellWidgetClass, 
@@ -722,24 +749,19 @@ void main_show_info_popup(XEvent *event)
 }
 
 /**************************************************************************
-...
+ Update the connected users list at pregame state.
 **************************************************************************/
-void sound_bell(void)
+void update_conn_list_dialog(void)
 {
-  XBell(display, 100);
+  /* PORTME */
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-void enable_turn_done_button(void)
+void sound_bell(void)
 {
-  if(game.player_ptr->ai.control && !ai_manual_turn_done)
-    user_ended_turn();
-
-  update_turn_done_button(1);
-  XtSetSensitive(turn_done_button, 
-                 !game.player_ptr->ai.control||ai_manual_turn_done);
+  XBell(display, 100);
 }
 
 /**************************************************************************
@@ -750,15 +772,38 @@ static void get_net_input(XtPointer client_data, int *fid, XtInputId *id)
   input_from_server(*fid);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+static void set_wait_for_writable_socket(struct connection *pc,
+					 bool socket_writable)
+{
+  static bool previous_state = FALSE;
+
+  if (previous_state == socket_writable)
+    return;
+  freelog(LOG_DEBUG, "set_wait_for_writable_socket(%d)", socket_writable);
+  XtRemoveInput(x_input_id);
+  x_input_id = XtAppAddInput(app_context, aconnection.sock,
+			     (XtPointer) (XtInputReadMask |
+					  (socket_writable ?
+					   XtInputWriteMask : 0) |
+					  XtInputExceptMask),
+			     (XtInputCallbackProc) get_net_input, NULL);
+  previous_state = socket_writable;
+}
+
 /**************************************************************************
  This function is called after the client succesfully
  has connected to the server
 **************************************************************************/
 void add_net_input(int sock)
 {
-  x_input_id=XtAppAddInput(app_context, sock, 
-			   (XtPointer) XtInputReadMask,
-			   (XtInputCallbackProc) get_net_input, NULL);
+  x_input_id = XtAppAddInput(app_context, sock,
+			     (XtPointer) (XtInputReadMask |
+					  XtInputExceptMask),
+			     (XtInputCallbackProc) get_net_input, NULL);
+  aconnection.notify_of_writable_data = set_wait_for_writable_socket;
 }
 
 /**************************************************************************
@@ -784,43 +829,11 @@ void end_turn_callback(Widget w, XtPointer client_data, XtPointer call_data)
 /**************************************************************************
 ...
 **************************************************************************/
-void timer_callback(XtPointer client_data, XtIntervalId *id)
+void timer_callback(XtPointer client_data, XtIntervalId * id)
 {
-  static int flip;
-  
-  x_interval_id=XtAppAddTimeOut(app_context, 500,
-				(XtTimerCallbackProc)timer_callback, NULL);  
-  
-  if(get_client_state()==CLIENT_GAME_RUNNING_STATE) {
-  
-    if(game.player_ptr->is_connected && game.player_ptr->is_alive && 
-       !game.player_ptr->turn_done) { 
-      int i, is_waiting, is_moving;
-      
-      for(i=0, is_waiting=0, is_moving=0; i<game.nplayers; i++)
-	if(game.players[i].is_alive && game.players[i].is_connected) {
-	  if(game.players[i].turn_done)
-	    is_waiting++;
-	  else
-	    is_moving++;
-	}
-      
-      if(is_moving==1 && is_waiting) 
-	update_turn_done_button(0);  /* stress the slow player! */
-    }
-    
-    blink_active_unit();
-    
-    if(flip) {
-      update_timeout_label();
-      if(seconds_to_turndone > 0)
-	seconds_to_turndone--;
-      else
-	seconds_to_turndone = 0;
-    }
-    
-    flip=!flip;
-  }
+  x_interval_id = XtAppAddTimeOut(app_context, TIMER_INTERVAL,
+				  timer_callback, NULL);
+  real_timer_callback();
 }
 
 /**************************************************************************
@@ -841,11 +854,11 @@ void set_unit_icon(int idx, struct unit *punit)
     unit_ids[idx] = punit ? punit->id : 0;
   }
   
-  if (flags_are_transparent || punit==NULL) {
-    XawPixcommClear(w);
-  }
+  XawPixcommClear(w);
   if (punit) {
-    put_unit_pixmap(punit, XawPixcommPixmap(w), 0, 0);
+    struct canvas store = {XawPixcommPixmap(w)};
+
+    put_unit(punit, &store, 0, 0);
     xaw_expose_now(w);
   }
 }
@@ -855,15 +868,74 @@ void set_unit_icon(int idx, struct unit *punit)
   Maintains a static record of current state to avoid unnecessary redraws.
   Note initial state should match initial gui setup (off).
 **************************************************************************/
-void set_unit_icons_more_arrow(int onoff)
+void set_unit_icons_more_arrow(bool onoff)
 {
-  static int showing = 0;
-  if (onoff==1 && !showing) {
+  static bool showing = FALSE;
+
+  if (onoff && !showing) {
     xaw_set_bitmap(more_arrow_label, sprites.right_arrow->pixmap);
-    showing = 1;
+    showing = TRUE;
   }
-  else if(onoff==0 && showing) {
+  else if(!onoff && showing) {
     xaw_set_bitmap(more_arrow_label, None);
-    showing = 0;
+    showing = FALSE;
   }
 }
+
+/**************************************************************************
+  Called to fill econ_label pixmaps (showing tax/lux/sci rates).
+
+  It may be called again if the tileset changes.
+**************************************************************************/
+void fill_econ_label_pixmaps(void)
+{
+  int i;
+  int econ_label_count = 10;
+
+  for(i = 0; i < econ_label_count; i++) {
+    struct Sprite *s = i < 5 ? sprites.tax_science : sprites.tax_gold;
+
+    XtVaSetValues(econ_label[i], XtNbitmap,
+		  s->pixmap, NULL);
+    XtAddCallback(econ_label[i], XtNcallback, taxrates_callback,
+		  INT_TO_XTPOINTER(i));
+  }
+}
+
+/**************************************************************************
+  Called to fill unit_below pixmaps. They are on the left of the
+  screen that shows all of the inactive units in the current tile.
+
+  It may be called again if the tileset changes.
+**************************************************************************/
+void fill_unit_below_pixmaps(void)
+{
+  long i;
+
+  for (i = 0; i < num_units_below; i++) {
+    unit_below_pixmap[i] = XCreatePixmap(display, XtWindow(overview_canvas),
+					 UNIT_TILE_WIDTH, UNIT_TILE_HEIGHT,
+					 display_depth);
+  }
+}
+
+/**************************************************************************
+  Called when the tileset is changed to reset indicators pixmaps.
+**************************************************************************/
+void reset_econ_label_pixmaps(void)
+{
+  fill_econ_label_pixmaps();
+}
+
+/**************************************************************************
+  Called when the tileset is changed to reset unit pixmaps.
+**************************************************************************/
+void reset_unit_below_pixmaps(void)
+{
+  long i;
+
+  for (i = 0; i < num_units_below; i++) {
+    XFreePixmap(display, unit_below_pixmap[i]);
+  }
+  fill_unit_below_pixmaps();
+}
