@@ -10,13 +10,14 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -64,9 +65,7 @@ static Widget help_title;
 static Widget help_improvement_cost, help_improvement_cost_data;
 static Widget help_improvement_upkeep, help_improvement_upkeep_data;
 static Widget help_improvement_req, help_improvement_req_data;
-static Widget help_improvement_variant, help_improvement_variant_data;
 static Widget help_wonder_obsolete, help_wonder_obsolete_data;
-static Widget help_wonder_variant, help_wonder_variant_data;
 
 static Widget help_tech_tree;
 static Widget help_tree_viewport;
@@ -90,6 +89,9 @@ static Widget help_terrain_irrigation_result_time, help_terrain_irrigation_resul
 static Widget help_terrain_mining_result_time, help_terrain_mining_result_time_data;
 static Widget help_terrain_transform_result_time, help_terrain_transform_result_time_data;
 
+/* HACK: we use a static string for convenience. */
+static char long_buffer[64000];
+
 static void create_help_page(enum help_page_type type);
 
 static void help_close_command_callback(Widget w, XtPointer client_data, 
@@ -119,9 +121,9 @@ static char *topic_list[1024];
 *****************************************************************/
 static void set_title_topic(const struct help_item *pitem)
 {
-  if(!strcmp(pitem->topic, "Freeciv")
-     || !strcmp(pitem->topic, "About")
-     || !strcmp(pitem->topic, _("About"))) {
+  if (strcmp(pitem->topic, "Freeciv") == 0
+      || strcmp(pitem->topic, "About") == 0
+      || strcmp(pitem->topic, _("About")) == 0) {
     xaw_set_label(help_title, freeciv_name_version());
   } else {
     xaw_set_label(help_title, pitem->topic);
@@ -143,7 +145,7 @@ void popdown_help_dialog(void)
 /****************************************************************
 ...
 *****************************************************************/
-void popup_help_dialog_typed(char *item, enum help_page_type htype)
+void popup_help_dialog_typed(const char *item, enum help_page_type htype)
 {
   Position x, y;
   Dimension width, height;
@@ -167,7 +169,7 @@ void popup_help_dialog_typed(char *item, enum help_page_type htype)
 Not sure if this should call _(item) as it does, or whether all
 callers of this function should do so themselves... --dwp
 *****************************************************************/
-void popup_help_dialog_string(char *item)
+void popup_help_dialog_string(const char *item)
 {
   popup_help_dialog_typed(_(item), HELP_ANY);
 }
@@ -279,8 +281,8 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
   }
   
   my_snprintf(label, sizeof(label),
-	      "%s:%d",advances[tech].name,
-	      tech_goal_turns(game.player_ptr,tech));
+	      "%s:%d", advances[tech].name,
+	      num_unknown_techs_for_goal(game.player_ptr, tech));
 
   if(parent) {
     l=XtVaCreateManagedWidget("treenode", 
@@ -298,7 +300,8 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
 			      NULL);
   }
 
-  XtAddCallback(l, XtNcallback, help_tree_node_callback, (XtPointer)tech);
+  XtAddCallback(l, XtNcallback, help_tree_node_callback,
+		INT_TO_XTPOINTER(tech));
 
   XtVaSetValues(l, XtVaTypedArg, XtNbackground, 
 		XtRString, bg, strlen(bg)+1, NULL);
@@ -402,18 +405,7 @@ static void create_help_page(enum help_page_type type)
 				labelWidgetClass, 
 				help_right_form,
 				NULL);
-      help_improvement_variant =
-	I_L(XtVaCreateManagedWidget("helpimprvariant", 
-				    labelWidgetClass, 
-				    help_right_form,
-				    NULL));
-      help_improvement_variant_data =
-	XtVaCreateManagedWidget("helpimprvariantdata",
-				labelWidgetClass, 
-				help_right_form,
-				NULL);
-    }
-    else {
+    } else {
       help_wonder_obsolete =
 	I_L(XtVaCreateManagedWidget("helpwonderobsolete", 
 				    labelWidgetClass, 
@@ -424,16 +416,6 @@ static void create_help_page(enum help_page_type type)
 				labelWidgetClass, 
 				help_right_form,
 				NULL);
-      help_wonder_variant =
-	I_L(XtVaCreateManagedWidget("helpwondervariant", 
-				    labelWidgetClass, 
-				    help_right_form,
-				    NULL));
-      help_wonder_variant_data =
-	XtVaCreateManagedWidget("helpwondervariantdata",
-				labelWidgetClass, 
-				help_right_form,
-				NULL);
     }
 
     XtVaGetValues(help_improvement_req, XtNy, &ay, XtNheight, &ah, NULL);
@@ -761,18 +743,16 @@ static void create_help_page(enum help_page_type type)
 static void help_update_improvement(const struct help_item *pitem,
 				    char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
   
   create_help_page(HELP_IMPROVEMENT);
   
   if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d ", imp->build_cost);
+    sprintf(buf, "%d ", impr_build_shield_cost(which));
     xaw_set_label(help_improvement_cost_data, buf);
     sprintf(buf, "%d ", imp->upkeep);
     xaw_set_label(help_improvement_upkeep_data, buf);
-    sprintf(buf, "%d ", imp->variant);
-    xaw_set_label(help_improvement_variant_data, buf);
     if (imp->tech_req == A_LAST) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
@@ -784,12 +764,11 @@ static void help_update_improvement(const struct help_item *pitem,
   else {
     xaw_set_label(help_improvement_cost_data, "0 ");
     xaw_set_label(help_improvement_upkeep_data, "0 ");
-    xaw_set_label(help_improvement_variant_data,"0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
-    create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3);
+    create_tech_tree(help_tech_tree, 0, A_LAST, 3);
   }
   set_title_topic(pitem);
-  helptext_improvement(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 }
   
@@ -799,36 +778,37 @@ static void help_update_improvement(const struct help_item *pitem,
 static void help_update_wonder(const struct help_item *pitem,
 			       char *title, int which)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
   
   create_help_page(HELP_WONDER);
 
   if (which<game.num_impr_types) {
     struct impr_type *imp = &improvement_types[which];
-    sprintf(buf, "%d ", imp->build_cost);
+    sprintf(buf, "%d ", impr_build_shield_cost(which));
     xaw_set_label(help_improvement_cost_data, buf);
-    sprintf(buf, "%d ", imp->variant);
-    xaw_set_label(help_wonder_variant_data, buf);
     if (imp->tech_req == A_LAST) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
 		    advances[imp->tech_req].name);
     }
-    xaw_set_label(help_wonder_obsolete_data,
-		  advances[imp->obsolete_by].name);
+    if (tech_exists(imp->obsolete_by)) {
+      xaw_set_label(help_wonder_obsolete_data,
+		    advances[imp->obsolete_by].name);
+    } else {
+      xaw_set_label(help_wonder_obsolete_data, _("(Never)"));
+    }
     create_tech_tree(help_tech_tree, 0, imp->tech_req, 3);
   }
   else {
     /* can't find wonder */
     xaw_set_label(help_improvement_cost_data, "0 ");
-    xaw_set_label(help_wonder_variant_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
     xaw_set_label(help_wonder_obsolete_data, _("None"));
     create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3); 
   }
   set_title_topic(pitem);
-  helptext_wonder(buf, which, pitem->text);
+  helptext_building(buf, sizeof(buf), which, pitem->text);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 }
 
@@ -843,7 +823,7 @@ static void help_update_unit_type(const struct help_item *pitem,
   create_help_page(HELP_UNIT);
   if (i<game.num_unit_types) {
     struct unit_type *utype = get_unit_type(i);
-    sprintf(buf, "%d ", utype->build_cost);
+    sprintf(buf, "%d ", unit_build_shield_cost(i));
     xaw_set_label(help_unit_cost_data, buf);
     sprintf(buf, "%d ", utype->attack_strength);
     xaw_set_label(help_unit_attack_data, buf);
@@ -903,25 +883,26 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
   create_help_page(HELP_TECH);
   set_title_topic(pitem);
 
-  if (i<game.num_tech_types) {
+  if (!is_future_tech(i)) {
     create_tech_tree(help_tech_tree, 0, i, 3);
     helptext_tech(buf, i, pitem->text);
 
-    for(j=0; j<game.num_impr_types; ++j) {
+    impr_type_iterate(j) {
       if(i==improvement_types[j].tech_req) 
 	sprintf(buf+strlen(buf), _("Allows %s.\n"),
 		improvement_types[j].name);
       if(i==improvement_types[j].obsolete_by)
 	sprintf(buf+strlen(buf), _("Obsoletes %s.\n"),
 		improvement_types[j].name);
-    }
-    for(j=0; j<game.num_unit_types; ++j) {
+    } impr_type_iterate_end;
+
+    unit_type_iterate(j) {
       if(i==get_unit_type(j)->tech_requirement) 
 	sprintf(buf+strlen(buf), _("Allows %s.\n"), 
 		get_unit_type(j)->name);
-    }
+    } unit_type_iterate_end;
 
-    for(j=0; j<game.num_tech_types; ++j) {
+    for (j = 0; j < game.num_tech_types; j++) {
       if(i==advances[j].req[0]) {
 	if(advances[j].req[1]==A_NONE)
 	  sprintf(buf+strlen(buf), _("Allows %s.\n"), 
@@ -1033,7 +1014,7 @@ static void help_update_terrain(const struct help_item *pitem,
 		       tile_types[i].irrigation_time);
 	    }
 	}
-      else if (tile_types[i].irrigation_result != T_LAST)
+      else if (tile_types[i].irrigation_result != T_NONE)
 	{
 	  sprintf (buf, "%s / %d",
 		   tile_types[tile_types[i].irrigation_result].terrain_name,
@@ -1051,7 +1032,7 @@ static void help_update_terrain(const struct help_item *pitem,
 		       tile_types[i].mining_time);
 	    }
 	}
-      else if (tile_types[i].mining_result != T_LAST)
+      else if (tile_types[i].mining_result != T_NONE)
 	{
 	  sprintf (buf, "%s / %d",
 		   tile_types[tile_types[i].mining_result].terrain_name,
@@ -1059,7 +1040,7 @@ static void help_update_terrain(const struct help_item *pitem,
 	}
       xaw_set_label (help_terrain_mining_result_time_data, buf);
 
-      if (tile_types[i].transform_result != T_LAST)
+      if (tile_types[i].transform_result != T_NONE)
 	{
 	  sprintf (buf, "%s / %d",
 		   tile_types[tile_types[i].transform_result].terrain_name,
@@ -1099,7 +1080,9 @@ static void help_update_dialog(const struct help_item *pitem)
 
   /* figure out what kind of item is required for pitem ingo */
 
-  for(top=pitem->topic; *top==' '; ++top);
+  for (top = pitem->topic; *top == ' '; top++) {
+    /* nothing */
+  }
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
@@ -1153,7 +1136,7 @@ static int help_tree_destroy_children(Widget w)
 		XtNnumChildren, &cnt,
 		NULL);
 
-  for(; cnt>0; --cnt, ++children) {
+  for (; cnt > 0; cnt--, children++) {
     if(XtIsSubclass(*children, commandWidgetClass)) {
       Widget par;
       XtVaGetValues(*children, XtNtreeParent, &par, NULL);
