@@ -17,31 +17,23 @@
 
 #include "mapview_g.h"
 
+#include "citydlg_common.h"
+
 #include "graphics.h"
 
 struct unit;
 struct city;
 
 Pixmap get_thumb_pixmap(int onoff);
-Pixmap get_citizen_pixmap(int frame);
-
-void put_unit_pixmap(struct unit *punit, Pixmap pm, int xtile, int ytile);
+Pixmap get_citizen_pixmap(struct citizen_type type, int cnum,
+			  struct city *pcity);
 
 void put_unit_pixmap_city_overlays(struct unit *punit, Pixmap pm);
 
-void put_city_tile_output(Pixmap pm, int x, int y, 
-			  int food, int shield, int trade);
-
 void overview_canvas_expose(Widget w, XEvent *event, Region exposed, 
 			    void *client_data);
 void map_canvas_expose(Widget w, XEvent *event, Region exposed, 
 		       void *client_data);
-void map_canvas_resize(void);
-
-void pixmap_put_tile(Pixmap pm, int x, int y, int map_x, int map_y, 
-		     int citymode);
-void pixmap_put_black_tile(Pixmap pm, int x, int y);
-void pixmap_frame_tile_red(Pixmap pm, int x, int y);
 
 void scrollbar_jump_callback(Widget scrollbar, XtPointer client_data,
 			     XtPointer percent_ptr);
