@@ -10,13 +10,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -24,7 +24,6 @@
 #include <X11/Xaw/Label.h>
 #include <X11/Xaw/Command.h>
 #include <X11/Xaw/SimpleMenu.h>
-#include <X11/Xaw/Command.h>
 #include <X11/Xaw/List.h>
 #include <X11/Xaw/Viewport.h>
 #include <X11/Xaw/AsciiText.h>  
@@ -35,7 +34,6 @@
 
 #include "fcintl.h"
 #include "game.h"
-#include "genlist.h"
 #include "map.h"
 #include "mem.h"
 #include "packets.h"
@@ -55,7 +53,9 @@
 #include "mapctrl.h"
 #include "mapview.h"
 #include "repodlgs.h"
+#include "text.h"
 #include "tilespec.h"
+#include "climisc.h"
 
 #include "spaceshipdlg.h"
 
@@ -70,8 +70,16 @@ struct spaceship_dialog {
   Widget close_command;
 };
 
-static struct genlist dialog_list;
-static int dialog_list_has_been_initialised;
+#define SPECLIST_TAG dialog
+#define SPECLIST_TYPE struct spaceship_dialog
+#include "speclist.h"
+
+#define dialog_list_iterate(dialoglist, pdialog) \
+    TYPED_LIST_ITERATE(struct spaceship_dialog, dialoglist, pdialog)
+#define dialog_list_iterate_end  LIST_ITERATE_END
+
+static struct dialog_list dialog_list;
+static bool dialog_list_has_been_initialised = FALSE;
 
 struct spaceship_dialog *get_spaceship_dialog(struct player *pplayer);
 struct spaceship_dialog *create_spaceship_dialog(struct player *pplayer);
@@ -88,20 +96,18 @@ void spaceship_launch_callback(Widget w, XtPointer client_data, XtPointer call_d
 *****************************************************************/
 struct spaceship_dialog *get_spaceship_dialog(struct player *pplayer)
 {
-  struct genlist_iterator myiter;
-
-  if(!dialog_list_has_been_initialised) {
-    genlist_init(&dialog_list);
-    dialog_list_has_been_initialised=1;
+  if (!dialog_list_has_been_initialised) {
+    dialog_list_init(&dialog_list);
+    dialog_list_has_been_initialised = TRUE;
   }
-  
-  genlist_iterator_init(&myiter, &dialog_list, 0);
-    
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter))
-    if(((struct spaceship_dialog *)ITERATOR_PTR(myiter))->pplayer==pplayer)
-      return ITERATOR_PTR(myiter);
 
-  return 0;
+  dialog_list_iterate(dialog_list, pdialog) {
+    if (pdialog->pplayer == pplayer) {
+      return pdialog;
+    }
+  } dialog_list_iterate_end;
+
+  return NULL;
 }
 
 /****************************************************************
@@ -230,7 +236,7 @@ struct spaceship_dialog *create_spaceship_dialog(struct player *pplayer)
   XtAddCallback(pdialog->close_command, XtNcallback, spaceship_close_callback,
 		(XtPointer)pdialog);
 
-  genlist_insert(&dialog_list, pdialog, 0);
+  dialog_list_insert(&dialog_list, pdialog);
 
   XtRealizeWidget(pdialog->shell);
 
@@ -250,40 +256,8 @@ struct spaceship_dialog *create_spaceship_dialog(struct player *pplayer)
 *****************************************************************/
 void spaceship_dialog_update_info(struct spaceship_dialog *pdialog)
 {
-  char buf[512], arrival[16] = "-   ", travel_buf[100];
-  struct player_spaceship *pship=&(pdialog->pplayer->spaceship);
-
-  if (pship->propulsion) {
-    my_snprintf(travel_buf, sizeof(travel_buf),
-		_("Travel time:     %5.1f years"),
-		(float) (0.1 * ((int) (pship->travel_time * 10.0))));
-  } else {
-    my_snprintf(travel_buf, sizeof(travel_buf),
-		"%s",
-		"Travel time:        N/A     ");
-  }
-
-  if (pship->state == SSHIP_LAUNCHED) {
-    sz_strlcpy(arrival, textyear((int) (pship->launch_year
-					+ (int) pship->travel_time)));
-  }
-  my_snprintf(buf, sizeof(buf),
-	      _("Population:      %5d\n"
-		"Support:         %5d %%\n"
-		"Energy:          %5d %%\n"
-		"Mass:            %5d tons\n"
-		"%s\n"
-		"Success prob.:   %5d %%\n"
-		"Year of arrival: %8s"),
-	      pship->population,
-	      (int) (pship->support_rate * 100.0),
-	      (int) (pship->energy_rate * 100.0),
-	      pship->mass,
-	      travel_buf,
-	      (int) (pship->success_rate * 100.0),
-	      arrival);
-
-  xaw_set_label(pdialog->info_label, buf);
+  xaw_set_label(pdialog->info_label,
+		get_spaceship_descr(&pdialog->pplayer->spaceship));
 }
 
 /****************************************************************
@@ -370,7 +344,7 @@ void spaceship_dialog_update_image(struct spaceship_dialog *pdialog)
 void close_spaceship_dialog(struct spaceship_dialog *pdialog)
 {
   XtDestroyWidget(pdialog->shell);
-  genlist_unlink(&dialog_list, pdialog);
+  dialog_list_unlink(&dialog_list, pdialog);
 
   free(pdialog);
 }
@@ -388,14 +362,12 @@ void spaceshipdlg_key_close(Widget w)
 *****************************************************************/
 void spaceshipdlg_msg_close(Widget w)
 {
-  struct genlist_iterator myiter;
-
-  genlist_iterator_init(&myiter, &dialog_list, 0);
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter))
-    if(((struct spaceship_dialog *)ITERATOR_PTR(myiter))->shell==w) {
-      close_spaceship_dialog((struct spaceship_dialog *)ITERATOR_PTR(myiter));
+  dialog_list_iterate(dialog_list, pdialog) {
+    if (pdialog->shell == w) {
+      close_spaceship_dialog(pdialog);
       return;
     }
+  } dialog_list_iterate_end;
 }
 
 
@@ -411,12 +383,9 @@ void spaceship_close_callback(Widget w, XtPointer client_data, XtPointer call_da
 /****************************************************************
 ...
 *****************************************************************/
-void spaceship_launch_callback(Widget w, XtPointer client_data, XtPointer call_data)
+void spaceship_launch_callback(Widget w, XtPointer client_data,
+			       XtPointer call_data)
 {
-  struct packet_spaceship_action packet;
-
-  packet.action = SSHIP_ACT_LAUNCH;
-  packet.num = 0;
-  send_packet_spaceship_action(&aconnection, &packet);
+  send_packet_spaceship_launch(&aconnection);
   /* close_spaceship_dialog((struct spaceship_dialog *)client_data); */
 }
