@@ -13,15 +13,15 @@
 #ifndef FC__CITYDLG_G_H
 #define FC__CITYDLG_G_H
 
-struct unit;
-struct city;
+#include "fc_types.h"
 
-void popup_city_dialog(struct city *pcity, int make_modal);
+#include "citydlg_common.h"
+
+void popup_city_dialog(struct city *pcity, bool make_modal);
 void popdown_city_dialog(struct city *pcity);
 void popdown_all_city_dialogs(void);
 void refresh_city_dialog(struct city *pcity);
 void refresh_unit_city_dialogs(struct unit *punit);
-int city_dialog_is_open(struct city *pcity);
+bool city_dialog_is_open(struct city *pcity);
 
 #endif  /* FC__CITYDLG_G_H */
-
