@@ -15,10 +15,14 @@
 
 #include "shared.h"		/* bool type */
 
-struct city;
+#include "fc_types.h"
 
 void popup_city_report_dialog(bool make_modal);
 void city_report_dialog_update(void);
 void city_report_dialog_update_city(struct city *pcity);
 
+/* Area Selection on canvas */
+void hilite_cities_from_canvas(void);
+void toggle_city_hilite(struct city *pcity, bool on_off);
+
 #endif  /* FC__CITYREP_G_H */
