@@ -13,6 +13,10 @@
 #ifndef FC__CONNECTDLG_G_H
 #define FC__CONNECTDLG_G_H
 
+void close_connection_dialog(void);
+void really_close_connection_dialog(void);
+
 void gui_server_connect(void);
+void server_autoconnect(void);
 
 #endif  /* FC__CONNECTDLG_G_H */
