@@ -13,11 +13,11 @@
 #ifndef FC__REPODLGS_G_H
 #define FC__REPODLGS_G_H
 
+#include "packets.h"         
 #include "shared.h"		/* bool type */
 
 #include "repodlgs_common.h"
 
-char *get_report_title(char *report_name);
 void update_report_dialogs(void);
 
 void science_dialog_update(void);
@@ -26,5 +26,7 @@ void economy_report_dialog_update(void);
 void popup_economy_report_dialog(bool make_modal);
 void activeunits_report_dialog_update(void);
 void popup_activeunits_report_dialog(bool make_modal);
+void popup_endgame_report_dialog(struct packet_endgame_report *packet);
+void popup_settable_options_dialog(void);
 
 #endif  /* FC__REPODLGS_G_H */
