@@ -10,42 +10,43 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <assert.h>
-#include <ctype.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "audio.h"
-#include "clinet.h"
 #include "events.h"
 #include "fcintl.h"
-#include "log.h"
 #include "game.h"
+#include "log.h"
+#include "mem.h"
 #include "registry.h"
 #include "shared.h"
 #include "support.h"
-#include "tilespec.h"
 #include "version.h"
-#include "mem.h"
 
+#include "audio.h"
 #include "chatline_g.h"
-#include "cma_fec.h"
 #include "cityrepdata.h"
+#include "civclient.h"
+#include "clinet.h"
+#include "cma_fec.h"
+#include "plrdlg_common.h"
+#include "tilespec.h"
 
 #include "options.h"
  
 /** Defaults for options normally on command line **/
 
-char default_player_name[512] = "\0";
+char default_user_name[512] = "\0";
 char default_server_host[512] = "localhost";
 int  default_server_port = DEFAULT_SOCK_PORT;
 char default_metaserver[512] = METALIST_ADDR;
-char default_tile_set_name[512] = "\0";
+char default_tileset_name[512] = "\0";
 char default_sound_set_name[512] = "stdsounds";
 char default_sound_plugin_name[512] = "\0";
 
@@ -53,73 +54,73 @@ char default_sound_plugin_name[512] = "\0";
 
 bool solid_color_behind_units = FALSE;
 bool sound_bell_at_new_turn = FALSE;
-bool smooth_move_units = TRUE;
-int  smooth_move_unit_steps = 3;
+int  smooth_move_unit_msec = 30;
+int smooth_center_slide_msec = 200;
 bool do_combat_animation = TRUE;
 bool ai_popup_windows = FALSE;
-bool ai_manual_turn_done = FALSE;
+bool ai_manual_turn_done = TRUE;
 bool auto_center_on_unit = TRUE;
 bool auto_center_on_combat = FALSE;
 bool wakeup_focus = TRUE;
-bool draw_diagonal_roads = TRUE;
 bool center_when_popup_city = TRUE;
 bool concise_city_production = FALSE;
 bool auto_turn_done = FALSE;
 bool meta_accelerators = TRUE;
 bool map_scrollbars = TRUE;
 bool dialogs_on_top = TRUE;
+bool ask_city_name = TRUE;
+bool popup_new_cities = TRUE;
+bool keyboardless_goto = TRUE;
+bool show_task_icons = TRUE;
 
-#define GEN_INT_OPTION(name, desc) { #name, desc, COT_INT, \
-                                     &name, NULL, NULL, 0, NULL, NULL }
-#define GEN_BOOL_OPTION(name, desc) { #name, desc, COT_BOOL, \
-                                      NULL, &name, NULL, 0, NULL, NULL }
-#define GEN_STR_OPTION(name, desc, dflt) { #name, desc, COT_STR, \
-                                     NULL, NULL, name, sizeof(name), \
-                                     dflt, NULL }
-#define GEN_OPTION_TERMINATOR { NULL, NULL, COT_BOOL, \
-                                NULL, NULL, NULL, 0, NULL }
+/* This option is currently set by the client - not by the user. */
+bool update_city_text_in_refresh_tile = TRUE;
 
-client_option options[] = {
-  GEN_STR_OPTION(default_player_name,       N_("Default player's username"),
-		 NULL), 
-  GEN_STR_OPTION(default_server_host,       N_("Default server"), NULL),
+static client_option common_options[] = {
+  GEN_STR_OPTION(default_user_name,        N_("Default player's login name"),
+		 NULL, NULL), 
+  GEN_STR_OPTION(default_server_host,       N_("Default server"),
+		 NULL, NULL),
   GEN_INT_OPTION(default_server_port,       N_("Default server's port")),
-  GEN_STR_OPTION(default_metaserver,        N_("Default metaserver"), NULL),
-  GEN_STR_OPTION(default_tile_set_name,     N_("Default tileset"),
-		 get_tileset_list),
+  GEN_STR_OPTION(default_metaserver,        N_("Default metaserver"),
+		 NULL, NULL),
   GEN_STR_OPTION(default_sound_set_name,    N_("Default name of sound set"),
-		 get_soundset_list),
+		 get_soundset_list, NULL),
   GEN_STR_OPTION(default_sound_plugin_name, N_("Default sound plugin"),
-		 get_soundplugin_list),
+		 get_soundplugin_list, NULL),
+  GEN_STR_OPTION(default_tileset_name,     N_("Tileset"),
+		 get_tileset_list, tilespec_reread_callback),
 
   GEN_BOOL_OPTION(solid_color_behind_units, N_("Solid unit background color")),
   GEN_BOOL_OPTION(sound_bell_at_new_turn,   N_("Sound bell at new turn")),
-  GEN_BOOL_OPTION(smooth_move_units,        N_("Smooth unit moves")),
-  GEN_INT_OPTION(smooth_move_unit_steps,    N_("Smooth unit move steps")),
+  GEN_INT_OPTION(smooth_move_unit_msec,
+		 N_("Unit movement animation time (milliseconds)")),
+  GEN_INT_OPTION(smooth_center_slide_msec,
+		 N_("Mapview recentering time (milliseconds)")),
   GEN_BOOL_OPTION(do_combat_animation,      N_("Show combat animation")),
   GEN_BOOL_OPTION(ai_popup_windows,         N_("Popup dialogs in AI Mode")),
   GEN_BOOL_OPTION(ai_manual_turn_done,      N_("Manual Turn Done in AI Mode")),
   GEN_BOOL_OPTION(auto_center_on_unit,      N_("Auto Center on Units")),
   GEN_BOOL_OPTION(auto_center_on_combat,    N_("Auto Center on Combat")),
   GEN_BOOL_OPTION(wakeup_focus,             N_("Focus on Awakened Units")),
-  GEN_BOOL_OPTION(draw_diagonal_roads,      N_("Draw Diagonal Roads/Rails")),
   GEN_BOOL_OPTION(center_when_popup_city,   N_("Center map when Popup city")),
   GEN_BOOL_OPTION(concise_city_production,  N_("Concise City Production")),
   GEN_BOOL_OPTION(auto_turn_done,           N_("End Turn when done moving")),
-  GEN_BOOL_OPTION(meta_accelerators,        N_("Use Alt/Meta for accelerators (GTK+ only)")),
-  GEN_BOOL_OPTION(map_scrollbars,	    N_("Show Map Scrollbars (GTK+ only)")),
-  GEN_BOOL_OPTION(dialogs_on_top,	    N_("Keep dialogs on top (GTK+ 2.0 only)")),
-  GEN_OPTION_TERMINATOR
+  GEN_BOOL_OPTION(ask_city_name,            N_("Prompt for city names")),
+  GEN_BOOL_OPTION(popup_new_cities,         N_("Pop up city dialog for new cities")),
 };
 #undef GEN_INT_OPTION
 #undef GEN_BOOL_OPTION
 #undef GEN_STR_OPTION
-#undef GEN_OPTION_TERMINATOR
+
+int num_options;
+client_option *options;
 
 /** View Options: **/
 
 bool draw_map_grid = FALSE;
 bool draw_city_names = TRUE;
+bool draw_city_growth = TRUE;
 bool draw_city_productions = FALSE;
 bool draw_terrain = TRUE;
 bool draw_coastline = FALSE;
@@ -133,6 +134,7 @@ bool draw_cities = TRUE;
 bool draw_units = TRUE;
 bool draw_focus_unit = FALSE;
 bool draw_fog_of_war = TRUE;
+bool draw_borders = TRUE;
 
 #define VIEW_OPTION(name) { #name, &name }
 #define VIEW_OPTION_TERMINATOR { NULL, NULL }
@@ -140,6 +142,7 @@ bool draw_fog_of_war = TRUE;
 view_option view_options[] = {
   VIEW_OPTION(draw_map_grid),
   VIEW_OPTION(draw_city_names),
+  VIEW_OPTION(draw_city_growth),
   VIEW_OPTION(draw_city_productions),
   VIEW_OPTION(draw_terrain),
   VIEW_OPTION(draw_coastline),
@@ -153,6 +156,7 @@ view_option view_options[] = {
   VIEW_OPTION(draw_units),
   VIEW_OPTION(draw_focus_unit),
   VIEW_OPTION(draw_fog_of_war),
+  VIEW_OPTION(draw_borders),
   VIEW_OPTION_TERMINATOR
 };
 
@@ -172,7 +176,10 @@ int sorted_events[E_LAST];
  * to be sorted.
  */
 static struct {
-  char *enum_name, *tag_name, *descr_orig, *descr;
+  const char *enum_name;
+  char *tag_name;
+  const char *descr_orig;
+  const char *descr;
   enum event_type event;
 } events[] = {
   GEN_EV(N_("City: Building Unavailable Item"),       E_CITY_CANTBUILD),
@@ -252,16 +259,21 @@ static struct {
   GEN_EV(N_("Unit: Attack Failed"),                   E_UNIT_LOST_ATT),
   GEN_EV(N_("Unit: Attack Succeeded"),                E_UNIT_WIN_ATT),
   GEN_EV(N_("Unit: Bought"),                          E_UNIT_BUY),
-  GEN_EV(N_("Unit: Built"),                           E_UNIT_BUILD),
+  GEN_EV(N_("Unit: Built"),                           E_UNIT_BUILT),
   GEN_EV(N_("Unit: Defender Destroyed"),              E_UNIT_LOST),
   GEN_EV(N_("Unit: Defender Survived"),               E_UNIT_WIN),
+  GEN_EV(N_("Unit: Became More Veteran"),             E_UNIT_BECAME_VET),
   GEN_EV(N_("Unit: Production Upgraded"),             E_UNIT_UPGRADED),
   GEN_EV(N_("Unit: Relocated"),                       E_UNIT_RELOCATED),
+  GEN_EV(N_("Unit: Orders / goto events"),            E_UNIT_ORDERS),
   GEN_EV(N_("Wonder: Finished"),                      E_WONDER_BUILD),
   GEN_EV(N_("Wonder: Made Obsolete"),                 E_WONDER_OBSOLETE),
   GEN_EV(N_("Wonder: Started"),                       E_WONDER_STARTED),
   GEN_EV(N_("Wonder: Stopped"),                       E_WONDER_STOPPED),
   GEN_EV(N_("Wonder: Will Finish Next Turn"),         E_WONDER_WILL_BE_BUILT),
+  GEN_EV(N_("Diplomatic Message"),                    E_DIPLOMACY),
+  GEN_EV(N_("City: Production changed"),              E_CITY_PRODUCTION_CHANGED),
+  GEN_EV(N_("Treaty: Embassy"),                       E_TREATY_EMBASSY),
   GEN_EV_TERMINATOR
 };
 
@@ -272,20 +284,20 @@ static struct {
 static int event_to_index[E_LAST];
 
 static void save_cma_preset(struct section_file *file, char *name,
-			    const struct cma_parameter *const pparam,
+			    const struct cm_parameter *const pparam,
 			    int inx);
 static void load_cma_preset(struct section_file *file, int inx);
 
-static void save_global_worklist(struct section_file *file, char *path, 
+static void save_global_worklist(struct section_file *file, const char *path, 
                                  int wlinx, struct worklist *pwl);
 
-static void load_global_worklist(struct section_file *file, char *path, 
-                                 int wlinx, struct worklist *pwl);
+static void load_global_worklist(struct section_file *file, const char *path,
+				 int wlinx, struct worklist *pwl);
 
 /**************************************************************************
   Returns the translated description of the given event.
 **************************************************************************/
-const char *const get_message_text(enum event_type event)
+const char *get_message_text(enum event_type event)
 {
   assert(event >= 0 && event < E_LAST);
 
@@ -318,18 +330,19 @@ void init_messages_where(void)
   int out_only[] = { E_IMP_BUY, E_IMP_SOLD, E_UNIT_BUY,
 		     E_UNIT_LOST_ATT, E_UNIT_WIN_ATT, E_GAME_START,
 		     E_NATION_SELECTED, E_CITY_BUILD, E_NEXT_YEAR,
-		     E_TECH_LEARNED};
+		     E_CITY_PRODUCTION_CHANGED,
+		     E_CITY_MAY_SOON_GROW, E_WORKLIST};
   int all[] = { E_MESSAGE_WALL };
   int i;
 
   for(i=0; i<E_LAST; i++) {
-    messages_where[i] = MW_OUTPUT | MW_MESSAGES;
+    messages_where[i] = MW_MESSAGES;
   }
   for (i = 0; i < ARRAY_SIZE(out_only); i++) {
-    messages_where[out_only[i]] = MW_OUTPUT;
+    messages_where[out_only[i]] = 0;
   }
   for (i = 0; i < ARRAY_SIZE(all); i++) {
-    messages_where[all[i]] = MW_OUTPUT | MW_MESSAGES | MW_POPUP;
+    messages_where[all[i]] = MW_MESSAGES | MW_POPUP;
   }
   
   for (i = 0; i < ARRAY_SIZE(event_to_index); i++) {
@@ -409,18 +422,32 @@ void load_general_options(void)
   const char * const prefix = "client";
   char *name;
   int i, num;
-  client_option *o;
   view_option *v;
 
+  assert(options == NULL);
+  num_options = ARRAY_SIZE(common_options) + num_gui_options;
+  options = fc_malloc(num_options * sizeof(*options));
+  memcpy(options, common_options, sizeof(common_options));
+  memcpy(options + ARRAY_SIZE(common_options), gui_options,
+	 num_gui_options * sizeof(*options));
+
   name = option_file_name();
   if (!name) {
     /* fail silently */
     return;
   }
-  if (!section_file_load(&sf, name))
+  if (!section_file_load(&sf, name)) {
+    create_default_cma_presets();
     return;  
+  }
+
+  /* a "secret" option for the lazy. TODO: make this saveable */
+  sz_strlcpy(password, 
+             secfile_lookup_str_default(&sf, "", "%s.password", prefix));
+
+  for (i = 0; i < num_options; i++) {
+    client_option *o = options + i;
 
-  for (o = options; o->name; o++) {
     switch (o->type) {
     case COT_BOOL:
       *(o->p_bool_value) =
@@ -454,12 +481,22 @@ void load_general_options(void)
     *ip = secfile_lookup_bool_default(&sf, *ip, "%s.city_report_%s", prefix,
 				     city_report_spec_tagname(i));
   }
+  
+  for(i = 1; i < num_player_dlg_columns; i++) {
+    bool *show = &(player_dlg_columns[i].show);
+    *show = secfile_lookup_bool_default(&sf, *show, "%s.player_dlg_%s", prefix,
+                                        player_dlg_columns[i].tagname);
+  }
 
   /* Load cma presets. If cma.number_of_presets doesn't exist, don't load 
    * any, the order here should be reversed to keep the order the same */
-  num = secfile_lookup_int_default(&sf, 0, "cma.number_of_presets");
-  for (i = num - 1; i >= 0; i--) {
-    load_cma_preset(&sf, i);
+  num = secfile_lookup_int_default(&sf, -1, "cma.number_of_presets");
+  if (num == -1) {
+    create_default_cma_presets();
+  } else {
+    for (i = num - 1; i >= 0; i--) {
+      load_cma_preset(&sf, i);
+    }
   }
  
   section_file_free(&sf);
@@ -487,8 +524,8 @@ void load_ruleset_specific_options(void)
   /* load global worklists */
   for (i = 0; i < MAX_NUM_WORKLISTS; i++) {
     game.player_ptr->worklists[i].is_valid =
-                secfile_lookup_int_default(&sf, FALSE,
-                                          "worklists.worklist%d.is_valid", i);
+	secfile_lookup_bool_default(&sf, FALSE,
+				    "worklists.worklist%d.is_valid", i);
     strcpy(game.player_ptr->worklists[i].name,
            secfile_lookup_str_default(&sf, "",
                                       "worklists.worklist%d.name", i));
@@ -505,7 +542,6 @@ void load_ruleset_specific_options(void)
 void save_options(void)
 {
   struct section_file sf;
-  client_option *o;
   char *name = option_file_name();
   char output_buffer[256];
   view_option *v;
@@ -519,7 +555,9 @@ void save_options(void)
   section_file_init(&sf);
   secfile_insert_str(&sf, VERSION_STRING, "client.version");
 
-  for (o = options; o->name; o++) {
+  for (i = 0; i < num_options; i++) {
+    client_option *o = options + i;
+
     switch (o->type) {
     case COT_BOOL:
       secfile_insert_bool(&sf, *(o->p_bool_value), "client.%s", o->name);
@@ -548,12 +586,18 @@ void save_options(void)
 		       "client.city_report_%s",
 		       city_report_spec_tagname(i));
   }
+  
+  for (i = 1; i < num_player_dlg_columns; i++) {
+    secfile_insert_bool(&sf, player_dlg_columns[i].show,
+                        "client.player_dlg_%s",
+                        player_dlg_columns[i].tagname);
+  }
 
   /* insert global worklists */
   for(i = 0; i < MAX_NUM_WORKLISTS; i++){
     if (game.player_ptr->worklists[i].is_valid) {
-      secfile_insert_int(&sf, game.player_ptr->worklists[i].is_valid,
-                         "worklists.worklist%d.is_valid", i);
+      secfile_insert_bool(&sf, game.player_ptr->worklists[i].is_valid,
+			  "worklists.worklist%d.is_valid", i);
       secfile_insert_str(&sf, game.player_ptr->worklists[i].name,
                          "worklists.worklist%d.name", i);
       save_global_worklist(&sf, "worklists.worklist%d", i, 
@@ -590,8 +634,8 @@ void save_options(void)
 *****************************************************************/
 static void load_cma_preset(struct section_file *file, int inx)
 {
-  struct cma_parameter parameter;
-  char *name;
+  struct cm_parameter parameter;
+  const char *name;
   int i;
 
   name = secfile_lookup_str_default(file, "preset", 
@@ -603,11 +647,11 @@ static void load_cma_preset(struct section_file *file, int inx)
 	secfile_lookup_int_default(file, 0, "cma.preset%d.factor%d", inx, i);
   }
   parameter.require_happy =
-      secfile_lookup_int_default(file, 0, "cma.preset%d.reqhappy", inx);
-  parameter.factor_target =
-      secfile_lookup_int_default(file, 0, "cma.preset%d.factortarget", inx);
+      secfile_lookup_bool_default(file, FALSE, "cma.preset%d.reqhappy", inx);
   parameter.happy_factor =
       secfile_lookup_int_default(file, 0, "cma.preset%d.happyfactor", inx);
+  parameter.allow_disorder = FALSE;
+  parameter.allow_specialists = TRUE;
 
   cmafec_preset_add(name, &parameter);
 }
@@ -616,7 +660,7 @@ static void load_cma_preset(struct section_file *file, int inx)
  Does heavy lifting for inserting a preset.
 *****************************************************************/
 static void save_cma_preset(struct section_file *file, char *name,
-			    const struct cma_parameter *const pparam,
+			    const struct cm_parameter *const pparam,
 			    int inx)
 {
   int i;
@@ -628,10 +672,8 @@ static void save_cma_preset(struct section_file *file, char *name,
     secfile_insert_int(file, pparam->factor[i],
 		       "cma.preset%d.factor%d", inx, i);
   }
-  secfile_insert_int(file, pparam->require_happy,
-		     "cma.preset%d.reqhappy", inx);
-  secfile_insert_int(file, pparam->factor_target,
-		     "cma.preset%d.factortarget", inx);
+  secfile_insert_bool(file, pparam->require_happy,
+		      "cma.preset%d.reqhappy", inx);
   secfile_insert_int(file, pparam->happy_factor,
 		     "cma.preset%d.happyfactor", inx);
 }
@@ -639,7 +681,7 @@ static void save_cma_preset(struct section_file *file, char *name,
 /****************************************************************
 ... 
 *****************************************************************/
-const char *const get_sound_tag_for_event(enum event_type event)
+const char *get_sound_tag_for_event(enum event_type event)
 {
   if (event == E_NOEVENT) {
     return NULL;
@@ -657,8 +699,8 @@ const char *const get_sound_tag_for_event(enum event_type event)
 /****************************************************************
  loads global worklist from rc file
 *****************************************************************/
-static void load_global_worklist(struct section_file *file, char *path, 
-                                  int wlinx, struct worklist *pwl)
+static void load_global_worklist(struct section_file *file, const char *path,
+				 int wlinx, struct worklist *pwl)
 {
   char efpath[64];
   char idpath[64];
@@ -696,8 +738,8 @@ static void load_global_worklist(struct section_file *file, char *path,
 /****************************************************************
  saves global worklist to rc file
 *****************************************************************/
-static void save_global_worklist(struct section_file *file, char *path, 
-                                  int wlinx, struct worklist *pwl)
+static void save_global_worklist(struct section_file *file, const char *path,
+				 int wlinx, struct worklist *pwl)
 {
   char efpath[64];
   char idpath[64];
