@@ -14,23 +14,33 @@
 #define FC__CONNECTION_H
 
 #include <time.h>	/* time_t */
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 
+#define USE_COMPRESSION
+
 /**************************************************************************
   The connection struct and related stuff.
   Includes cmdlevel stuff, which is connection-based.
 ***************************************************************************/
 
-#include "shared.h"		/* MAX_LEN_ADDR */
-#include "shared.h"		/* bool type */
+#include "shared.h"		/* MAX_LEN_ADDR, bool type */
 
-struct player;
+#include "fc_types.h"
+
+struct hash_table;
+struct timer_list;
 
 #define MAX_LEN_PACKET   4096
-#define MAX_LEN_CAPSTR    512
+
 #define MAX_LEN_BUFFER   (MAX_LEN_PACKET * 128)
+#define MAX_LEN_CAPSTR    512
+#define MAX_LEN_PASSWORD  512 /* do not change this under any circumstances */
 
 /**************************************************************************
   Command access levels for client-side use; at present, they are only
@@ -51,9 +61,18 @@ enum cmdlevel_id {    /* access levels for users to issue commands        */
 
 /***************************************************************************
   On the distinction between nations(formerly races), players, and users,
-  see freeciv_hackers_guide.txt
+  see doc/HACKING
 ***************************************************************************/
 
+/* where the connection is in the authentication process */
+enum auth_status {
+  AS_NOT_ESTABLISHED = 0,
+  AS_FAILED,
+  AS_REQUESTING_NEW_PASS,
+  AS_REQUESTING_OLD_PASS,
+  AS_ESTABLISHED
+};
+
 /* get 'struct conn_list' and related functions: */
 /* do this with forward definition of struct connection, so that
  * connection struct can contain a struct conn_list */
@@ -77,6 +96,10 @@ struct socket_packet_buffer {
   unsigned char *data;
 };
 
+#define SPECVEC_TAG byte
+#define SPECVEC_TYPE unsigned char
+#include "specvec.h"
+
 /***********************************************************
   The connection struct represents a single client or server
   at the other end of a network connection.
@@ -86,40 +109,44 @@ struct connection {
   int sock;
   bool used;
   bool established;		/* have negotiated initial packets */
-  bool first_packet;		/* check byte order on first packet */
-  bool byte_swap;		/* connection uses non-network byte order */
   struct player *player;	/* NULL for connections not yet associated
 				   with a specific player */
-  bool observer;		/* connection is "observer", not controller;
- 				   may be observing specific player, or all
- 				   (implementation incomplete) */
+  /* 
+   * connection is "observer", not controller; may be observing
+   * specific player, or all (implementation incomplete).
+   */
+  bool observer;
   struct socket_packet_buffer *buffer;
   struct socket_packet_buffer *send_buffer;
   time_t last_write;
-  bool ponged;		        /* have received a PACKET_CONN_PONG? */
 
-  struct conn_list self;	/* list with this connection as single element */
-  char name[MAX_LEN_NAME];
+  double ping_time;
+  
+  struct conn_list self;     /* list with this connection as single element */
+  char username[MAX_LEN_NAME];
   char addr[MAX_LEN_ADDR];
-  char capability[MAX_LEN_CAPSTR];
-  /* "capability" gives the capability string of the executable (be it
+
+  /* 
+   * "capability" gives the capability string of the executable (be it
    * a client or server) at the other end of the connection.
    */
-  enum cmdlevel_id access_level;
-  /* Used in the server, "access_level" stores the access granted
-   * to the client corresponding to this connection.
-   */
-  struct map_position *route;
-  int route_length;
-  /* These are used when recieving goto routes; they are send split, and in
-     the time where the route is partially recieved it is stored here. */
+  char capability[MAX_LEN_CAPSTR];
 
+  /* 
+   * "access_level" stores the access granted to the client
+   * corresponding to this connection.
+   */
+  enum cmdlevel_id access_level;
+
+  /* 
+   * Something has occurred that means the connection should be
+   * closed, but the closing has been postponed. 
+   */
   bool delayed_disconnect;
-  /* Something has occured that means the connection should be closed, but
-     the closing has been postponed. */
 
   void (*notify_of_writable_data) (struct connection * pc,
 				   bool data_available_and_socket_full);
+
   struct {
     /* 
      * Increases for every packet send to the server.
@@ -149,6 +176,26 @@ struct connection {
      * Will increase for every received packet.
      */
     int last_request_id_seen;
+
+    /* 
+     * The start times of the PACKET_CONN_PING which have been sent
+     * but weren't PACKET_CONN_PONGed yet? 
+     */
+    struct timer_list *ping_timers;
+   
+    /* Holds number of tries for authentication from client. */
+    int authentication_tries;
+
+    /* the time that the server will reply after receiving an auth reply.
+     * this is used to throttle the connection. */
+    time_t authentication_stop;
+
+    /* used to follow where the connection is in the authentication process */
+    enum auth_status status;
+    char password[MAX_LEN_PASSWORD];
+
+    /* for reverse lookup and blacklisting in db */
+    char ipaddr[MAX_LEN_ADDR];
   } server;
 
   /*
@@ -167,6 +214,22 @@ struct connection {
   void (*outgoing_packet_notify) (struct connection * pc,
 				  int packet_type, int size,
 				  int request_id);
+  struct {
+    struct hash_table **sent;
+    struct hash_table **received;
+    int *variant;
+  } phs;
+
+#ifdef USE_COMPRESSION
+  struct {
+    int frozen_level;
+
+    struct byte_vector queue;
+  } compression;
+#endif
+  struct {
+    int bytes_send;
+  } statistics;
 };
 
 
@@ -176,10 +239,10 @@ enum cmdlevel_id cmdlevel_named(const char *token);
 
 typedef void (*CLOSE_FUN) (struct connection *pc);
 void close_socket_set_callback(CLOSE_FUN fun);
+CLOSE_FUN close_socket_get_callback(void);
 
 int read_socket_data(int sock, struct socket_packet_buffer *buffer);
 void flush_connection_send_buffer_all(struct connection *pc);
-void flush_connection_send_buffer_packets(struct connection *pc);
 void send_connection_data(struct connection *pc, const unsigned char *data,
 			  int len);
 
@@ -189,13 +252,16 @@ void connection_do_unbuffer(struct connection *pc);
 void conn_list_do_buffer(struct conn_list *dest);
 void conn_list_do_unbuffer(struct conn_list *dest);
 
-struct connection *find_conn_by_name(const char *name);
-struct connection *find_conn_by_name_prefix(const char *name,
-					    enum m_pre_result *result);
+struct connection *find_conn_by_user(const char *user_name);
+struct connection *find_conn_by_user_prefix(const char *user_name,
+                                             enum m_pre_result *result);
 struct connection *find_conn_by_id(int id);
 
 struct socket_packet_buffer *new_socket_packet_buffer(void);
-void free_socket_packet_buffer(struct socket_packet_buffer *buf);
+void connection_common_init(struct connection *pconn);
+void connection_common_close(struct connection *pconn);
+void free_compression_queue(struct connection *pconn);
+void conn_clear_packet_cache(struct connection *pconn);
 
 const char *conn_description(const struct connection *pconn);
 
