@@ -14,16 +14,24 @@
 #define FC__GAME_H
 
 #include <time.h>	/* time_t */
+
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
 
-#include "connection.h"		/* struct conn_list */
 #include "shared.h"
+
+#include "connection.h"		/* struct conn_list */
+#include "fc_types.h"
+#include "improvement.h"	/* Impr_Status */
 #include "player.h"
 
-#define MAX_LEN_DEMOGRAPHY  16
-#define MAX_LEN_ALLOW_CONNECT 16
+/* Changing these will probably break network compatability. */
+#define MAX_LEN_DEMOGRAPHY 16
+#define MAX_LEN_ALLOW_TAKE 16
+#define MAX_ID_LEN 33
+#define MAX_GRANARY_INIS 24
+#define MAX_LEN_STARTUNIT (20+1)
 
 enum server_states { 
   PRE_GAME_STATE, 
@@ -37,12 +45,10 @@ enum client_states {
   CLIENT_PRE_GAME_STATE,
   CLIENT_SELECT_RACE_STATE,
   CLIENT_WAITING_FOR_GAME_START_STATE,
-  CLIENT_GAME_RUNNING_STATE
+  CLIENT_GAME_RUNNING_STATE,
+  CLIENT_GAME_OVER_STATE
 };
 
-struct unit;
-struct city;
-
 #define OVERFLIGHT_NOTHING  1
 #define OVERFLIGHT_FRIGHTEN 2
 
@@ -50,36 +56,48 @@ struct city;
 #define CONTAMINATION_FALLOUT   2
 
 struct civ_game {
-  int is_new_game;		/* 1 for games never started */
+  bool is_new_game;		/* 1 for games never started */
   int version;
+  char id[MAX_ID_LEN];		/* server only */
   int civstyle;
   int gold;
-  int settlers, explorer;
+  char start_units[MAX_LEN_STARTUNIT];
   int dispersion;
   int tech;
   int skill_level;
   int timeout;
+  int timeoutint;     /* increase timeout every N turns... */
+  int timeoutinc;     /* ... by this amount ... */
+  int timeoutincmult; /* ... and multiply timeoutinc by this amount ... */
+  int timeoutintinc;  /* ... and increase timeoutint by this amount */
+  int timeoutcounter; /* timeoutcounter - timeoutint = turns to next inc. */
   int tcptimeout;
   int netwait;
   time_t last_ping;
   int pingtimeout;
+  int pingtime;
   time_t turn_start;
   int end_year;
   int year;
+  int turn;
   int researchcost; /* Multiplier on cost of new research */
   int diplcost, freecost, conquercost;
   int diplchance;
   int cityfactor;
+  int citymindist;
   int civilwarsize;
+  int contactturns;
+  int rapturedelay;
   int min_players, max_players, nplayers;
   int aifill;
+  int notradesize, fulltradesize;
   int barbarianrate;
   int onsetbarbarian;
   int nbarbarians;
   int occupychance;
   int unhappysize;
+  bool angrycitizen;
   char *startmessage;
-  int conn_id;			/* client-only: id client known to server as */
   int player_idx;
   struct player *player_ptr;
   struct player players[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
@@ -90,6 +108,7 @@ struct civ_game {
   int global_wonders[B_LAST];              /* contains city id's */
          /* global_wonders[] may also be (-1), or the id of a city
 	    which no longer exists, if the wonder has been destroyed */
+  Impr_Status improvements[B_LAST];        /* impr. with equiv_range==World */
 
   int heating; /* Number of polluted squares. */
   int globalwarming; /* Total damage done. (counts towards a warming event.) */
@@ -109,17 +128,18 @@ struct civ_game {
   int killcitizen;
   int techpenalty;
   int razechance;
-  int scorelog;
-  int randseed;
+  bool scorelog;
+  int seed;
   int aqueduct_size;
-  int sewer_size;
   int add_to_size_limit;
-  int spacerace;
-  int turnblock;
-  int fixedlength;
-  int auto_ai_toggle;
-  int fogofwar;
-  int fogofwar_old;	/* as the fog_of_war bit get changed by setting
+  bool savepalace;
+  bool natural_city_names;
+  bool spacerace;
+  bool turnblock;
+  bool fixedlength;
+  bool auto_ai_toggle;
+  bool fogofwar;
+  bool fogofwar_old;	/* as the fog_of_war bit get changed by setting
 			   the server we need to remember the old setting */
 
   int num_unit_types;
@@ -135,19 +155,25 @@ struct civ_game {
   int playable_nation_count;
   int styles_count;
 
-  struct {
-    char techs[MAX_LEN_NAME];
-    char units[MAX_LEN_NAME];
-    char buildings[MAX_LEN_NAME];
-    char terrain[MAX_LEN_NAME];
-    char governments[MAX_LEN_NAME];
-    char nations[MAX_LEN_NAME];
-    char cities[MAX_LEN_NAME];
-    char game[MAX_LEN_NAME];
-  } ruleset;
+  int terrain_count;
+
+  int watchtower_extra_vision;
+  int watchtower_vision;
+  int allowed_city_names;
+
+  int borders;		/* distance of border from city; 0=disabled. */
+  bool happyborders;
+  int diplomacy;        /* who can do it */
+  bool slow_invasions;  /* land units lose all movement landing on shores */
+
+  char rulesetdir[MAX_LEN_NAME];
   int firepower_factor;		/* See README.rulesets */
+
+  Impr_Type_id default_building;
+  Impr_Type_id palace_building;
+  Impr_Type_id land_defend_building;
+
   struct {
-    int get_bonus_tech;		/* eg Philosophy */
     int cathedral_plus;		/* eg Theology */
     int cathedral_minus;	/* eg Communism */
     int colosseum_plus;		/* eg Electricity */
@@ -166,38 +192,58 @@ struct civ_game {
 
   /* values from game.ruleset */
   struct {
+    struct {
+      char name[MAX_LEN_NAME];
+      int min_size, bonus;
+    } specialists[SP_COUNT];
+    bool changable_tax;
+    int forced_science; /* only relevant if !changable_tax */
+    int forced_luxury;
+    int forced_gold;
     int min_city_center_food;
     int min_city_center_shield;
     int min_city_center_trade;
     int min_dist_bw_cities;
     int init_vis_radius_sq;
     int hut_overflight;
-    int pillage_select;
+    bool pillage_select;
     int nuke_contamination;
-    int granary_food_ini;
+    int granary_food_ini[MAX_GRANARY_INIS];
+    int granary_num_inis;
     int granary_food_inc;
+    int tech_cost_style;
+    int tech_leakage;
+    int tech_cost_double_year;
+
+    /* Items given to all players at game start.  Server only. */
+    int global_init_techs[MAX_NUM_TECH_LIST];
+    int global_init_buildings[MAX_NUM_BUILDING_LIST];
+
+    bool killstack;
   } rgame;
+  
+  struct {
+    int improvement_factor;
+    int unit_factor;
+    int total_factor;
+  } incite_cost;
 
   char demography[MAX_LEN_DEMOGRAPHY];
-  char allow_connect[MAX_LEN_ALLOW_CONNECT];
+  char allow_take[MAX_LEN_ALLOW_TAKE];
 
   /* used by the map editor to control game_save; could be used by the server too */
   struct {
-    int save_random;
-    int save_players;
-    int save_known; /* loading will just reveal the squares around cities and units */
-    int save_starts; /* start positions will be auto generated */
-    int save_private_map; /* FoW map; will be created if not saved */
+    bool save_random;
+    bool save_players;
+    bool save_known; /* loading will just reveal the squares around cities and units */
+    bool save_starts; /* start positions will be auto generated */
+    bool save_private_map; /* FoW map; will be created if not saved */
   } save_options;
-  /* Used by mapeditor to load only the map */
-  struct {
-    int load_random;
-    int load_players;
-    int load_known; /* Only makes sense if the players are loaded. */
-    int load_starts; /* Should be done if the players are loaded. */
-    int load_private_map; /* Only makes sense if the players are loaded. */
-    int load_settings;
-  } load_options;
+
+  int trireme_loss_chance[MAX_VET_LEVELS];
+  int work_veteran_chance[MAX_VET_LEVELS];
+  int veteran_chance[MAX_VET_LEVELS];
+  int revolution_length; /* 0=> random length, else the fixated length */
 };
 
 /* Unused? */
@@ -205,55 +251,57 @@ struct lvldat {
   int advspeed;
 };
 
+/* Server setting types.  Changing these will break network compatability. */
+enum sset_type {
+  SSET_BOOL, SSET_INT, SSET_STRING
+};
+
 void game_init(void);
+void game_free(void);
+void ruleset_data_free(void);
+
 int game_next_year(int);
 void game_advance_year(void);
 
 int civ_population(struct player *pplayer);
-struct city *game_find_city_by_name(char *name);
+struct city *game_find_city_by_name(const char *name);
 
 struct unit *find_unit_by_id(int id);
 struct city *find_city_by_id(int id);
 
-void game_remove_player(int plrno);
-void game_remove_all_players(void);
+void game_remove_player(struct player *pplayer);
 void game_renumber_players(int plrno);
 
-void game_remove_unit(int unit_id);
+void game_remove_unit(struct unit *punit);
 void game_remove_city(struct city *pcity);
-int research_time(struct player *pplayer);
-int total_player_citizens(struct player *pplayer);
-int civ_score(struct player *pplayer);
 void initialize_globals(void);
 
 void translate_data_names(void);
 
 struct player *get_player(int player_id);
+bool is_valid_player_id(int player_id);
 int get_num_human_and_ai_players(void);
 
-extern struct civ_game game;
+const char *population_to_text(int thousand_citizen);
 
-#define GAME_DEFAULT_RANDSEED        0
-#define GAME_MIN_RANDSEED            0
-#define GAME_MAX_RANDSEED            (MAX_UINT32 >> 1)
+extern struct civ_game game;
+extern bool is_server;
+
+#define GAME_DEFAULT_SEED        0
+#define GAME_MIN_SEED            0
+#define GAME_MAX_SEED            (MAX_UINT32 >> 1)
 
 #define GAME_DEFAULT_GOLD        50
 #define GAME_MIN_GOLD            0
 #define GAME_MAX_GOLD            5000
 
-#define GAME_DEFAULT_SETTLERS    2
-#define GAME_MIN_SETTLERS        1
-#define GAME_MAX_SETTLERS        10
-
-#define GAME_DEFAULT_EXPLORER    1
-#define GAME_MIN_EXPLORER        0
-#define GAME_MAX_EXPLORER        10
+#define GAME_DEFAULT_START_UNITS  "ccx"
 
 #define GAME_DEFAULT_DISPERSION  0
 #define GAME_MIN_DISPERSION      0
 #define GAME_MAX_DISPERSION      10
 
-#define GAME_DEFAULT_TECHLEVEL   3
+#define GAME_DEFAULT_TECHLEVEL   0
 #define GAME_MIN_TECHLEVEL       0
 #define GAME_MAX_TECHLEVEL       50
 
@@ -261,6 +309,8 @@ extern struct civ_game game;
 #define GAME_MIN_UNHAPPYSIZE     1
 #define GAME_MAX_UNHAPPYSIZE     6
 
+#define GAME_DEFAULT_ANGRYCITIZEN TRUE
+
 #define GAME_DEFAULT_END_YEAR    2000
 #define GAME_MIN_END_YEAR        GAME_START_YEAR
 #define GAME_MAX_END_YEAR        5000
@@ -277,7 +327,7 @@ extern struct civ_game game;
 #define GAME_MIN_AIFILL              0
 #define GAME_MAX_AIFILL              GAME_MAX_MAX_PLAYERS
 
-#define GAME_DEFAULT_RESEARCHCOST	   10
+#define GAME_DEFAULT_RESEARCHCOST	   20
 #define GAME_MIN_RESEARCHCOST	       4
 #define GAME_MAX_RESEARCHCOST        100
 
@@ -285,9 +335,21 @@ extern struct civ_game game;
 #define GAME_MIN_DIPLCOST            0
 #define GAME_MAX_DIPLCOST            100
 
-#define GAME_DEFAULT_FOGOFWAR        1
-#define GAME_MIN_FOGOFWAR            0
-#define GAME_MAX_FOGOFWAR            1
+#define GAME_DEFAULT_FOGOFWAR        TRUE
+
+/* 0 means no national borders.  Performance dropps quickly as the border
+ * distance increases (o(n^2) or worse). */
+#define GAME_DEFAULT_BORDERS         7
+#define GAME_MIN_BORDERS             0
+#define GAME_MAX_BORDERS             24
+
+#define GAME_DEFAULT_HAPPYBORDERS    TRUE
+
+#define GAME_DEFAULT_SLOW_INVASIONS  TRUE
+
+#define GAME_DEFAULT_DIPLOMACY       0
+#define GAME_MIN_DIPLOMACY           0
+#define GAME_MAX_DIPLOMACY           4
 
 #define GAME_DEFAULT_DIPLCHANCE      80
 #define GAME_MIN_DIPLCHANCE          1
@@ -305,10 +367,26 @@ extern struct civ_game game;
 #define GAME_MIN_CITYFACTOR          6
 #define GAME_MAX_CITYFACTOR          100
 
+#define GAME_DEFAULT_CITYMINDIST     0
+#define GAME_MIN_CITYMINDIST         0 /* if 0, ruleset will overwrite this */
+#define GAME_MAX_CITYMINDIST         5
+
 #define GAME_DEFAULT_CIVILWARSIZE    10
 #define GAME_MIN_CIVILWARSIZE        6
 #define GAME_MAX_CIVILWARSIZE        1000
 
+#define GAME_DEFAULT_CONTACTTURNS    20
+#define GAME_MIN_CONTACTTURNS        0
+#define GAME_MAX_CONTACTTURNS        100
+
+#define GAME_DEFAULT_RAPTUREDELAY    1
+#define GAME_MIN_RAPTUREDELAY        1
+#define GAME_MAX_RAPTUREDELAY        99 /* 99 practicaly disables rapturing */
+ 
+#define GAME_DEFAULT_SAVEPALACE      TRUE
+
+#define GAME_DEFAULT_NATURALCITYNAMES TRUE
+
 #define GAME_DEFAULT_FOODBOX         10
 #define GAME_MIN_FOODBOX             5
 #define GAME_MAX_FOODBOX             30
@@ -333,20 +411,25 @@ extern struct civ_game game;
 #define GAME_MIN_CIVSTYLE            1
 #define GAME_MAX_CIVSTYLE            2
 
-#define GAME_DEFAULT_SCORELOG        0
-#define GAME_MIN_SCORELOG            0
-#define GAME_MAX_SCORELOG            1
+#define GAME_DEFAULT_SCORELOG        FALSE
 
-#define GAME_DEFAULT_SPACERACE       1
-#define GAME_MIN_SPACERACE           0
-#define GAME_MAX_SPACERACE           1
+#define GAME_DEFAULT_SPACERACE       TRUE
 
-#define GAME_DEFAULT_AUTO_AI_TOGGLE  0
-#define GAME_MIN_AUTO_AI_TOGGLE      0
-#define GAME_MAX_AUTO_AI_TOGGLE      1
+#define GAME_DEFAULT_TURNBLOCK       TRUE
+
+#define GAME_DEFAULT_AUTO_AI_TOGGLE  FALSE
 
 #define GAME_DEFAULT_TIMEOUT         0
+#define GAME_DEFAULT_TIMEOUTINT      0
+#define GAME_DEFAULT_TIMEOUTINTINC   0
+#define GAME_DEFAULT_TIMEOUTINC      0
+#define GAME_DEFAULT_TIMEOUTINCMULT  1
+
+#ifndef NDEBUG
+#define GAME_MIN_TIMEOUT             -1
+#else
 #define GAME_MIN_TIMEOUT             0
+#endif
 #define GAME_MAX_TIMEOUT             8639999
 
 #define GAME_DEFAULT_TCPTIMEOUT      10
@@ -357,10 +440,22 @@ extern struct civ_game game;
 #define GAME_MIN_NETWAIT             0
 #define GAME_MAX_NETWAIT             20
 
+#define GAME_DEFAULT_PINGTIME        20
+#define GAME_MIN_PINGTIME            1
+#define GAME_MAX_PINGTIME            1800
+
 #define GAME_DEFAULT_PINGTIMEOUT     60
 #define GAME_MIN_PINGTIMEOUT         60
 #define GAME_MAX_PINGTIMEOUT         1800
 
+#define GAME_DEFAULT_NOTRADESIZE     0
+#define GAME_MIN_NOTRADESIZE         0
+#define GAME_MAX_NOTRADESIZE         49
+
+#define GAME_DEFAULT_FULLTRADESIZE   1
+#define GAME_MIN_FULLTRADESIZE       1
+#define GAME_MAX_FULLTRADESIZE       50
+
 #define GAME_DEFAULT_BARBARIANRATE   2
 #define GAME_MIN_BARBARIANRATE       0
 #define GAME_MAX_BARBARIANRATE       4
@@ -374,14 +469,15 @@ extern struct civ_game game;
 #define GAME_MIN_OCCUPYCHANCE        0
 #define GAME_MAX_OCCUPYCHANCE        100
 
+#define GAME_DEFAULT_RULESETDIR      "default"
+
 #define GAME_DEFAULT_SAVE_NAME       "civgame"
-#define GAME_DEFAULT_RULESET         "default"
 
 #define GAME_DEFAULT_SKILL_LEVEL 3      /* easy */
 #define GAME_OLD_DEFAULT_SKILL_LEVEL 5  /* normal; for old save games */
 
 #define GAME_DEFAULT_DEMOGRAPHY      "NASRLPEMOqrb"
-#define GAME_DEFAULT_ALLOW_CONNECT   "NHAhad"
+#define GAME_DEFAULT_ALLOW_TAKE      "HAhadOo"
 
 #define GAME_DEFAULT_COMPRESS_LEVEL 6    /* if we have compression */
 #define GAME_MIN_COMPRESS_LEVEL     0
@@ -392,6 +488,32 @@ extern struct civ_game game;
 #define GAME_MAX_REPUTATION 1000
 #define GAME_REPUTATION_INCR 2
 
+#define GAME_DEFAULT_WATCHTOWER_VISION 2
+#define GAME_MIN_WATCHTOWER_VISION 1
+#define GAME_MAX_WATCHTOWER_VISION 3
+
+#define GAME_DEFAULT_WATCHTOWER_EXTRA_VISION 0
+#define GAME_MIN_WATCHTOWER_EXTRA_VISION 0
+#define GAME_MAX_WATCHTOWER_EXTRA_VISION 2
+
+#define GAME_DEFAULT_ALLOWED_CITY_NAMES 1
+#define GAME_MIN_ALLOWED_CITY_NAMES 0
+#define GAME_MAX_ALLOWED_CITY_NAMES 3
+
+#define GAME_DEFAULT_REVOLUTION_LENGTH 0
+#define GAME_MIN_REVOLUTION_LENGTH 0
+#define GAME_MAX_REVOLUTION_LENGTH 10
+
 #define GAME_START_YEAR -4000
 
+#define specialist_type_iterate(sp)					    \
+{									    \
+  int sp;                                                                   \
+                                                                            \
+  for (sp = 0; sp < SP_COUNT; sp++) {
+
+#define specialist_type_iterate_end                                         \
+  }                                                                         \
+}
+
 #endif  /* FC__GAME_H */
