@@ -15,8 +15,8 @@
 
 #include "shared.h"
 
-struct city;
-struct player;
+#include "fc_types.h"
+
 struct Sprite;			/* opaque; client-gui specific */
 
 #define G_MAGIC (127)		/* magic constant, used as flag value */
@@ -31,6 +31,12 @@ enum government_flag_id {
   G_UNBRIBABLE,
   G_INSPIRES_PARTISANS,
   G_RAPTURE_CITY_GROWTH,        /* allows city to grow by celebrating */
+  G_FANATIC_TROOPS,             /* for building troops with F_FANATIC flag */
+  G_NO_UNHAPPY_CITIZENS,        /* no unhappy citizen, needed by
+				   fundamentism */
+  G_CONVERT_TITHES_TO_MONEY,    /* tithes to money, needed by fundamentalism */
+  G_REDUCED_RESEARCH,           /* penalty for research, needed by
+				   fundamentalism */
   G_LAST_FLAG
 };
 #define G_FIRST_FLAG G_BUILD_VETERAN_DIPLOMAT
@@ -50,8 +56,8 @@ enum government_hint_id {
 struct ruler_title
 {
   int  nation;
-  char male_title[MAX_LEN_NAME];
-  char female_title[MAX_LEN_NAME];
+  const char *male_title; /* Translated string - doesn't need freeing. */
+  const char *female_title; /* Translated string - doesn't need freeing. */
   
   /* untranslated copies: */
   char male_title_orig[MAX_LEN_NAME];    
@@ -64,12 +70,11 @@ struct ruler_title
 struct government
 {
   int   index;			/* index into governments[] array */
-  char  name[MAX_LEN_NAME];	/* government name */
+  const char *name; /* Translated string - doesn't need freeing. */
   char  name_orig[MAX_LEN_NAME]; /* untranslated copy */
   char  graphic_str[MAX_LEN_NAME];
   char  graphic_alt[MAX_LEN_NAME];
   int   required_tech;		/* tech required to change to this gov */
-  int   subgoal;		/* for AI; another government or -1 */
 
   struct ruler_title *ruler_titles;
   int   num_ruler_titles;
@@ -128,13 +133,19 @@ struct government
   int   fixed_corruption_distance;
   int   corruption_distance_factor;
   int   extra_corruption_distance;
+  int   corruption_max_distance_cap;
   
+  /* waste modifiers, see governments.ruleset for more detail */
+  int   waste_level;
+  int   waste_modifier;
+  int   fixed_waste_distance;
+  int   waste_distance_factor;
+  int   extra_waste_distance;
+  int   waste_max_distance_cap;
+    
   /* other flags: bits in enum government_flag_id order,
      use government_has_flag() to access */
   int   flags;
-  /* other hints: bits in enum government_hint_id order,
-     use government_has_hint() to access */
-  int   hints;
 
   struct Sprite *sprite;
   
@@ -142,14 +153,13 @@ struct government
 };
 
 /* This should possibly disappear; we don't bother sending these to client;
-   see code in ai_city.c: ai_manage_cities() for what they mean...
-*/
+ * See code in aitech.c for what the fields mean */
 struct ai_gov_tech_hint {
   int tech;
   int turns_factor;
   int const_factor;
-  int get_first;
-  int done;
+  bool get_first;
+  bool done;
 };
 
 extern struct government *governments;
@@ -159,27 +169,39 @@ extern struct ai_gov_tech_hint ai_gov_tech_hints[MAX_NUM_TECH_LIST];
    and techs before that are guaranteed to exist */
 
 struct government *get_government(int gov);
-struct government *get_gov_iplayer(int player_num);
 struct government *get_gov_pplayer(struct player *pplayer);
-struct government *get_gov_pcity(struct city *pcity);
+struct government *get_gov_pcity(const struct city *pcity);
 
-struct government *find_government_by_name(char *name);
+struct government *find_government_by_name(const char *name);
+struct government *find_government_by_name_orig(const char *name);
 
 enum government_flag_id government_flag_from_str(const char *s);
-int government_has_flag(const struct government *gov,
+bool government_has_flag(const struct government *gov,
 			enum government_flag_id flag);
-enum government_hint_id government_hint_from_str(const char *s);
-int government_has_hint(const struct government *gov,
-			enum government_hint_id hint);
 
 int get_government_max_rate(int type);
 int get_government_civil_war_prob(int type);
-char *get_government_name(int type);
-char *get_ruler_title(int gov, int male, int nation);
+const char *get_government_name(int type);
+const char *get_ruler_title(int gov, bool male, int nation);
 
-int can_change_to_government(struct player *pplayer, int government);
+bool can_change_to_government(struct player *pplayer, int government);
 
 void set_ruler_title(struct government *gov, int nation, 
-                     char *male, char *female);
+                     const char *male, const char *female);
+void governments_alloc(int num);
+void governments_free(void);
+
+#define government_iterate(gov)                                             \
+{                                                                           \
+  int GI_index;                                                             \
+                                                                            \
+  for (GI_index = 0; GI_index < game.government_count; GI_index++) {        \
+    struct government *gov = get_government(GI_index);                      \
+    {
+
+#define government_iterate_end                                              \
+    }                                                                       \
+  }                                                                         \
+}
 
 #endif  /* FC__GOVERNMENT_H */
