@@ -13,17 +13,24 @@
 #ifndef FC__DIPLHAND_H
 #define FC__DIPLHAND_H
 
-#include "packets.h"
+#define REPUTATION_LOSS_NUKE (GAME_MAX_REPUTATION * 0.03)
 
-void handle_diplomacy_cancel_meeting(struct player *pplayer, 
-				     struct packet_diplomacy_info *packet);
-void handle_diplomacy_create_clause(struct player *pplayer, 
-				    struct packet_diplomacy_info *packet);
-void handle_diplomacy_remove_clause(struct player *pplayer, 
-				    struct packet_diplomacy_info *packet);
-void handle_diplomacy_init(struct player *pplayer, 
-			   struct packet_diplomacy_info *packet);
-void handle_diplomacy_accept_treaty(struct player *pplayer, 
-				    struct packet_diplomacy_info *packet);
+#include "fc_types.h"
+
+#include "hand_gen.h"
+
+struct Treaty;
+struct packet_diplomacy_info;
+struct connection;
+
+void establish_embassy(struct player *pplayer, struct player *aplayer);
+
+void diplhand_init(void);
+void diplhand_free(void);
+
+struct Treaty *find_treaty(struct player *plr0, struct player *plr1);
+
+void send_diplomatic_meetings(struct connection *dest);
+void cancel_all_meetings(struct player *pplayer);
 
 #endif  /* FC__DIPLHAND_H */
