@@ -14,6 +14,7 @@
 #define FC__GAMEHAND_H
 
 struct section_file;
+struct connection;
 struct conn_list;
 
 void init_new_game(void);
@@ -24,4 +25,12 @@ void send_start_turn_to_clients(void);
 
 int update_timeout(void);
 
+const char *new_challenge_filename(struct connection *pc);
+
+struct packet_single_want_hack_req;
+
+void handle_single_want_hack_req(struct connection *pc,
+    				 const struct packet_single_want_hack_req
+				 *packet);
+
 #endif  /* FC__GAMEHAND_H */
