@@ -13,23 +13,30 @@
 #ifndef FC__MAPHAND_H
 #define FC__MAPHAND_H
 
+#include "fc_types.h"
 #include "map.h"
 #include "packets.h"
+#include "terrain.h"
+
+#include "hand_gen.h"
+
+enum ocean_land_change { OLC_NONE, OLC_OCEAN_TO_LAND, OLC_LAND_TO_OCEAN };
 
-struct player;
 struct section_file;
 struct conn_list;
 
 struct dumb_city{
   int id;
+  bool has_walls;
+  bool occupied;
+  bool happy, unhappy;
   char name[MAX_LEN_NAME];
   unsigned short size;
-  unsigned char has_walls;
   unsigned char owner;
 };
 
-struct player_tile{
-  enum tile_terrain_type terrain;
+struct player_tile {
+  Terrain_type_id terrain;
   enum tile_special_type special;
   unsigned short seen;
   unsigned short own_seen;
@@ -42,6 +49,11 @@ struct player_tile{
   short last_updated;
 };
 
+/* The maximum number of continents and oceans. */
+#define MAP_NCONT 600
+
+void assign_continent_numbers(bool skip_unsafe);
+
 void global_warming(int effect);
 void nuclear_winter(int effect);
 void give_map_from_player_to_player(struct player *pfrom, struct player *pdest);
@@ -49,38 +61,51 @@ void give_seamap_from_player_to_player(struct player *pfrom, struct player *pdes
 void give_citymap_from_player_to_player(struct city *pcity,
 					struct player *pfrom, struct player *pdest);
 void send_all_known_tiles(struct conn_list *dest);
-void send_tile_info(struct conn_list *dest, int x, int y);
-void unfog_area(struct player *pplayer, int x, int y, int len);
-void fog_area(struct player *pplayer, int x, int y, int len);
-void upgrade_city_rails(struct player *pplayer, int discovery);
+void send_tile_info(struct conn_list *dest, struct tile *ptile);
+void reveal_hidden_units(struct player *pplayer, struct tile *ptile);
+void conceal_hidden_units(struct player *pplayer, struct tile *ptile);
+void unfog_area(struct player *pplayer, struct tile *ptile, int len);
+void fog_area(struct player *pplayer, struct tile *ptile, int len);
+void upgrade_city_rails(struct player *pplayer, bool discovery);
 void send_map_info(struct conn_list *dest);
 void map_fog_city_area(struct city *pcity);
 void map_unfog_city_area(struct city *pcity);
 void remove_unit_sight_points(struct unit *punit);
-void show_area(struct player *pplayer,int x, int y, int len);
-void map_unfog_pseudo_city_area(struct player *pplayer, int x,int y);
-void map_fog_pseudo_city_area(struct player *pplayer, int x,int y);
+void show_area(struct player *pplayer,struct tile *ptile, int len);
+void map_unfog_pseudo_city_area(struct player *pplayer, struct tile *ptile);
+void map_fog_pseudo_city_area(struct player *pplayer, struct tile *ptile);
 
-int map_get_known_and_seen(int x, int y, int playerid);
-void map_change_seen(int x, int y, int playerid, int change);
-int map_get_own_seen(int x, int y, int playerid);
-void map_change_own_seen(int x, int y, int playerid, int change);
-int map_get_known(int x, int y, struct player *pplayer);
-void map_set_known(int x, int y, struct player *pplayer);
-void map_clear_known(int x, int y, struct player *pplayer);
+bool map_is_known_and_seen(const struct tile *ptile, struct player *pplayer);
+void map_change_seen(struct tile *ptile, struct player *pplayer, int change);
+bool map_is_known(const struct tile *ptile, struct player *pplayer);
+void map_set_known(struct tile *ptile, struct player *pplayer);
+void map_clear_known(struct tile *ptile, struct player *pplayer);
 void map_know_all(struct player *pplayer);
 void map_know_and_see_all(struct player *pplayer);
 void show_map_to_all(void);
+
 void player_map_allocate(struct player *pplayer);
-struct player_tile *map_get_player_tile(int x, int y, int playerid);
-void update_tile_knowledge(struct player *pplayer,int x, int y);
-void update_player_tile_last_seen(struct player *pplayer, int x, int y);
+void player_map_free(struct player *pplayer);
+struct player_tile *map_get_player_tile(const struct tile *ptile,
+					struct player *pplayer);
+bool update_player_tile_knowledge(struct player *pplayer,struct tile *ptile);
+void update_tile_knowledge(struct tile *ptile);
+void update_player_tile_last_seen(struct player *pplayer, struct tile *ptile);
 
 void give_shared_vision(struct player *pfrom, struct player *pto);
 void remove_shared_vision(struct player *pfrom, struct player *pto);
-void handle_player_remove_vision(struct player *pplayer,
-				 struct packet_generic_integer *packet);
 
 void enable_fog_of_war(void);
 void disable_fog_of_war(void);
+
+void map_update_borders_city_destroyed(struct tile *ptile);
+void map_update_borders_city_change(struct city *pcity);
+void map_update_borders_landmass_change(struct tile *ptile);
+void map_calculate_borders(void);
+
+enum ocean_land_change check_terrain_ocean_land_change(struct tile *ptile,
+                                              Terrain_type_id oldter);
+int get_continent_size(Continent_id id);
+int get_ocean_size(Continent_id id);
+
 #endif  /* FC__MAPHAND_H */
