@@ -10,69 +10,74 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <assert.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <errno.h>
-#include <assert.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_TIME_H
-#include <sys/time.h>
-#endif
-#ifdef HAVE_PWD_H
-#include <pwd.h>
-#endif
-
-#ifdef HAVE_SYS_SELECT_H
-#include <sys/select.h>
-#endif
-
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
 #endif
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
-#ifdef HAVE_SYS_UIO_H
-#include <sys/uio.h>
-#endif
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+#ifdef HAVE_LIBREADLINE
+#include <readline/history.h>
+#include <readline/readline.h>
+#endif
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
 #endif
 #ifdef HAVE_WINSOCK
 #include <winsock.h>
 #endif
 
-#ifdef HAVE_LIBREADLINE
-#include <readline/readline.h>
-#include <readline/history.h>
-#endif
+#include "fciconv.h"
 
+#include "capability.h"
+#include "dataio.h"
+#include "events.h"
 #include "fcintl.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
 #include "packets.h"
-#include "plrhand.h"
 #include "shared.h"
 #include "support.h"
-#include "capability.h"
+#include "timing.h"
+
+#include "connecthand.h"
 #include "console.h"
 #include "meta.h"
+#include "plrhand.h"
 #include "srv_main.h"
 #include "stdinhand.h"
 
@@ -85,6 +90,7 @@ TEndpointInfo serv_info;
 EndpointRef serv_ep;
 #else
 static int sock;
+static int socklan;
 #endif
 
 #if defined(__VMS)
@@ -105,25 +111,38 @@ static int sock;
    void user_interrupt_callback();
 #endif
 
+#define SPECLIST_TAG timer
+#define SPECLIST_TYPE struct timer
+#include "speclist.h"
+
 #define PROCESSING_TIME_STATISTICS 0
 
 static int server_accept_connection(int sockfd);
 static void start_processing_request(struct connection *pconn,
 				     int request_id);
 static void finish_processing_request(struct connection *pconn);
+static void ping_connection(struct connection *pconn);
+static void send_ping_times_to_all(void);
+
+static void get_lanserver_announcement(void);
+static void send_lanserver_response(void);
 
 static bool no_input = FALSE;
 
 /*****************************************************************************
   This happens if you type an EOF character with nothing on the current line.
 *****************************************************************************/
-#ifndef SOCKET_ZERO_ISNT_STDIN
 static void handle_stdin_close(void)
 {
+  /* Note this function may be called even if SOCKET_ZERO_ISNT_STDIN, so
+   * the preprocessor check has to come inside the function body.  But
+   * perhaps we want to do this even when SOCKET_ZERO_ISNT_STDIN? */
+#ifndef SOCKET_ZERO_ISNT_STDIN
   freelog(LOG_NORMAL, _("Server cannot read standard input. Ignoring input."));
   no_input = TRUE;
-}
 #endif
+}
+
 #ifdef HAVE_LIBREADLINE
 /****************************************************************************/
 
@@ -141,6 +160,8 @@ static bool readline_initialized = FALSE;
 *****************************************************************************/
 static void handle_readline_input_callback(char *line)
 {
+  char *line_internal;
+
   if (no_input)
     return;
 
@@ -153,7 +174,9 @@ static void handle_readline_input_callback(char *line)
     add_history(line);
 
   con_prompt_enter();		/* just got an 'Enter' hit */
-  handle_stdin_input((struct connection*)NULL, line);
+  line_internal = local_to_internal_string_malloc(line);
+  (void) handle_stdin_input(NULL, line_internal, FALSE);
+  free(line_internal);
 
   readline_handled_input = TRUE;
 }
@@ -164,20 +187,23 @@ static void handle_readline_input_callback(char *line)
 *****************************************************************************/
 void close_connection(struct connection *pconn)
 {
+  while (timer_list_size(pconn->server.ping_timers) > 0) {
+    struct timer *timer = timer_list_get(pconn->server.ping_timers, 0);
+
+    timer_list_unlink(pconn->server.ping_timers, timer);
+    free_timer(timer);
+  }
+  assert(timer_list_size(pconn->server.ping_timers) == 0);
+  timer_list_unlink_all(pconn->server.ping_timers);
+
   /* safe to do these even if not in lists: */
   conn_list_unlink(&game.all_connections, pconn);
   conn_list_unlink(&game.est_connections, pconn);
   conn_list_unlink(&game.game_connections, pconn);
 
-  my_closesocket(pconn->sock);
-  pconn->used = FALSE;
-  pconn->established = FALSE;
   pconn->player = NULL;
   pconn->access_level = ALLOW_NONE;
-  free_socket_packet_buffer(pconn->buffer);
-  free_socket_packet_buffer(pconn->send_buffer);
-  pconn->buffer = NULL;
-  pconn->send_buffer = NULL;
+  connection_common_close(pconn);
 }
 
 /*****************************************************************************
@@ -186,31 +212,35 @@ void close_connection(struct connection *pconn)
 void close_connections_and_socket(void)
 {
   int i;
-
-  struct packet_generic_message gen_packet;
-  gen_packet.message[0]='\0';
-
-  lsend_packet_generic_message(&game.all_connections, PACKET_SERVER_SHUTDOWN,
-			       &gen_packet);
+  lsend_packet_server_shutdown(&game.all_connections);
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     if(connections[i].used) {
       close_connection(&connections[i]);
     }
+    conn_list_unlink_all(&connections[i].self);
   }
+
+  /* Remove the game connection lists and make sure they are empty. */
+  assert(conn_list_size(&game.all_connections) == 0);
+  conn_list_unlink_all(&game.all_connections);
+  assert(conn_list_size(&game.est_connections) == 0);
+  conn_list_unlink_all(&game.est_connections);
+  assert(conn_list_size(&game.game_connections) == 0);
+  conn_list_unlink_all(&game.game_connections);
+
   my_closesocket(sock);
+  my_closesocket(socklan);
 
 #ifdef HAVE_LIBREADLINE
   if (history_file) {
     write_history(history_file);
     history_truncate_file(history_file, HISTORY_LENGTH);
   }
-  if (readline_initialized) {
-    rl_callback_handler_remove();
-  }
 #endif
 
-  server_close_udp();
+  send_server_info_to_metaserver(META_GOODBYE);
+  server_close_meta();
 
   my_shutdown_network();
 }
@@ -337,7 +367,7 @@ int sniff_packets(void)
       }
 
       rl_initialize();
-      rl_callback_handler_install("> ", handle_readline_input_callback);
+      rl_callback_handler_install((char *) "> ", handle_readline_input_callback);
       rl_attempted_completion_function = freeciv_completion;
 
       readline_initialized = TRUE;
@@ -349,8 +379,10 @@ int sniff_packets(void)
   if(year!=game.year) {
     if (server_state == RUN_GAME_STATE) year=game.year;
   }
-  if (game.timeout == 0)
+  if (game.timeout == 0) {
+    /* Just in case someone sets timeout we keep game.turn_start updated */
     game.turn_start = time(NULL);
+  }
   
   while(TRUE) {
     con_prompt_on();		/* accepting new input */
@@ -361,63 +393,80 @@ int sniff_packets(void)
       return 2;
     }
 
+    get_lanserver_announcement();
 
-    /* quit server if no players for 'srvarg.quitidle' seconds */
+    /* end server if no players for 'srvarg.quitidle' seconds */
     if (srvarg.quitidle != 0 && server_state != PRE_GAME_STATE) {
       static time_t last_noplayers;
       if(conn_list_size(&game.est_connections) == 0) {
 	if (last_noplayers != 0) {
 	  if (time(NULL)>last_noplayers + srvarg.quitidle) {
-	    sz_strlcpy(srvarg.metaserver_info_line,
-		       "restarting for lack of players");
-	    freelog(LOG_NORMAL, srvarg.metaserver_info_line);
-	    (void) send_server_info_to_metaserver(TRUE, FALSE);
+	    set_meta_message_string("restarting for lack of players");
+	    freelog(LOG_NORMAL, get_meta_message_string());
+	    (void) send_server_info_to_metaserver(META_INFO);
 
-	    close_connections_and_socket();
-	    exit(EXIT_SUCCESS);
+            server_state = GAME_OVER_STATE;
+            force_end_of_sniff = TRUE;
+            conn_list_iterate(game.est_connections, pconn) {
+              lost_connection_to_client(pconn);
+              close_connection(pconn);
+            } conn_list_iterate_end;
 	  }
 	} else {
+          char buf[256];
 	  last_noplayers = time(NULL);
 	  
-	  my_snprintf(srvarg.metaserver_info_line,
-		      sizeof(srvarg.metaserver_info_line),
+	  my_snprintf(buf, sizeof(buf),
 		      "restarting in %d seconds for lack of players",
 		      srvarg.quitidle);
-	  freelog(LOG_NORMAL, srvarg.metaserver_info_line);
-	  (void) send_server_info_to_metaserver(TRUE, FALSE);
+          set_meta_message_string((const char *)buf);
+	  freelog(LOG_NORMAL, get_meta_message_string());
+	  (void) send_server_info_to_metaserver(META_INFO);
 	}
       } else {
         last_noplayers = 0;
       }
     }
 
-    /* send PACKET_CONN_PING & cut mute players */
-    if ((time(NULL)>game.last_ping + game.pingtimeout)) {
-      for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
-	struct connection *pconn = &connections[i];
-	if (pconn->used) {
-	  send_packet_generic_empty(pconn, PACKET_CONN_PING);
+    /* Pinging around for statistics */
+    if (time(NULL) > (game.last_ping + game.pingtime)) {
+      /* send data about the previous run */
+      send_ping_times_to_all();
 
-	  if (pconn->ponged) {
-	    pconn->ponged = FALSE;
+      conn_list_iterate(game.all_connections, pconn) {
+	if ((timer_list_size(pconn->server.ping_timers) > 0
+	     &&
+	     read_timer_seconds(timer_list_get(pconn->server.ping_timers, 0))
+	     > game.pingtimeout) || pconn->ping_time > game.pingtimeout) {
+	  /* cut mute players, except for hack-level ones */
+	  if (pconn->access_level == ALLOW_HACK) {
+	    freelog(LOG_NORMAL,
+		    "ignoring ping timeout to hack-level connection %s",
+		    conn_description(pconn));
 	  } else {
 	    freelog(LOG_NORMAL, "cut connection %s due to ping timeout",
 		    conn_description(pconn));
 	    close_socket_callback(pconn);
 	  }
+	} else {
+	  ping_connection(pconn);
 	}
-      }
+      } conn_list_iterate_end;
       game.last_ping = time(NULL);
     }
 
+    /* if we've waited long enough after a failure, respond to the client */
+    conn_list_iterate(game.all_connections, pconn) {
+      if (pconn->server.status == AS_FAILED
+          && pconn->server.authentication_stop > 0
+          && time(NULL) >= pconn->server.authentication_stop) {
+        unfail_authentication(pconn);
+      }
+    } conn_list_iterate_end
+
     /* Don't wait if timeout == -1 (i.e. on auto games) */
     if (server_state != PRE_GAME_STATE && game.timeout == -1) {
-      (void) send_server_info_to_metaserver(FALSE, FALSE);
-
-      /* kick out of the srv_main loop */
-      if (server_state == GAME_OVER_STATE) {
-	server_state = PRE_GAME_STATE;
-      }
+      (void) send_server_info_to_metaserver(META_REFRESH);
       return 0;
     }
 
@@ -455,7 +504,7 @@ int sniff_packets(void)
     con_prompt_off();		/* output doesn't generate a new prompt */
 
     if(select(max_desc+1, &readfs, &writefs, &exceptfs, &tv)==0) { /* timeout */
-      (void) send_server_info_to_metaserver(FALSE, FALSE);
+      (void) send_server_info_to_metaserver(META_REFRESH);
       if(game.timeout != 0
 	&& (time(NULL)>game.turn_start + game.timeout)
 	&& (server_state == RUN_GAME_STATE)){
@@ -483,8 +532,10 @@ int sniff_packets(void)
 #endif /* !__VMS */
       }
     }
-    if (game.timeout == 0)
+    if (game.timeout == 0) {
+      /* Just in case someone sets timeout we keep game.turn_start updated */
       game.turn_start = time(NULL);
+    }
 
     if(FD_ISSET(sock, &exceptfs)) {	     /* handle Ctrl-Z suspend/resume */
       continue;
@@ -506,8 +557,11 @@ int sniff_packets(void)
     
 #ifdef SOCKET_ZERO_ISNT_STDIN
     if (!no_input && (bufptr = my_read_console())) {
+      char *bufptr_internal = local_to_internal_string_malloc(bufptr);
+
       con_prompt_enter();	/* will need a new prompt, regardless */
-      handle_stdin_input((struct connection *)NULL, bufptr);
+      handle_stdin_input(NULL, bufptr_internal, FALSE);
+      free(bufptr_internal);
     }
 #else  /* !SOCKET_ZERO_ISNT_STDIN */
     if(!no_input && FD_ISSET(0, &readfs)) {    /* input from server operator */
@@ -521,10 +575,10 @@ int sniff_packets(void)
 #else  /* !HAVE_LIBREADLINE */
       ssize_t didget;
       char buf[BUF_SIZE+1];
+      char *buf_internal;
       
       if((didget=read(0, buf, BUF_SIZE))==-1) {
-	freelog(LOG_FATAL, "read from stdin failed");
-	exit(EXIT_FAILURE);
+	die("read from stdin failed");
       }
 
       if(didget==0) {
@@ -533,7 +587,9 @@ int sniff_packets(void)
 
       *(buf+didget)='\0';
       con_prompt_enter();	/* will need a new prompt, regardless */
-      handle_stdin_input((struct connection *)NULL, buf);
+      buf_internal = local_to_internal_string_malloc(buf);
+      handle_stdin_input(NULL, buf_internal, FALSE);
+      free(buf_internal);
 #endif /* !HAVE_LIBREADLINE */
     }
     else
@@ -571,7 +627,10 @@ int sniff_packets(void)
 					 pconn->server.
 					 last_request_id_seen);
 		command_ok = handle_packet_input(pconn, packet, type);
-		packet = NULL;
+		if (packet) {
+		  free(packet);
+		  packet = NULL;
+		}
 
 		finish_processing_request(pconn);
 		connection_do_unbuffer(pconn);
@@ -646,7 +705,7 @@ static const char *makeup_connection_name(int *id)
     if (!find_player_by_name(name)
 	&& !find_player_by_user(name)
 	&& !find_conn_by_id(i)
-	&& !find_conn_by_name(name)) {
+	&& !find_conn_by_user(name)) {
       *id = i;
       return name;
     }
@@ -669,52 +728,58 @@ static int server_accept_connection(int sockfd)
 # endif
 
   int new_sock;
-  struct sockaddr_in fromend;
+  union my_sockaddr fromend;
   struct hostent *from;
   int i;
 
   fromlen = sizeof(fromend);
 
-  if ((new_sock=accept(sockfd, (struct sockaddr *) &fromend, &fromlen)) == -1) {
-    freelog(LOG_ERROR, "accept failed: %s", mystrerror(errno));
+  if ((new_sock = accept(sockfd, &fromend.sockaddr, &fromlen)) == -1) {
+    freelog(LOG_ERROR, "accept failed: %s", mystrerror());
     return -1;
   }
 
   my_nonblock(new_sock);
 
-  from=gethostbyaddr((char *)&fromend.sin_addr, sizeof(fromend.sin_addr),
-		     AF_INET);
+  from =
+      gethostbyaddr((char *) &fromend.sockaddr_in.sin_addr,
+		    sizeof(fromend.sockaddr_in.sin_addr), AF_INET);
 
   for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
     struct connection *pconn = &connections[i];
     if (!pconn->used) {
-      pconn->used = TRUE;
+      connection_common_init(pconn);
       pconn->sock = new_sock;
-      pconn->established = FALSE;
       pconn->observer = FALSE;
       pconn->player = NULL;
-      pconn->buffer = new_socket_packet_buffer();
-      pconn->send_buffer = new_socket_packet_buffer();
-      pconn->last_write = 0;
-      pconn->ponged = TRUE;
-      pconn->first_packet = TRUE;
-      pconn->byte_swap = FALSE;
       pconn->capability[0] = '\0';
       pconn->access_level = access_level_for_next_connection();
       pconn->delayed_disconnect = FALSE;
       pconn->notify_of_writable_data = NULL;
       pconn->server.currently_processed_request_id = 0;
       pconn->server.last_request_id_seen = 0;
+      pconn->server.authentication_tries = 0;
+      pconn->server.authentication_stop = 0;
+      pconn->server.status = AS_NOT_ESTABLISHED;
+      pconn->server.ping_timers =
+	  fc_malloc(sizeof(*pconn->server.ping_timers));
+      timer_list_init(pconn->server.ping_timers);
+      pconn->ping_time = -1.0;
       pconn->incoming_packet_notify = NULL;
       pconn->outgoing_packet_notify = NULL;
 
-      sz_strlcpy(pconn->name, makeup_connection_name(&pconn->id));
+      sz_strlcpy(pconn->username, makeup_connection_name(&pconn->id));
       sz_strlcpy(pconn->addr,
-		 (from ? from->h_name : inet_ntoa(fromend.sin_addr)));
+		 (from ? from->
+		  h_name : inet_ntoa(fromend.sockaddr_in.sin_addr)));
+      sz_strlcpy(pconn->server.ipaddr,
+                 inet_ntoa(fromend.sockaddr_in.sin_addr));
 
       conn_list_insert_back(&game.all_connections, pconn);
   
-      freelog(LOG_VERBOSE, "connection (%s) from %s", pconn->name, pconn->addr);
+      freelog(LOG_VERBOSE, "connection (%s) from %s (%s)", 
+              pconn->username, pconn->addr, pconn->server.ipaddr);
+      ping_connection(pconn);
       return 0;
     }
   }
@@ -725,39 +790,75 @@ static int server_accept_connection(int sockfd)
 
 /********************************************************************
  open server socket to be used to accept client connections
+ and open a server socket for server LAN announcements.
 ********************************************************************/
 int server_open_socket(void)
 {
   /* setup socket address */
-  struct sockaddr_in src;
+  union my_sockaddr src;
+  union my_sockaddr addr;
+  struct ip_mreq mreq;
+  const char *group;
   int opt;
 
+  /* Create socket for client connections. */
   if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
-    freelog(LOG_FATAL, "socket failed: %s", mystrerror(errno));
-    exit(EXIT_FAILURE);
+    die("socket failed: %s", mystrerror());
   }
 
   opt=1; 
   if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
 		(char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror(errno));
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
   }
 
-  memset(&src, 0, sizeof(src));
-  src.sin_family = AF_INET;
-  src.sin_addr.s_addr = htonl(INADDR_ANY);
-  src.sin_port = htons(srvarg.port);
+  if (!net_lookup_service(srvarg.bind_addr, srvarg.port, &src)) {
+    freelog(LOG_ERROR, _("Server: bad address: [%s:%d]."),
+	    srvarg.bind_addr, srvarg.port);
+    exit(EXIT_FAILURE);
+  }
 
-  if(bind(sock, (struct sockaddr *) &src, sizeof (src)) == -1) {
-    freelog(LOG_FATAL, "bind failed: %s", mystrerror(errno));
+  if(bind(sock, &src.sockaddr, sizeof (src)) == -1) {
+    freelog(LOG_FATAL, "bind failed: %s", mystrerror());
     exit(EXIT_FAILURE);
   }
 
   if(listen(sock, MAX_NUM_CONNECTIONS) == -1) {
-    freelog(LOG_FATAL, "listen failed: %s", mystrerror(errno));
+    freelog(LOG_FATAL, "listen failed: %s", mystrerror());
     exit(EXIT_FAILURE);
   }
 
+  /* Create socket for server LAN announcements */
+  if ((socklan = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
+     freelog(LOG_ERROR, "socket failed: %s", mystrerror());
+  }
+
+  if (setsockopt(socklan, SOL_SOCKET, SO_REUSEADDR,
+                 (char *)&opt, sizeof(opt)) == -1) {
+    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", mystrerror());
+  }
+
+  my_nonblock(socklan);
+
+  group = get_multicast_group();
+
+  memset(&addr, 0, sizeof(addr));
+  addr.sockaddr_in.sin_family = AF_INET;
+  addr.sockaddr_in.sin_addr.s_addr = htonl(INADDR_ANY);
+  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT);
+
+  if (bind(socklan, &addr.sockaddr, sizeof(addr)) < 0) {
+    freelog(LOG_ERROR, "bind failed: %s", mystrerror());
+  }
+
+  mreq.imr_multiaddr.s_addr = inet_addr(group);
+  mreq.imr_interface.s_addr = htonl(INADDR_ANY);
+
+  if (setsockopt(socklan, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                 (const char*)&mreq, sizeof(mreq)) < 0) {
+    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+  }
+
   close_socket_set_callback(close_socket_callback);
   return 0;
 }
@@ -779,7 +880,6 @@ void init_connections(void)
     pconn->used = FALSE;
     conn_list_init(&pconn->self);
     conn_list_insert(&pconn->self, pconn);
-    pconn->route = NULL;
   }
 #if defined(__VMS)
   {
@@ -801,7 +901,7 @@ static void start_processing_request(struct connection *pconn,
   assert(pconn->server.currently_processed_request_id == 0);
   freelog(LOG_DEBUG, "start processing packet %d from connection %d",
 	  request_id, pconn->id);
-  send_packet_generic_empty(pconn, PACKET_PROCESSING_STARTED);
+  send_packet_processing_started(pconn);
   pconn->server.currently_processed_request_id = request_id;
 }
 
@@ -813,6 +913,213 @@ static void finish_processing_request(struct connection *pconn)
   assert(pconn->server.currently_processed_request_id);
   freelog(LOG_DEBUG, "finish processing packet %d from connection %d",
 	  pconn->server.currently_processed_request_id, pconn->id);
-  send_packet_generic_empty(pconn, PACKET_PROCESSING_FINISHED);
+  send_packet_processing_finished(pconn);
   pconn->server.currently_processed_request_id = 0;
 }
+
+/**************************************************************************
+...
+**************************************************************************/
+static void ping_connection(struct connection *pconn)
+{
+  freelog(LOG_DEBUG, "sending ping to %s (open=%d)",
+	  conn_description(pconn),
+	  timer_list_size(pconn->server.ping_timers));
+  timer_list_insert_back(pconn->server.ping_timers,
+			 new_timer_start(TIMER_USER, TIMER_ACTIVE));
+  send_packet_conn_ping(pconn);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void handle_conn_pong(struct connection *pconn)
+{
+  struct timer *timer;
+
+  if (timer_list_size(pconn->server.ping_timers) == 0) {
+    freelog(LOG_NORMAL, "got unexpected pong from %s",
+	    conn_description(pconn));
+    return;
+  }
+
+  timer = timer_list_get(pconn->server.ping_timers, 0);
+  timer_list_unlink(pconn->server.ping_timers, timer);
+  pconn->ping_time = read_timer_seconds_free(timer);
+  freelog(LOG_DEBUG, "got pong from %s (open=%d); ping time = %fs",
+	  conn_description(pconn),
+	  timer_list_size(pconn->server.ping_timers), pconn->ping_time);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void send_ping_times_to_all(void)
+{
+  struct packet_conn_ping_info packet;
+  int i;
+
+  i = 0;
+  conn_list_iterate(game.game_connections, pconn) {
+    if (!pconn->used) {
+      continue;
+    }
+    i++;
+  } conn_list_iterate_end;
+
+  packet.connections = i;
+  packet.old_connections = MIN(i, MAX_NUM_PLAYERS);
+
+  i = 0;
+  conn_list_iterate(game.game_connections, pconn) {
+    if (!pconn->used) {
+      continue;
+    }
+    assert(i < ARRAY_SIZE(packet.conn_id));
+    packet.conn_id[i] = pconn->id;
+    packet.ping_time[i] = pconn->ping_time;
+    if (i < packet.old_connections) {
+      packet.old_conn_id[i] = pconn->id;
+      packet.old_ping_time[i] = pconn->ping_time;
+    }
+    i++;
+  } conn_list_iterate_end;
+  lsend_packet_conn_ping_info(&game.est_connections, &packet);
+}
+
+/********************************************************************
+  Listen for UDP packets multicasted from clients requesting
+  announcement of servers on the LAN.
+********************************************************************/
+static void get_lanserver_announcement(void)
+{
+  char msgbuf[128];
+  struct data_in din;
+  int type;
+  fd_set readfs, exceptfs;
+  struct timeval tv;
+
+  FD_ZERO(&readfs);
+  FD_ZERO(&exceptfs);
+  FD_SET(socklan, &exceptfs);
+  FD_SET(socklan, &readfs);
+
+  tv.tv_sec = 0;
+  tv.tv_usec = 0;
+
+  while (select(socklan + 1, &readfs, NULL, &exceptfs, &tv) == -1) {
+    if (errno != EINTR) {
+      freelog(LOG_ERROR, "select failed: %s", mystrerror());
+      return;
+    }
+    /* EINTR can happen sometimes, especially when compiling with -pg.
+     * Generally we just want to run select again. */
+  }
+
+  if (FD_ISSET(socklan, &readfs)) {
+    if (0 < recvfrom(socklan, msgbuf, sizeof(msgbuf), 0, NULL, NULL)) {
+      dio_input_init(&din, msgbuf, 1);
+      dio_get_uint8(&din, &type);
+      if (type == SERVER_LAN_VERSION) {
+        freelog(LOG_DEBUG, "Received request for server LAN announcement.");
+        send_lanserver_response();
+      } else {
+        freelog(LOG_DEBUG,
+                "Received invalid request for server LAN announcement.");
+      }
+    }
+  }
+}
+
+/********************************************************************
+  This function broadcasts an UDP packet to clients with
+  that requests information about the server state.
+********************************************************************/
+static void send_lanserver_response(void)
+{
+  unsigned char buffer[MAX_LEN_PACKET];
+  unsigned char hostname[512];
+  unsigned char port[256];
+  unsigned char version[256];
+  unsigned char players[256];
+  unsigned char status[256];
+  struct data_out dout;
+  union my_sockaddr addr;
+  int socksend, setting = 1;
+  const char *group;
+  size_t size;
+  unsigned char ttl;
+
+  /* Create a socket to broadcast to client. */
+  if ((socksend = socket(AF_INET,SOCK_DGRAM, 0)) < 0) {
+    freelog(LOG_ERROR, "socket failed: %s", mystrerror());
+    return;
+  }
+
+  /* Set the UDP Multicast group IP address of the packet. */
+  group = get_multicast_group();
+  memset(&addr, 0, sizeof(addr));
+  addr.sockaddr_in.sin_family = AF_INET;
+  addr.sockaddr_in.sin_addr.s_addr = inet_addr(group);
+  addr.sockaddr_in.sin_port = htons(SERVER_LAN_PORT + 1);
+
+  /* Set the Time-to-Live field for the packet.  */
+  ttl = SERVER_LAN_TTL;
+  if (setsockopt(socksend, IPPROTO_IP, IP_MULTICAST_TTL, 
+                 (const char*)&ttl, sizeof(ttl))) {
+    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    return;
+  }
+
+  if (setsockopt(socksend, SOL_SOCKET, SO_BROADCAST, 
+                 (const char*)&setting, sizeof(setting))) {
+    freelog(LOG_ERROR, "setsockopt failed: %s", mystrerror());
+    return;
+  }
+
+  /* Create a description of server state to send to clients.  */
+  if (my_gethostname(hostname, sizeof(hostname)) != 0) {
+    sz_strlcpy(hostname, "none");
+  }
+
+  my_snprintf(version, sizeof(version), "%d.%d.%d%s",
+              MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION, VERSION_LABEL);
+
+  switch (server_state) {
+  case PRE_GAME_STATE:
+    my_snprintf(status, sizeof(status), "Pregame");
+    break;
+  case RUN_GAME_STATE:
+    my_snprintf(status, sizeof(status), "Running");
+    break;
+  case GAME_OVER_STATE:
+    my_snprintf(status, sizeof(status), "Game over");
+    break;
+  default:
+    my_snprintf(status, sizeof(status), "Waiting");
+  }
+
+   my_snprintf(players, sizeof(players), "%d",
+               get_num_human_and_ai_players());
+   my_snprintf(port, sizeof(port), "%d",
+              srvarg.port );
+
+  dio_output_init(&dout, buffer, sizeof(buffer));
+  dio_put_uint8(&dout, SERVER_LAN_VERSION);
+  dio_put_string(&dout, hostname);
+  dio_put_string(&dout, port);
+  dio_put_string(&dout, version);
+  dio_put_string(&dout, status);
+  dio_put_string(&dout, players);
+  dio_put_string(&dout, get_meta_message_string());
+  size = dio_output_used(&dout);
+
+  /* Sending packet to client with the information gathered above. */
+  if (sendto(socksend, buffer,  size, 0, &addr.sockaddr,
+      sizeof(addr)) < 0) {
+    freelog(LOG_ERROR, "sendto failed: %s", mystrerror());
+    return;
+  }
+
+  my_closesocket(socksend);
+}
