@@ -13,32 +13,30 @@
 #ifndef FC__SETTLERS_H
 #define FC__SETTLERS_H
 
+#include "fc_types.h"
 #include "map.h"
 
-struct player;
-struct unit;
-struct city;
-
-void auto_settler_do_goto(struct player *pplayer, struct unit *punit, int x,
-			  int y);
-void auto_settlers(void);
-int find_boat(struct player *pplayer, int *x, int *y, int cap);
+void auto_settlers_init(void);
+void auto_settlers_player(struct player *pplayer);
+int find_boat(struct player *pplayer, struct tile **boat_tile, int cap);
 
 #define MORT 24
 
 int amortize(int benefit, int delay);
 void ai_manage_settler(struct player *pplayer, struct unit *punit);
 
-void generate_minimap(void);
-void remove_city_from_minimap(int x, int y);
-void add_city_to_minimap(int x, int y);
-void initialize_infrastructure_cache(struct city *pcity);
+void init_settlers(void);
+
+int city_tile_value(struct city *pcity, int x, int y,
+		    int foodneed, int prodneed);
+void initialize_infrastructure_cache(struct player *pplayer);
 
 void contemplate_terrain_improvements(struct city *pcity);
 void contemplate_new_city(struct city *pcity);
 
 struct unit *other_passengers(struct unit *punit);
 
-extern signed int minimap[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
+extern signed int *minimap;
+#define MINIMAP(ptile) minimap[(ptile)->index]
 
 #endif   /* FC__SETTLERS_H */
